@@ -1,4 +1,4 @@
-/* $Id: alglin1.c,v 1.92 2002/05/13 18:25:46 karim Exp $
+/* $Id: alglin1.c,v 1.100 2002/09/12 20:11:08 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -170,6 +170,19 @@ _vec(GEN x) { GEN v = cgetg(2, t_VEC); v[1] = (long)x; return v; }
 GEN
 _col(GEN x) { GEN v = cgetg(2, t_COL); v[1] = (long)x; return v; }
 
+/* routines for naive growarrays */
+GEN
+cget1(long l, long t)
+{
+  GEN z = new_chunk(l);
+  z[0] = evaltyp(t) | evallg(1); return z;
+}
+void
+appendL(GEN x, GEN t)
+{
+  long l = lg(x); x[l] = (long)t; setlg(x, l+1);
+}
+
 GEN
 concatsp(GEN x, GEN y)
 {
@@ -465,6 +478,26 @@ rowextract_i(GEN A, long x1, long x2)
   return B;
 }
 
+/* A[x0,] */
+GEN
+row(GEN A, long x0)
+{
+  long i, lB = lg(A);
+  GEN B  = cgetg(lB, t_VEC);
+  for (i=1; i<lB; i++) B[i] = coeff(A, x0, i);
+  return B;
+}
+
+/* A[x0, x1..x2] */
+GEN
+row_i(GEN A, long x0, long x1, long x2)
+{
+  long i, lB = x2 - x1 + 2;
+  GEN B  = cgetg(lB, t_VEC);
+  for (i=x1; i<=x2; i++) B[i] = coeff(A, x0, i);
+  return B;
+}
+
 GEN
 vecextract_p(GEN A, GEN p)
 {
@@ -604,6 +637,19 @@ extract0(GEN x, GEN l1, GEN l2)
   return matextract(x,l1,l2);
 }
 
+/* v[a] + ... + v[b] */
+GEN
+sum(GEN v, long a, long b)
+{
+  GEN p;
+  long i;
+  if (a > b) return gzero;
+  if (b > lg(v)-1) err(talker,"incorrect length in sum");
+  p = (GEN)v[a];
+  for (i=a+1; i<=b; i++) p = gadd(p, (GEN)v[i]);
+  return p;
+}
+
 /*******************************************************************/
 /*                                                                 */
 /*                     SCALAR-MATRIX OPERATIONS                    */
@@ -1145,23 +1191,27 @@ _Fq_addmul(GEN b, long k, long i, GEN m, GEN T, GEN p)
   b[k] = (long)ladd((GEN)b[k], gmul(m, (GEN)b[i]));
 }
 
+/* assume m < p && u_OK_ULONG(p) && (! (b[i] & MASK)) */
+static void
+_u_Fp_addmul_OK(GEN b, long k, long i, ulong m, ulong p)
+{
+  b[k] += m * b[i];
+  if (b[k] & MASK) b[k] %= p;
+}
 /* assume m < p */
 static void
 _u_Fp_addmul(GEN b, long k, long i, ulong m, ulong p)
 {
-  ulong a;
-  if (u_OK_ULONG(p))
-  {
-    if (b[i] & MASK) b[i] %= p;
-    a = b[k] + m * b[i];
-  }
-  else
-  {
-    b[i] %= p;
-    a = b[k] + mulssmod(m, b[i], p);
-  }
-  if (a & MASK) a %= p;
-  b[k] = (long)a;
+  b[i] %= p;
+  b[k] += mulssmod(m, b[i], p);
+  if (b[k] & MASK) b[k] %= p;
+}
+/* same m = 1 */
+static void
+_u_Fp_add(GEN b, long k, long i, ulong p)
+{
+  b[k] += b[i];
+  if (b[k] & MASK) b[k] %= p;
 }
 
 /* Gaussan Elimination. Compute a^(-1)*b
@@ -1303,8 +1353,16 @@ u_FpM_gauss_sp(GEN a, GEN b, ulong p)
       {
 	m = mulssmod(m, u_invmod(piv,p), p);
         m = p - m;
-	for (j=i+1; j<=aco; j++) _u_Fp_addmul((GEN)a[j],k,i,m, p);
-        for (j=1;   j<=bco; j++) _u_Fp_addmul((GEN)b[j],k,i,m, p);
+        if (u_OK_ULONG(p))
+        {
+          for (j=i+1; j<=aco; j++) _u_Fp_addmul_OK((GEN)a[j],k,i,m, p);
+          for (j=1;   j<=bco; j++) _u_Fp_addmul_OK((GEN)b[j],k,i,m, p);
+        }
+        else
+        {
+          for (j=i+1; j<=aco; j++) _u_Fp_addmul((GEN)a[j],k,i,m, p);
+          for (j=1;   j<=bco; j++) _u_Fp_addmul((GEN)b[j],k,i,m, p);
+        }
       }
     }
   }
@@ -1529,11 +1587,10 @@ ZM_inv(GEN M, GEN dM)
   av2 = avma;
   H = NULL;
   d += 3000; /* 27449 = prime(3000) */
-  for(p = 27449; ; p+= *d++)
+  for(p = 27449; ; )
   {
-    if (!*d) err(primer1);
     dMp = umodiu(dM,p);
-    if (!dMp) continue;
+    if (!dMp) goto repeat;
     Hp = u_FpM_inv_sp(u_Fp_FpM(M,p), p);
     if (dMp != 1) Hp = u_FpM_Fp_mul_ip(Hp, dMp, p);
 
@@ -1557,6 +1614,8 @@ ZM_inv(GEN M, GEN dM)
       if (DEBUGMEM>1) err(warnmem,"ZM_inv");
       gerepilemany(av2,gptr, 2);
     }
+   repeat:
+    NEXT_PRIME_VIADIFF_CHECK(p,d);
   }
   if (DEBUGLEVEL>5) msgtimer("ZM_inv done");
   return gerepilecopy(av, H);
@@ -1767,7 +1826,7 @@ approx_0(GEN x, GEN y)
 static long
 gauss_get_pivot_max(GEN x, GEN x0, GEN c, long i0)
 {
-  long i,e, k = i0, ex = -HIGHEXPOBIT, lx = lg(x);
+  long i,e, k = i0, ex = - (long)HIGHEXPOBIT, lx = lg(x);
   GEN p,r;
   if (c)
     for (i=i0; i<lx; i++)
@@ -2300,7 +2359,7 @@ rank(GEN x)
 
 /* if p != NULL, assume x integral and compute rank over Fp */
 static GEN
-indexrank0(GEN x, GEN p, int small)
+indexrank0(GEN x, GEN p, int vecsmall)
 {
   gpmem_t av = avma;
   long i,j,n,r;
@@ -2313,8 +2372,8 @@ indexrank0(GEN x, GEN p, int small)
   n = lg(x)-1; r = n - r;
 
   avma=av; res=cgetg(3,t_VEC);
-  p1=cgetg(r+1,small? t_VECSMALL: t_VEC); res[1]=(long)p1;
-  p2=cgetg(r+1,small? t_VECSMALL: t_VEC); res[2]=(long)p2;
+  p1=cgetg(r+1,vecsmall? t_VECSMALL: t_VEC); res[1]=(long)p1;
+  p2=cgetg(r+1,vecsmall? t_VECSMALL: t_VEC); res[2]=(long)p2;
   if (d)
   {
     for (i=0,j=1; j<=n; j++)
@@ -2322,7 +2381,7 @@ indexrank0(GEN x, GEN p, int small)
     free(d);
     qsort(p1+1,r,sizeof(long),(QSCOMP)pari_compare_long);
   }
-  if (!small)
+  if (!vecsmall)
     for (i=1;i<=r;i++) { p1[i]=lstoi(p1[i]); p2[i]=lstoi(p2[i]); }
   return res;
 }
@@ -2438,15 +2497,23 @@ u_FpM_ker_sp(GEN x, ulong p, long deplin)
       for (i=k+1; i<=n; i++)
 	coeff(x,j,i) = (piv * coeff(x,j,i)) % p;
       for (t=1; t<=m; t++)
-	if (t!=j)
-	{
-	  piv = coeff(x,t,k) % p;
-          if (piv)
-          {
-            coeff(x,t,k) = 0;
+      {
+	if (t==j) continue;
+
+        piv = coeff(x,t,k) % p;
+        if (!piv) continue;
+
+        coeff(x,t,k) = 0;
+        if (piv == 1)
+          for (i=k+1; i<=n; i++) _u_Fp_add((GEN)x[i],t,j,p);
+        else
+        {
+          if (u_OK_ULONG(p))
+            for (i=k+1; i<=n; i++) _u_Fp_addmul_OK((GEN)x[i],t,j,piv,p);
+          else
             for (i=k+1; i<=n; i++) _u_Fp_addmul((GEN)x[i],t,j,piv,p);
-          }
-	}
+        }
+      }
     }
   }
   if (deplin) return NULL;
@@ -2519,18 +2586,18 @@ FpM_ker_i(GEN x, GEN p, long deplin)
       for (i=k+1; i<=n; i++)
 	coeff(x,j,i) = lmodii(mulii(piv,gcoeff(x,j,i)), p);
       for (t=1; t<=m; t++)
-	if (t!=j)
-	{
-	  piv = modii(gcoeff(x,t,k), p);
-          if (signe(piv))
-          {
-            coeff(x,t,k)=zero;
-            for (i=k+1; i<=n; i++)
-              coeff(x,t,i) = laddii(gcoeff(x,t,i),mulii(piv,gcoeff(x,j,i)));
-            if (low_stack(lim, stack_lim(av,1)))
-              gerepile_gauss_FpM_ker(x,p,k,t,av);
-          }
-	}
+      {
+	if (t==j) continue;
+
+        piv = modii(gcoeff(x,t,k), p);
+        if (!signe(piv)) continue;
+
+        coeff(x,t,k)=zero;
+        for (i=k+1; i<=n; i++)
+          coeff(x,t,i) = laddii(gcoeff(x,t,i),mulii(piv,gcoeff(x,j,i)));
+        if (low_stack(lim, stack_lim(av,1)))
+          gerepile_gauss_FpM_ker(x,p,k,t,av);
+      }
     }
   }
   if (deplin) { avma = av0; return NULL; }
@@ -2885,18 +2952,18 @@ FqM_ker_i(GEN x, GEN T, GEN p, long deplin)
       for (i=k+1; i<=n; i++)
 	coeff(x,j,i) = (long) Fq_mul(piv,gcoeff(x,j,i), T, p);
       for (t=1; t<=m; t++)
-	if (t!=j)
-	{
-	  piv = Fq_res(gcoeff(x,t,k), T, p);
-          if (signe(piv))
-          {
-            coeff(x,t,k)=zero;
-            for (i=k+1; i<=n; i++)
-              coeff(x,t,i) = (long)Fq_add(gcoeff(x,t,i),Fq_mul(piv,gcoeff(x,j,i), T, p), T, p);
-            if (low_stack(lim, stack_lim(av,1)))
-              Fq_gerepile_gauss_ker(x,T,p,k,t,av);
-          }
-	}
+      {
+        if (t==j) continue;
+
+        piv = Fq_res(gcoeff(x,t,k), T, p);
+        if (!signe(piv)) continue;
+
+        coeff(x,t,k)=zero;
+        for (i=k+1; i<=n; i++)
+          coeff(x,t,i) = (long)Fq_add(gcoeff(x,t,i),Fq_mul(piv,gcoeff(x,j,i), T, p), T, p);
+        if (low_stack(lim, stack_lim(av,1)))
+          Fq_gerepile_gauss_ker(x,T,p,k,t,av);
+      }
     }
   }
   if (deplin) { avma = av0; return NULL; }
