@@ -1,4 +1,4 @@
-/* $Id: arith2.c,v 1.49 2002/05/16 11:40:38 bill Exp $
+/* $Id: arith2.c,v 1.54 2002/09/11 21:57:53 cvsadmin Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -39,13 +39,12 @@ GEN
 prime(long n)
 {
   byteptr p = diffptr;
-  long c, prime = 0;
+  long prime = 0;
 
   if (n <= 0) err(talker, "n-th prime meaningless if n = %ld",n);
   while (n--)
   {
-    c = *p++; if (!c) err(primer1);
-    prime += c;
+    NEXT_PRIME_VIADIFF_CHECK(prime,p);
   }
   return stoi(prime);
 }
@@ -58,7 +57,10 @@ pith(long n)
 
   if (n <= 0) err(talker, "pith meaningless if n = %ld",n);
   if (maxprime() <= (ulong)n) err(primer1);
-  while (prime<=(ulong)n) { prime += *p++; res++; }
+  while (prime<=(ulong)n) {
+      NEXT_PRIME_VIADIFF(prime,p);
+      res++;
+  }
   return stoi(res-1);
 }
 
@@ -66,15 +68,15 @@ GEN
 primes(long n)
 {
   byteptr p = diffptr;
-  long c, prime = 0;
+  long prime = 0;
   GEN y,z;
 
   if (n < 0) n = 0;
   z = y = cgetg(n+1,t_VEC);
   while (n--)
   {
-    c = *p++; if (!c) err(primer1);
-    prime += c; *++z = lstoi(prime);
+    NEXT_PRIME_VIADIFF_CHECK(prime,p);
+    *++z = lstoi(prime);
   }
   return y;
 }
@@ -86,7 +88,7 @@ primes(long n)
  * when maxnum (size) is moderate.
  */
 static byteptr
-initprimes1(long size, long *lenp, long *lastp)
+initprimes1(ulong size, long *lenp, long *lastp)
 {
   long k;
   byteptr q,r,s,fin, p = (byteptr) gpmalloc(size+2);
@@ -120,16 +122,17 @@ initprimes1(long size, long *lenp, long *lastp)
    recursive call will bottom out and invoke initprimes1() at once.
    (Not static;  might conceivably be useful to someone in library mode) */
 byteptr
-initprimes0(ulong maxnum, long *lenp, long *lastp)
+initprimes0(ulong maxnum, long *lenp, ulong *lastp)
 {
-  long k, size, alloced, asize, psize, rootnum, curlow, last, remains;
+  long k, size, alloced, asize, psize, rootnum;
   byteptr q,r,s,fin, p, p1, fin1, plast, curdiff;
+  ulong last, remains, curlow;
 
 #if 0
   fprintferr("initprimes0: called for maxnum = %lu\n", maxnum);
 #endif
   if (maxnum <= 1ul<<17)	/* Arbitrary. */
-    return initprimes1(maxnum>>1, lenp, lastp);
+    return initprimes1(maxnum>>1, lenp, (long*)lastp);
 
   maxnum |= 1;			/* make it odd. */
 
@@ -183,6 +186,7 @@ initprimes0(ulong maxnum, long *lenp, long *lastp)
     }
     memset(p, 0, asize);
     /* p corresponds to curlow.  q runs over primediffs.  */
+    /* Don't care about DIFFPTR_SKIP: false positives provide no problem */
     for (q = p1+2, k = 3; q <= fin1; k += *q++)
     {
       /* The first odd number which is >= curlow and divisible by p
@@ -208,9 +212,14 @@ initprimes0(ulong maxnum, long *lenp, long *lastp)
     /* now q runs over addresses corresponding to primes */
     for (q = p; ; plast = q++)
     {
+      long d;
+
       while (*q) q++;		/* use 0-sentinel at end */
       if (q >= fin) break;
-      *curdiff++ = (q - plast) << 1;
+      d = (q - plast) << 1;
+      while (d >= DIFFPTR_SKIP)
+	*curdiff++ = DIFFPTR_SKIP, d -= DIFFPTR_SKIP;
+      *curdiff++ = d;
     }
     plast -= asize - 1;
     remains -= asize - 1;
@@ -239,17 +248,17 @@ ulong
 maxprime(void) { return _maxprime; }
 
 byteptr
-initprimes(long maxnum)
+initprimes(ulong maxnum)
 {
-  long len, last;
+  long len;
+  ulong last;
   byteptr p;
-  /* The algorithm must see the next prime beyond maxnum, whence the +257. */
-  /* switch to unsigned: adding the 257 _could_ wrap to a negative number. */
-  ulong maxnum1 = ((maxnum<65302?65302:maxnum)+257ul);
-
-  if (maxnum1 > 436273000)
-    err(talker,"impossible to have prestored primes > 436273009");
+  /* The algorithm must see the next prime beyond maxnum, whence the +512. */
+  /* switch to unsigned: adding the 512 _could_ wrap to a negative number. */
+  ulong maxnum1 = ((maxnum<65302?65302:maxnum)+512ul);
 
+  if ((maxnum>>1) > VERYBIGINT - 1024)
+      err(talker, "Too large primelimit");
   p = initprimes0(maxnum1, &len, &last);
 #if 0 /* not yet... GN */
   static int build_the_tables = 1;
@@ -438,7 +447,7 @@ auxdecomp1(GEN n, long (*ifac_break)(GEN n, GEN pairs, GEN here, GEN state),
   p = 2;
   while (*d && p <= lim1)
   {
-    p += *d++;
+    NEXT_PRIME_VIADIFF(p,d);
     if (mpdivisis(n,p,n))
     {
       nb++; k=1; while (mpdivisis(n,p,n)) k++;
@@ -693,7 +702,7 @@ mu(GEN n)
 
   while (*d && p < lim1)
   {
-    p += *d++;
+    NEXT_PRIME_VIADIFF(p,d);
     if (mpdivisis(n,p,n))
     {
       if (smodis(n,p) == 0) { avma=av; return 0; }
@@ -739,7 +748,7 @@ issquarefree(GEN x)
     p = 2;
     while (*d && p < lim1)
     {
-      p += *d++;
+      NEXT_PRIME_VIADIFF(p,d);
       if (mpdivisis(x,p,x))
       {
 	if (smodis(x,p) == 0) { avma = av; return 0; }
@@ -779,7 +788,7 @@ omega(GEN n)
 
   while (*d && p < lim1)
   {
-    p += *d++;
+    NEXT_PRIME_VIADIFF(p,d);
     if (mpdivisis(n,p,n))
     {
       nb++; while (mpdivisis(n,p,n)); /* empty */
@@ -816,7 +825,7 @@ bigomega(GEN n)
 
   while (*d && p < lim1)
   {
-    p += *d++;
+    NEXT_PRIME_VIADIFF(p,d);
     if (mpdivisis(n,p,n))
     {
       do nb++; while (mpdivisis(n,p,n));
@@ -854,7 +863,7 @@ phi(GEN n)
 
   while (*d && p < lim1)
   {
-    p += *d++;
+    NEXT_PRIME_VIADIFF(p,d);
     if (mpdivisis(n,p,n))
     {
       m = mulis(m, p-1);
@@ -897,7 +906,7 @@ numbdiv(GEN n)
 
   while (*d && p < lim1)
   {
-    p += *d++;
+    NEXT_PRIME_VIADIFF(p,d);
     l=1; while (mpdivisis(n,p,n)) l++;
     m = mulsi(l,m); if (is_pm1(n)) { return gerepileupto(av,m); }
   }
@@ -936,7 +945,7 @@ sumdiv(GEN n)
 
   while (*d && p < lim1)
   {
-    p += *d++;
+    NEXT_PRIME_VIADIFF(p,d);
     if (mpdivisis(n,p,n))
     {
       m1 = utoi(p+1);
@@ -985,7 +994,7 @@ sumdivk(GEN n, long k)
 
   while (*d && p < lim1)
   {
-    p += *d++;
+    NEXT_PRIME_VIADIFF(p,d);
     if (mpdivisis(n,p,n))
     {
       pk = gpowgs(utoi(p),k); m1 = addsi(1,pk);
@@ -1661,11 +1670,11 @@ GEN
 codeform5(GEN x, long prec)
 {
   GEN y = cgetg(6,t_VEC);
-  y[1]=x[1];
-  y[2]=x[2];
-  y[3]=x[3];
-  y[4]=zero;
-  y[5]=(long)realun(prec); return y;
+  y[1] = x[1];
+  y[2] = x[2];
+  y[3] = x[3];
+  y[4] = zero;
+  y[5] = (long)realun(prec); return y;
 }
 
 static GEN
@@ -1893,52 +1902,55 @@ qfbred0(GEN x, long flag, GEN D, GEN isqrtD, GEN sqrtD)
 GEN
 primeform(GEN x, GEN p, long prec)
 {
-  gpmem_t av,tetpil;
-  long s=signe(x);
-  GEN y,b;
+  gpmem_t av;
+  long s, sx = signe(x);
+  GEN y, b, c;
 
-  if (typ(x) != t_INT || !s) err(arither1);
+  if (typ(x) != t_INT || !sx) err(arither1);
   if (typ(p) != t_INT || signe(p) <= 0) err(arither1);
   if (is_pm1(p))
-    return s<0? imag_unit_form_by_disc(x)
-              : real_unit_form_by_disc(x,prec);
-  if (s < 0)
+    return sx<0? imag_unit_form_by_disc(x)
+               : real_unit_form_by_disc(x,prec);
+  s = mod8(x);
+  if (sx < 0)
   {
-    y = cgetg(4,t_QFI); s = 8-mod8(x);
+    if (s) s = 8-s;
+    y = cgetg(4, t_QFI);
   }
   else
   {
-    y = cgetg(5,t_QFR); s = mod8(x);
-    y[4]=(long)realzero(prec);
+    y = cgetg(5, t_QFR);
+    y[4] = (long)realzero(prec);
   }
   switch(s&3)
   {
     case 2: case 3: err(funder2,"primeform");
   }
-  y[1] = (long)icopy(p); av = avma;
-  if (egalii(p,gdeux))
+  av = avma;
+  if (egalii(p, gdeux))
   {
     switch(s)
     {
-      case 0: y[2]=zero; break;
-      case 8: s=0; y[2]=zero; break;
-      case 1: s=1; y[2]=un; break;
-      case 4: s=4; y[2]=deux; break;
-      default: err(sqrter5);
+      case 0: b = gzero; break;
+      case 1: b = gun;   break;
+      case 4: b = gdeux; break;
+      default: err(sqrter5); b = NULL; /* -Wall */
     }
-    b=subsi(s,x); tetpil=avma; b=shifti(b,-3);
+    c = shifti(subsi(s,x), -3);
   }
   else
   {
     b = mpsqrtmod(x,p); if (!b) err(sqrter5);
-    if (mod2(b) == mod2(x)) y[2] = (long)b;
-    else
-    { tetpil = avma; y[2] = lpile(av,tetpil,subii(p,b)); }
+    s &= 1; /* s = x mod 2 */
+    /* mod(b) != mod2(x) ? [Warning: we may have b == 0] */
+    if ((!signe(b) && s) || mod2(b) != s) b = gerepileuptoint(av, subii(p,b));
 
-    av=avma; b=shifti(subii(sqri((GEN)y[2]),x),-2);
-    tetpil=avma; b=divii(b,p);
+    av = avma;
+    c = diviiexact(shifti(subii(sqri(b), x), -2), p);
   }
-  y[3]=lpile(av,tetpil,b); return y;
+  y[3] = lpileuptoint(av, c);
+  y[2] = (long)b;
+  y[1] = licopy(p); return y;
 }
 
 /*********************************************************************/
