@@ -1,4 +1,4 @@
-/* $Id: base1.c,v 1.96 2002/06/07 17:41:17 karim Exp $
+/* $Id: base1.c,v 1.103 2002/09/07 13:34:38 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -20,6 +20,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /**************************************************************/
 #include "pari.h"
 #include "parinf.h"
+
+int new_galois_format = 0;
+
 extern GEN R_from_QR(GEN x, long prec);
 extern GEN to_polmod(GEN x, GEN mod);
 extern GEN roots_to_pol_r1r2(GEN a, long r1, long v);
@@ -34,30 +37,49 @@ extern ulong smulss(ulong x, ulong y, ulong *rem);
 void
 checkrnf(GEN rnf)
 {
-  if (typ(rnf)!=t_VEC) err(idealer1);
-  if (lg(rnf)!=12) err(idealer1);
+  if (typ(rnf)!=t_VEC || lg(rnf)!=12) err(idealer1);
 }
 
 GEN
-checkbnf(GEN bnf)
+_checkbnf(GEN bnf)
 {
-  if (typ(bnf)!=t_VEC) err(idealer1);
-  switch (lg(bnf))
-  {
-    case 11: return bnf;
-    case 10:
-      if (typ(bnf[1])==t_POL)
-        err(talker,"please apply bnfinit first");
-      break;
-    case 7:
-      return checkbnf((GEN)bnf[1]);
+  if (typ(bnf) == t_VEC)
+    switch (lg(bnf))
+    {
+      case 11: return bnf;
+      case 7:  return checkbnf((GEN)bnf[1]);
 
-    case 3:
-      if (typ(bnf[2])==t_POLMOD)
-        return checkbnf((GEN)bnf[1]);
+      case 3:
+        if (typ(bnf[2])==t_POLMOD)
+          return checkbnf((GEN)bnf[1]);
+    }
+  return NULL;
+}
+
+GEN
+_checknf(GEN nf)
+{
+  if (typ(nf)==t_VEC)
+    switch(lg(nf))
+    {
+      case 10: return nf;
+      case 11: return checknf((GEN)nf[7]);
+      case 7:  return checknf((GEN)nf[1]);
+      case 3: if (typ(nf[2]) == t_POLMOD) return checknf((GEN)nf[1]);
+    }
+  return NULL;
+}
+
+GEN
+checkbnf(GEN x)
+{
+  GEN bnf = _checkbnf(x);
+  if (!bnf)
+  {
+    if (_checknf(x)) err(talker,"please apply bnfinit first");
+    err(idealer1);
   }
-  err(idealer1);
-  return NULL; /* not reached */
+  return bnf;
 }
 
 GEN
@@ -70,19 +92,15 @@ checkbnf_discard(GEN bnf)
 }
 
 GEN
-checknf(GEN nf)
+checknf(GEN x)
 {
-  if (typ(nf)==t_POL) err(talker,"please apply nfinit first");
-  if (typ(nf)!=t_VEC) err(idealer1);
-  switch(lg(nf))
+  GEN nf = _checknf(x);
+  if (!nf)
   {
-    case 10: return nf;
-    case 11: return checknf((GEN)nf[7]);
-    case 7:  return checknf((GEN)nf[1]);
-    case 3: if (typ(nf[2]) == t_POLMOD) return checknf((GEN)nf[1]);
+    if (typ(x)==t_POL) err(talker,"please apply nfinit first");
+    err(idealer1);
   }
-  err(idealer1);
-  return NULL; /* not reached */
+  return nf;
 }
 
 void
@@ -362,6 +380,7 @@ _res(long n, long s, long k)
   GEN y = cgetg(4, t_VEC);
   y[1] = lstoi(n);
   y[2] = lstoi(s);
+  if (!new_galois_format) k = (n == 6 && (k == 6 || k == 2))? 2: 1;
   y[3] = lstoi(k); return y;
 }
 
@@ -678,6 +697,16 @@ galoisapply(GEN nf, GEN aut, GEN x)
 GEN pol_to_monic(GEN pol, GEN *lead);
 int cmp_pol(GEN x, GEN y);
 
+GEN
+get_bnfpol(GEN x, GEN *bnf, GEN *nf)
+{
+  *bnf = _checkbnf(x);
+  *nf  = _checknf(x);
+  if (*nf) return (GEN)(*nf)[1];
+  if (typ(x) != t_POL) err(idealer1);
+  return x;
+}
+
 GEN
 get_nfpol(GEN x, GEN *nf)
 {
@@ -993,7 +1022,7 @@ static void
 get_roots_for_M(nffp_t *F)
 {
   long n, eBD, er, PREC;
-  
+
   if (F->extraprec < 0)
   { /* not initialized yet */
     n = degpol(F->x);
@@ -1033,7 +1062,7 @@ make_M(nffp_t *F, int trunc)
       for (i=1; i<l; i++) m[i] = lmul((GEN)m[i], invd);
     }
   }
-  
+
   if (trunc && gprecision(M) > F->prec)
   {
     M     = gprec_w(M, F->prec);
@@ -1298,9 +1327,9 @@ ok_pol(void *TT, GEN xn)
 {
   ok_pol_t *T = (ok_pol_t*)TT;
   GEN dxn;
-  
+
   if (++T->ind > T->indmax && T->xbest) return T->xbest;
-  
+
   if (!ZX_is_squarefree(xn)) return (T->ind == T->indmax)? T->xbest: NULL;
   if (DEBUGLEVEL>3) outerr(xn);
   dxn = ZX_disc(xn);
@@ -1667,19 +1696,8 @@ factoredpolred2(GEN x, GEN fa)
 /********************************************************************/
 
 GEN
-sum(GEN v, long a, long b)
+T2_from_embed_norm(GEN x, long r1)
 {
-  GEN p;
-  long i;
-  if (a > b) return gzero;
-  p = (GEN)v[a];
-  for (i=a+1; i<=b; i++) p = gadd(p, (GEN)v[i]);
-  return p;
-}
-
-GEN
-  T2_from_embed_norm(GEN x, long r1)
-  {
   GEN p = sum(x, 1, r1);
   GEN q = sum(x, r1+1, lg(x)-1);
   if (q != gzero) p = gadd(p, gmul2n(q,1));
@@ -1767,7 +1785,8 @@ chk_gen_init(FP_chk_fun *chk, GEN gram, GEN mat)
     if (prev && degpol(prev) < N && !gegal(prev,P))
     {
       if (degpol(prev) * degpol(P) > 64) continue; /* too expensive */
-      P = (GEN)compositum(prev,P)[1];
+      P = compositum(prev,P);
+      P = (GEN)P[lg(P)-1];
       if (degpol(P) == N) continue;
       if (DEBUGLEVEL>2 && degpol(P) != degpol(prev))
         fprintferr("chk_gen_init: subfield %Z\n",P);
@@ -1814,7 +1833,7 @@ findmindisc(GEN *py, GEN *pa)
 {
   GEN v, dmin, z, b, discs, y = *py, a = *pa;
   long i,k, l = lg(y);
-  
+
   if (l == 2) { *py = (GEN)y[1]; *pa = (GEN)a[1]; return; }
 
   discs = cgetg(l,t_VEC);
@@ -1963,14 +1982,11 @@ polredabs0(GEN x, long flag)
 }
 
 GEN
-polredabsall(GEN x, long flun)
-{ return polredabs0(x, flun | nf_ALL); }
+polredabsall(GEN x, long flun) { return polredabs0(x, flun | nf_ALL); }
 GEN
-polredabs(GEN x)
-{ return polredabs0(x,0); }
+polredabs(GEN x) { return polredabs0(x,0); }
 GEN
-polredabs2(GEN x)
-{ return polredabs0(x,nf_ORIG); }
+polredabs2(GEN x) { return polredabs0(x,nf_ORIG); }
 
 static long
 nf_pm1(GEN y)
@@ -2078,7 +2094,7 @@ dirzetak0(GEN nf, long N0)
 
   while (court[2]<=N0)
   {
-    court[2] += *d++; if (! *d) err(primer1);
+    NEXT_PRIME_VIADIFF_CHECK(court[2], d);
     if (smodis(disc,court[2])) /* court does not divide index */
       { vect = (GEN) simplefactmod(pol,court)[1]; lx=lg(vect); }
     else
@@ -2140,7 +2156,7 @@ initzeta(GEN pol, long prec)
   /*************** Calcul du residu et des constantes ***************/
   eps=gmul2n(gun,-bit_accuracy(prec)-6); p1=dbltor(0.5);
   nfz=cgetg(10,t_VEC);
-  bnf=buchinit(pol,p1,p1,prec+1); prec=(prec<<1)-1;
+  bnf = bnfinit0(pol,2,NULL,prec+1); prec=(prec<<1)-1;
   bnf = checkbnf_discard(bnf);
   Pi = mppi(prec); racpi=gsqrt(Pi,prec);
 
