@@ -1,4 +1,4 @@
-/* $Id: base3.c,v 1.81 2002/05/20 12:19:32 karim Exp $
+/* $Id: base3.c,v 1.87 2002/09/08 17:09:39 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -579,20 +579,6 @@ element_val(GEN nf, GEN x, GEN vp)
   avma=av; return w + vcx*e;
 }
 
-/* d = a multiple of norm(x), x integral */
-long
-element_val2(GEN nf, GEN x, GEN d, GEN vp)
-{
-  GEN p = (GEN)vp[1];
-  gpmem_t av;
-  long v = ggval(d,p);
-
-  if (!v) return 0;
-  av=avma;
-  v = int_elt_val(nf,x,p,(GEN)vp[5],NULL);
-  avma=av; return v;
-}
-
 /* polegal without comparing variables */
 long
 polegal_spec(GEN x, GEN y)
@@ -632,7 +618,7 @@ basistoalg(GEN nf, GEN x)
       for (i=1; i<lx; i++) z[i]=(long)basistoalg(nf,(GEN)x[i]);
       return z;
     case t_MAT: z=cgetg(lx,t_MAT);
-      if (!lx) return z;
+      if (lx == 1) return z;
       l = lg(x[1]);
       for (j=1; j<lx; j++)
       {
@@ -817,6 +803,22 @@ zsigne(GEN nf,GEN x,GEN arch)
   avma = av; setlg(vecsign,j); return vecsign;
 }
 
+/* return the t_COL vector of signs of x; the matrix of such if x is a vector
+ * of nf elements */
+GEN
+zsigns(GEN nf, GEN x)
+{
+  long r1, i, l;
+  GEN arch, S;
+
+  nf = checknf(nf); r1 = nf_get_r1(nf);
+  arch = cgetg(r1+1,t_VEC); for (i=1; i<=r1; i++) arch[i] = un;
+  if (typ(x) != t_VEC) return zsigne(nf, x, arch);
+  l = lg(x); S = cgetg(l, t_MAT);
+  for (i=1; i<l; i++) S[i] = (long)zsigne(nf, (GEN)x[i], arch);
+  return S;
+}
+
 GEN
 lllreducemodmatrix(GEN x,GEN y)
 {
@@ -1765,6 +1767,7 @@ zideallog(GEN nf, GEN x, GEN bid)
     case t_POLMOD: case t_POL:
       x = algtobasis(nf,x); break;
     case t_MAT:
+      if (lg(x) == 1) return zerocol(c-1);
       return famat_ideallog(nf,(GEN)x[1],(GEN)x[2],bid);
     case t_COL: break;
     default: err(talker,"not an element in zideallog");
@@ -1775,8 +1778,8 @@ zideallog(GEN nf, GEN x, GEN bid)
   {
     GEN g = cgetg(3, t_COL);
     GEN e = cgetg(3, t_COL);
-    g[1] = lmul(x,den); e[1] = un;
-    g[2] = (long)den;   e[2] = lstoi(-1);
+    g[1] = (long)Q_muli_to_int(x,den); e[1] = un;
+    g[2] = (long)den;                  e[2] = lstoi(-1);
     p1 = famat_ideallog(nf, g, e, bid);
   }
   else
@@ -2003,7 +2006,7 @@ ideallistzstarall(GEN bnf,long bound,long flag)
   if (bound > (long)maxprime()) err(primer1);
   for (p[2]=0; p[2]<=bound; )
   {
-    p[2] += *ptdif++;
+    NEXT_PRIME_VIADIFF(p[2], ptdif);
     if (DEBUGLEVEL>1) { fprintferr("%ld ",p[2]); flusherr(); }
     fa = primedec(nf,p);
     for (j=1; j<lg(fa); j++)
@@ -2106,6 +2109,7 @@ ideallist_arch(GEN nf,GEN list,GEN arch,long flun)
   long nba,i,j,lx,ly;
   GEN p1,z,p2;
 
+  if (typ(arch) != t_VEC) err(typeer,"ideallistarch");
   nba=0; for (i=1; i<lg(arch); i++) if (signe(arch[i])) nba++;
   lx=lg(list); z=cgetg(lx,t_VEC);
   for (i=1; i<lx; i++)
