@@ -1,4 +1,4 @@
-/* $Id: base4.c,v 1.128 2002/06/06 09:56:15 karim Exp $
+/* $Id: base4.c,v 1.143 2002/09/08 10:41:21 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -25,6 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 extern GEN makeprimetoideal(GEN nf,GEN UV,GEN uv,GEN x);
 extern GEN gauss_triangle_i(GEN A, GEN B,GEN t);
 extern GEN hnf_invimage(GEN A, GEN b);
+extern int hnfdivide(GEN A, GEN B);
 extern GEN colreducemodHNF(GEN x, GEN y, GEN *Q);
 extern GEN zinternallog_pk(GEN nf,GEN a0,GEN y,GEN pr,GEN prk,GEN list,GEN *psigne);
 extern GEN special_anti_uniformizer(GEN nf, GEN pr);
@@ -190,6 +191,8 @@ GEN
 principalideal(GEN nf, GEN x)
 {
   GEN z = cgetg(2,t_MAT);
+
+  nf = checknf(nf);
   switch(typ(x))
   {
     case t_INT: case t_FRAC: case t_FRACN:
@@ -215,72 +218,166 @@ principalideal(GEN nf, GEN x)
 static GEN
 mylog(GEN x, long prec)
 {
-  if (gcmp0(x))
-    err(precer,"get_arch");
+  if (gcmp0(x)) err(precer,"get_arch");
   return glog(x,prec);
 }
 
-/* for internal use */
+GEN get_arch(GEN nf,GEN x,long prec);
+
+static GEN
+famat_get_arch(GEN nf, GEN x, long prec)
+{
+  GEN A, a, g = (GEN)x[1], e = (GEN)x[2];
+  long i, l = lg(e);
+
+  if (l <= 1)
+    return get_arch(nf, gun, prec);
+  A = NULL; /* -Wall */
+  for (i=1; i<l; i++)
+  {
+    a = get_arch(nf, (GEN)g[i], prec);
+    a = gmul((GEN)e[i], a);
+    A = (i == 1)? a: gadd(A, a);
+  }
+  return A;
+}
+
+static GEN
+scalar_get_arch(long R1, long RU, GEN x, long prec)
+{
+  GEN v = cgetg(RU+1,t_VEC);
+  GEN p1 = glog(x, prec);
+  long i;
+
+  for (i=1; i<=R1; i++) v[i] = (long)p1;
+  if (i <= RU)
+  {
+    p1 = gmul2n(p1,1);
+    for (   ; i<=RU; i++) v[i] = (long)p1;
+  }
+  return v;
+}
+
+/* For internal use. Get archimedean components: [e_i log( sigma_i(x) )],
+ * with e_i = 1 (resp 2.) for i <= R1 (resp. > R1) */
 GEN
 get_arch(GEN nf,GEN x,long prec)
 {
-  long i,R1,RU;
-  GEN v,p1,p2;
+  long i, RU, R1 = nf_get_r1(nf);
+  GEN v;
+
+  RU = lg(nf[6]) - 1;
+  switch(typ(x))
+  {
+    case t_MAT: return famat_get_arch(nf,x,prec);
+    case t_POLMOD:
+    case t_POL: x = algtobasis_i(nf,x);   /* fall through */
+    case t_COL: if (!isnfscalar(x)) break;
+      x = (GEN)x[1]; /* fall through */
+    default: /* rational number */
+      return scalar_get_arch(R1, RU, x, prec);
+  }
 
-  R1=itos(gmael(nf,2,1)); RU = R1+itos(gmael(nf,2,2));
-  if (typ(x)!=t_COL) x = algtobasis_i(nf,x);
   v = cgetg(RU+1,t_VEC);
   if (isnfscalar(x)) /* rational number */
   {
-    p1 = glog((GEN)x[1],prec);
-    p2 = (RU > R1)? gmul2n(p1,1): NULL;
-    for (i=1; i<=R1; i++) v[i]=(long)p1;
-    for (   ; i<=RU; i++) v[i]=(long)p2;
-  }
-  else
-  {
-    x = gmul(gmael(nf,5,1),x);
-    for (i=1; i<=R1; i++) v[i] = (long)mylog((GEN)x[i],prec);
-    for (   ; i<=RU; i++) v[i] = lmul2n(mylog((GEN)x[i],prec),1);
+    GEN p1 = glog((GEN)x[1], prec);
+
+    for (i=1; i<=R1; i++) v[i] = (long)p1;
+    if (i <= RU)
+    {
+      p1 = gmul2n(p1,1);
+      for (   ; i<=RU; i++) v[i] = (long)p1;
+    }
   }
+  x = gmul(gmael(nf,5,1),x);
+  for (i=1; i<=R1; i++) v[i] = (long)mylog((GEN)x[i],prec);
+  for (   ; i<=RU; i++) v[i] = lmul2n(mylog((GEN)x[i],prec),1);
   return v;
 }
 
+GEN get_arch_real(GEN nf,GEN x,GEN *emb,long prec);
+
+static GEN
+famat_get_arch_real(GEN nf,GEN x,GEN *emb,long prec)
+{
+  GEN A, T, a, t, g = (GEN)x[1], e = (GEN)x[2];
+  long i, l = lg(e);
+
+  if (l <= 1)
+    return get_arch_real(nf, gun, emb, prec);
+  A = T = NULL; /* -Wall */
+  for (i=1; i<l; i++)
+  {
+    a = get_arch_real(nf, (GEN)g[i], &t, prec);
+    if (!a) return NULL;
+    a = gmul((GEN)e[i], a);
+    t = vecpow(t, (GEN)e[i]);
+    if (i == 1) { A = a;          T = t; }
+    else        { A = gadd(A, a); T = vecmul(T, t); }
+  }
+  *emb = T; return A;
+}
+
+static GEN
+scalar_get_arch_real(long R1, long RU, GEN u, GEN *emb, long prec)
+{
+  GEN v, x, p1;
+  long i, s;
+
+  s = gsigne(u);
+  if (!s) err(talker,"0 in get_arch_real");
+  x = cgetg(RU+1, t_COL);
+  for (i=1; i<=RU; i++) x[i] = (long)u;
+
+  v = cgetg(RU+1, t_COL);
+  p1 = (s > 0)? glog(u,prec): gzero;
+  for (i=1; i<=R1; i++) v[i] = (long)p1;
+  if (i <= RU)
+  {
+    p1 = gmul2n(p1,1);
+    for (   ; i<=RU; i++) v[i] = (long)p1;
+  }
+  *emb = x; return v;
+}
+
+static int
+low_prec(GEN x)
+{
+  return gcmp0(x) || (typ(x) == t_REAL && lg(x) == 3);
+}
+
 /* as above but return NULL if precision problem, and set *emb to the
  * embeddings of x */
 GEN
-get_arch_real(GEN nf,GEN x,GEN *emb,long prec)
+get_arch_real(GEN nf, GEN x, GEN *emb, long prec)
 {
-  long i,R1,RU;
-  GEN v,p1,p2;
+  long i, RU, R1 = nf_get_r1(nf);
+  GEN v, t;
 
-  R1=itos(gmael(nf,2,1)); RU = R1+itos(gmael(nf,2,2));
-  if (typ(x)!=t_COL) x = algtobasis_i(nf,x);
-  v = cgetg(RU+1,t_COL);
-  if (isnfscalar(x)) /* rational number */
+  RU = lg(nf[6])-1;
+  switch(typ(x))
   {
-    GEN u = (GEN)x[1];
-    i = signe(u);
-    if (!i) err(talker,"0 in get_arch_real");
-    p1= (i > 0)? glog(u,prec): gzero;
-    p2 = (RU > R1)? gmul2n(p1,1): NULL;
-    for (i=1; i<=R1; i++) v[i]=(long)p1;
-    for (   ; i<=RU; i++) v[i]=(long)p2;
+    case t_MAT: return famat_get_arch_real(nf,x,emb,prec);
+
+    case t_POLMOD:
+    case t_POL: x = algtobasis_i(nf,x);   /* fall through */
+    case t_COL: if (!isnfscalar(x)) break;
+      x = (GEN)x[1]; /* fall through */
+    default: /* rational number */
+      return scalar_get_arch_real(R1, RU, x, emb, prec);
   }
-  else
+  v = cgetg(RU+1,t_COL);
+  x = gmul(gmael(nf,5,1), x);
+  for (i=1; i<=R1; i++)
   {
-    GEN t;
-    x = gmul(gmael(nf,5,1),x);
-    for (i=1; i<=R1; i++)
-    {
-      t = gabs((GEN)x[i],prec); if (gcmp0(t)) return NULL;
-      v[i] = llog(t,prec);
-    }
-    for (   ; i<=RU; i++)
-    {
-      t = gnorm((GEN)x[i]); if (gcmp0(t)) return NULL;
-      v[i] = llog(t,prec);
-    }
+    t = gabs((GEN)x[i],prec); if (low_prec(t)) return NULL;
+    v[i] = llog(t,prec);
+  }
+  for (   ; i<=RU; i++)
+  {
+    t = gnorm((GEN)x[i]); if (low_prec(t)) return NULL;
+    v[i] = llog(t,prec);
   }
   *emb = x; return v;
 }
@@ -288,12 +385,12 @@ get_arch_real(GEN nf,GEN x,GEN *emb,long prec)
 GEN
 principalidele(GEN nf, GEN x, long prec)
 {
-  GEN p1,y = cgetg(3,t_VEC);
+  GEN p1, y = cgetg(3,t_VEC);
   gpmem_t av;
 
   p1 = principalideal(nf,x);
   y[1] = (long)p1;
-  av =avma; p1 = get_arch(nf,(GEN)p1[1],prec);
+  av = avma; p1 = get_arch(nf,(GEN)p1[1],prec);
   y[2] = lpileupto(av,p1); return y;
 }
 
@@ -1091,6 +1188,30 @@ idealmat_mul(GEN nf, GEN x, GEN y)
   return cx? gmul(y,cx): y;
 }
 
+/* operations on elements in factored form */
+
+GEN
+to_famat(GEN g, GEN e)
+{
+  GEN h = cgetg(3,t_MAT);
+  if (typ(g) != t_COL) { g = dummycopy(g); settyp(g, t_COL); }
+  if (typ(e) != t_COL) { e = dummycopy(e); settyp(e, t_COL); }
+  h[1] = (long)g;
+  h[2] = (long)e; return h;
+}
+
+GEN
+to_famat_all(GEN x, GEN y) { return to_famat(_col(x), _col(y)); }
+
+static GEN
+append(GEN v, GEN x)
+{
+  long i, l = lg(v);
+  GEN w = cgetg(l+1, typ(v));
+  for (i=1; i<l; i++) w[i] = lcopy((GEN)v[i]);
+  w[i] = lcopy(x); return w;
+}
+
 /* add x^1 to factorisation f */
 static GEN
 famat_add(GEN f, GEN x)
@@ -1103,7 +1224,7 @@ famat_add(GEN f, GEN x)
   }
   else
   {
-    h[1] = (long)concat((GEN)f[1], x);
+    h[1] = (long)append((GEN)f[1], x); /* x may be a t_COL */
     h[2] = (long)concat((GEN)f[2], gun);
   }
   return h;
@@ -1133,7 +1254,7 @@ famat_sqr(GEN f)
   h[2] = lmul2n((GEN)f[2],1);
   return h;
 }
-static GEN
+GEN
 famat_inv(GEN f)
 {
   GEN h;
@@ -1143,11 +1264,12 @@ famat_inv(GEN f)
   h[2] = lneg((GEN)f[2]);
   return h;
 }
-static GEN
+GEN
 famat_pow(GEN f, GEN n)
 {
   GEN h;
   if (lg(f) == 1) return cgetg(1,t_MAT);
+  if (typ(f) != t_MAT) return to_famat_all(f,n);
   h = cgetg(3,t_MAT);
   h[1] = lcopy((GEN)f[1]);
   h[2] = lmul((GEN)f[2],n);
@@ -1176,7 +1298,7 @@ elt_cmp(GEN x, GEN y)
 {
   long tx = typ(x), ty = typ(y);
   if (ty == tx)
-    return tx == t_POL? cmp_pol(x,y): lexcmp(x,y);
+    return (tx == t_POL || tx == t_POLMOD)? cmp_pol(x,y): lexcmp(x,y);
   return tx - ty;
 }
 static int
@@ -1198,6 +1320,7 @@ famat_reduce(GEN fa)
   L = gen_sort(g, cmp_IND|cmp_C, &elt_cmp);
   G = cgetg(l, t_COL);
   E = cgetg(l, t_COL);
+  /* merge */
   for (k=i=1; i<l; i++,k++) 
   {
     G[k] = g[L[i]];
@@ -1208,23 +1331,20 @@ famat_reduce(GEN fa)
       k--;
     }
   }
+  /* kill 0 exponents */
+  l = k;
+  for (k=i=1; i<l; i++)
+    if (!gcmp0((GEN)E[i]))
+    {
+      G[k] = G[i];
+      E[k] = E[i]; k++;
+    }
   F = cgetg(3, t_MAT);
   setlg(G, k); F[1] = (long)G;
   setlg(E, k); F[2] = (long)E; return F;
 }
 
-GEN
-to_famat(GEN g, GEN e)
-{
-  GEN h = cgetg(3,t_MAT);
-  h[1] = (long)g;
-  h[2] = (long)e; return h;
-}
-
-GEN
-to_famat_all(GEN x, GEN y) { return to_famat(_col(x), _col(y)); }
-
-/* assume (num(g[i]), id) = 1 and for all i. return prod g[i]^e[i] mod id */
+/* assume (num(g[i]), id) = 1 for all i. Return prod g[i]^e[i] mod id */
 GEN
 famat_to_nf_modideal_coprime(GEN nf, GEN g, GEN e, GEN id)
 {
@@ -1246,44 +1366,47 @@ famat_to_nf_modideal_coprime(GEN nf, GEN g, GEN e, GEN id)
   return t? t: gscalcol(gun, lg(id)-1);
 }
 
-/* assume prh has degree 1 and coprime to numerator(x) */
-GEN
-nf_to_Fp_simple(GEN x, GEN prh)
+/* assume pr has degree 1 and coprime to numerator(x) */
+static GEN
+nf_to_Fp_simple(GEN x, GEN modpr, GEN p)
 {
-  GEN ch = denom(x), p = gcoeff(prh,1,1);
-  if (!is_pm1(ch))
-  {
-    x = gmul(gmul(x,ch), mpinvmod(ch,p));
-  }
-  ch = colreducemodHNF(gmod(x, p), prh, NULL);
-  return (GEN)ch[1]; /* in Fp^* */
+  GEN c, r = zk_to_ff(Q_primitive_part(x, &c), modpr);
+  if (c) r = gmod(gmul(r, c), p);
+  return r;
 }
 
-GEN
-famat_to_Fp_simple(GEN nf, GEN g, GEN e, GEN prh)
+static GEN
+famat_to_Fp_simple(GEN nf, GEN x, GEN modpr, GEN p)
 {
-  GEN t = gun, h,n, p = gcoeff(prh,1,1), q = subis(p,1);
-  long i, th, lx = lg(g);
-  for (i=1; i<lx; i++)
+  GEN h, n, t = gun, g = (GEN)x[1], e = (GEN)x[2], q = subis(p,1);
+  long i, l = lg(g);
+
+  for (i=1; i<l; i++)
   {
     n = (GEN)e[i]; n = modii(n,q);
     if (!signe(n)) continue;
-    h = (GEN)g[i]; th = typ(h);
-    if (th == t_POL || th == t_POLMOD) h = algtobasis(nf, h);
-    if (th != t_COL) h = gmod(h, p); else h = nf_to_Fp_simple(h, prh);
+
+    h = (GEN)g[i];
+    switch(typ(h))
+    {
+      case t_POL: case t_POLMOD: h = algtobasis(nf, h);  /* fall through */
+      case t_COL: h = nf_to_Fp_simple(h, modpr, p); break;
+      default: h = gmod(h, p);
+    }
     t = mulii(t, powmodulo(h, n, p)); /* not worth reducing */
   }
   return modii(t, p);
 }
 
-/* cf famat_to_nf_modideal_coprime, but id is a prime of degree 1 (=prh) */
+/* cf famat_to_nf_modideal_coprime, but id is a prime of degree 1 (=pr) */
 GEN
-to_Fp_simple(GEN nf, GEN x, GEN prh)
+to_Fp_simple(GEN nf, GEN x, GEN pr)
 {
+  GEN T, p, modpr = zk_to_ff_init(nf, &pr, &T, &p);
   switch(typ(x))
   {
-    case t_COL: return nf_to_Fp_simple(x,prh);
-    case t_MAT: return famat_to_Fp_simple(nf,(GEN)x[1],(GEN)x[2],prh);
+    case t_COL: return nf_to_Fp_simple(x,modpr,p);
+    case t_MAT: return famat_to_Fp_simple(nf,x,modpr,p);
     default: err(impl,"generic conversion to finite field");
   }
   return NULL;
@@ -1300,7 +1423,7 @@ to_Fp_simple(GEN nf, GEN x, GEN prh)
 static GEN
 famat_makecoprime(GEN nf, GEN g, GEN e, GEN pr, GEN prk, GEN EX)
 {
-  long i,k, l = lg(g);
+  long i, l = lg(g);
   GEN prkZ,cx,x,u, zpow = gzero, p = (GEN)pr[1], b = (GEN)pr[5];
   GEN mul = eltmul_get_table(nf, b);
   GEN newg = cgetg(l+1, t_VEC); /* room for z */
@@ -1310,12 +1433,15 @@ famat_makecoprime(GEN nf, GEN g, GEN e, GEN pr, GEN prk, GEN EX)
   {
     x = (GEN)g[i];
     if (typ(x) != t_COL) x = algtobasis(nf, x);
-    cx = denom(x); x = gmul(x,cx);
-    k = pvaluation(cx, p, &u);
-    if (!gcmp1(u)) /* could avoid the inversion, but prkZ is small--> cheap */
-      x = gmul(x, mpinvmod(u, prkZ));
-    if (k)
-      zpow = addii(zpow, mulsi(k, (GEN)e[i]));
+    x = Q_remove_denom(x, &cx);
+    if (cx)
+    {
+      long k = pvaluation(cx, p, &u);
+      if (!gcmp1(u)) /* could avoid the inversion, but prkZ is small--> cheap */
+        x = gmul(x, mpinvmod(u, prkZ));
+      if (k)
+        zpow = addii(zpow, mulsi(k, (GEN)e[i]));
+    }
     (void)int_elt_val(nf, x, p, mul, &x);
     newg[i] = (long)colreducemodHNF(x, prk, NULL);
   }
@@ -1336,14 +1462,15 @@ famat_ideallog(GEN nf, GEN g, GEN e, GEN bid)
   GEN vp = gmael(bid, 3,1), ep = gmael(bid, 3,2), arch = gmael(bid,1,2);
   GEN cyc = gmael(bid,2,2), list_set = (GEN)bid[4], U = (GEN)bid[5];
   GEN p1,y0,x,y, psigne;
-  long i;
+  long i, l;
   if (lg(cyc) == 1) return cgetg(1,t_COL);
   y0 = y = cgetg(lg(U), t_COL);
   psigne = zsigne(nf, to_famat(g,e), arch);
-  for (i=1; i<lg(vp); i++)
+  l = lg(vp);
+  for (i=1; i < l; i++)
   {
     GEN pr = (GEN)vp[i], prk;
-    prk = idealpow(nf, pr, (GEN)ep[i]);
+    prk = (l==2)? gmael(bid,1,1): idealpow(nf, pr, (GEN)ep[i]);
     /* TODO: FIX group exponent (should be mod prk, not f !) */
     x = famat_makecoprime(nf, g, e, pr, prk, (GEN)cyc[1]);
     y = zinternallog_pk(nf, x, y, pr, prk, (GEN)list_set[i], &psigne);
@@ -1384,6 +1511,7 @@ famat_to_arch(GEN nf, GEN fa, long prec)
   GEN g,e, y = NULL;
   long i,l;
 
+  if (typ(fa) != t_MAT) return get_arch(nf, fa, prec);
   if (lg(fa) == 1) return zerovec(lg(nf[6])-1);
   g = (GEN)fa[1];
   e = (GEN)fa[2]; l = lg(e);
@@ -1497,10 +1625,13 @@ idealmul(GEN nf, GEN x, GEN y)
           p1 = idealhermite_aux(nf, element_mul(nf,x,y));
           break;
         case id_PRIME:
-          p1 = eltmul_get_table(nf, x);
-          p1 = concatsp(gmul(p1,(GEN)y[1]), gmul(p1,(GEN)y[2]));
+        {
+          GEN mx = eltmul_get_table(nf, x);
+          GEN mpi= eltmul_get_table(nf, (GEN)y[2]);
+          p1 = concatsp(gmul(mx,(GEN)y[1]), gmul(mx,mpi));
           p1 = idealmat_to_hnf(nf, p1);
           break;
+        }
         default: /* id_MAT */
           p1 = idealmat_to_hnf(nf, gmul(eltmul_get_table(nf,x), y));
       }break;
@@ -1757,15 +1888,6 @@ idealpows(GEN nf, GEN ideal, long e)
   affsi(e,court); return idealpow(nf,ideal,court);
 }
 
-GEN
-init_idele(GEN nf)
-{
-  GEN x = cgetg(3,t_VEC);
-  long RU;
-  nf = checknf(nf); RU = lg(nf[6])-1;
-  x[2] = (long)zerovec(RU); return x;
-}
-
 static GEN
 _idealmulred(GEN nf, GEN x, GEN y, long prec)
 {
@@ -1979,7 +2101,7 @@ chk_vdir(GEN nf, GEN vdir)
 
 /* assume I in NxN matrix form (not necessarily HNF) */
 static GEN
-ideallllred_elt_i(GEN *ptnf, GEN I, GEN vdir, long *ptprec)
+idealred_elt_i(GEN *ptnf, GEN I, GEN vdir, long *ptprec)
 {
   GEN G, u, y, nf = *ptnf;
   long i, e, prec = *ptprec;
@@ -2004,10 +2126,15 @@ ideallllred_elt_i(GEN *ptnf, GEN I, GEN vdir, long *ptprec)
 }
 
 GEN
-ideallllred_elt(GEN nf, GEN I)
+ideallllred_elt(GEN nf, GEN I, GEN vdir)
 {
   long prec = DEFAULTPREC;
-  return ideallllred_elt_i(&nf, I, NULL, &prec);
+  return idealred_elt_i(&nf, I, vdir, &prec);
+}
+GEN
+idealred_elt(GEN nf, GEN I)
+{
+  return ideallllred_elt(nf, I, NULL);
 }
 
 GEN
@@ -2035,7 +2162,7 @@ ideallllred(GEN nf, GEN I, GEN vdir, long prec)
     Ired = lllint_ip(I,4);
   else
     Ired = I;
-  y = ideallllred_elt_i(&nf, Ired, chk_vdir(nf,vdir), &prec);
+  y = idealred_elt_i(&nf, Ired, chk_vdir(nf,vdir), &prec);
 
   if (isnfscalar(y))
   { /* already reduced */
@@ -2065,7 +2192,7 @@ END:
 
   switch(typ(aI))
   {
-    case t_POLMOD: case t_MAT: /* compute y, I0 = J y */
+    case t_POLMOD: /* compute y, I0 = J y */
       if (!Nx) y = c1;
       else
       {
@@ -2074,6 +2201,15 @@ END:
       }
       break;
 
+    case t_MAT: /* compute y, I0 = J y */
+      if (!Nx) y = c1;
+      else
+      {
+        c = mul_content(c,c1);
+        y = c? gmul(y, gdiv(c,Nx)): gdiv(y, Nx);
+      }
+      break;
+
     case t_COL:
       if (y) y = vecinv(gmul(gmael(nf,5,1), y));
       break;
@@ -2286,6 +2422,13 @@ make_integral(GEN nf, GEN L0, GEN f, GEN *listpr, GEN *ptd1)
   *ptd1 = d1; return L;
 }
 
+void
+check_listpr(GEN x)
+{
+  long l = lg(x), i;
+  for (i=1; i<l; i++) checkprimeid((GEN)x[i]);
+}
+
 /* Given a prime ideal factorization with possibly zero or negative
  * exponents, gives b such that v_p(b) = v_p(x) for all prime ideals pr | x
  * and v_pr(b)> = 0 for all other pr.
@@ -2301,9 +2444,10 @@ idealapprfact_i(GEN nf, GEN x)
 
   nf = checknf(nf);
   if (typ(x) != t_MAT || lg(x) != 3)
-    err(talker,"not a prime ideal factorization in idealapprfact");
+    err(talker,"not a factorization in idealapprfact");
+  check_listpr((GEN)x[1]);
   if (DEBUGLEVEL>4) {
-    fprintferr(" entering idealapprfact() :\n");
+    fprintferr(" entering idealapprfact():\n");
     fprintferr(" x = %Z\n", x);
   }
   list= (GEN)x[1];
@@ -2810,7 +2954,7 @@ nfsmith(GEN nf, GEN x)
 #define trivlift(x) ((typ(x)==t_POLMOD)? (GEN)x[2]: lift_intern(x))
 
 GEN
-element_mulmodpr2(GEN nf, GEN x, GEN y, GEN modpr)
+element_mulmodpr(GEN nf, GEN x, GEN y, GEN modpr)
 {
   gpmem_t av=avma;
   GEN p1;
@@ -2854,7 +2998,9 @@ element_powmodpr(GEN nf,GEN x,GEN k,GEN pr)
   modpr = nf_to_ff_init(nf,&pr,&T,&p);
   z = nf_to_ff(nf,x,modpr);
   z = FpXQ_pow(z,k,T,p);
-  return gerepileupto(av, algtobasis(nf, ff_to_nf(z,modpr)));
+  z = ff_to_nf(z,modpr);
+  if (typ(z) != t_COL) z = algtobasis(nf, z);
+  return gerepilecopy(av, z);
 }
 
 GEN
