@@ -1,4 +1,4 @@
-/* $Id: bibli2.c,v 1.36 2002/05/09 15:19:15 karim Exp $
+/* $Id: bibli2.c,v 1.38 2002/09/07 21:45:57 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -29,7 +29,7 @@ extern GEN _ei(long n, long i);
 /********************************************************************/
 
 GEN
-tayl(GEN x, long v, long precdl)
+tayl(GEN x, long v, long precS)
 {
   long i, vx = gvar9(x);
   gpmem_t tetpil, av=avma;
@@ -38,13 +38,13 @@ tayl(GEN x, long v, long precdl)
   if (v <= vx)
   {
     long p1[] = { evaltyp(t_SER)|_evallg(2), 0 };
-    p1[1] = evalvalp(precdl) | evalvarn(v);
+    p1[1] = evalvalp(precS) | evalvarn(v);
     return gadd(p1,x);
   }
   p1=cgetg(v+2,t_VEC);
   for (i=0; i<v; i++) p1[i+1]=lpolx[i];
   p1[vx+1]=lpolx[v]; p1[v+1]=lpolx[vx];
-  y = tayl(changevar(x,p1), vx,precdl); tetpil=avma;
+  y = tayl(changevar(x,p1), vx,precS); tetpil=avma;
   return gerepile(av,tetpil, changevar(y,p1));
 }
 
@@ -67,7 +67,8 @@ grando0(GEN x, long n, long do_clone)
   {
     if (tx != t_POL && ! is_rfrac_t(tx))
       err(talker,"incorrect argument in O()");
-    v=gvar(x); m=n*gval(x,v);
+    v = gvar(x); if (v > MAXVARN) err(talker,"incorrect object in O()");
+    m = n*gval(x,v);
   }
   return zeroser(v,m);
 }
@@ -527,6 +528,23 @@ binome(GEN n, long k)
   return gerepileupto(av, y);
 }
 
+/* Assume n >= 1, return bin, bin[k+1] = binomial(n, k) */
+GEN
+vecbinome(long n)
+{
+  long d = (n + 1)/2, k;
+  GEN bin = cgetg(n+2, t_VEC), *C;
+  C = (GEN*)(bin + 1); /* C[k] = binomial(n, k) */
+  C[0] = gun;
+  for (k=1; k <= d; k++)
+  {
+    gpmem_t av = avma;
+    C[k] = gerepileuptoint(av, diviiexact(mulsi(n-k+1, C[k-1]), stoi(k)));
+  }
+  for (   ; k <= n; k++) C[k] = C[n - k];
+  return bin;
+}
+
 /********************************************************************/
 /**                                                                **/
 /**                  POLYNOMIAL INTERPOLATION                      **/
