@@ -1,4 +1,4 @@
-/* $Id: buch3.c,v 1.88 2002/06/07 21:13:27 karim Exp $
+/* $Id: buch3.c,v 1.96 2002/09/08 10:40:52 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -21,29 +21,35 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "pari.h"
 #include "parinf.h"
 
+extern void testprimes(GEN bnf, long bound);
+extern GEN Fp_PHlog(GEN a, GEN g, GEN p, GEN ord);
 extern GEN FqX_factor(GEN x, GEN T, GEN p);
-extern long FqX_is_squarefree(GEN P, GEN T, GEN p);
-extern GEN sqred1_from_QR(GEN x, long prec);
-extern GEN make_integral(GEN nf, GEN L0, GEN f, GEN *listpr, GEN *ptd1);
-extern GEN idealprodprime(GEN nf, GEN L);
 extern GEN anti_unif_mod_f(GEN nf, GEN pr, GEN sqf);
-extern GEN unif_mod_f(GEN nf, GEN pr, GEN sqf);
+extern GEN arch_mul(GEN x, GEN y);
+extern GEN check_and_build_cycgen(GEN bnf);
 extern GEN colreducemodHNF(GEN x, GEN y, GEN *Q);
+extern GEN detcyc(GEN cyc);
+extern GEN famat_reduce(GEN fa);
 extern GEN famat_to_nf_modideal_coprime(GEN nf, GEN g, GEN e, GEN id);
 extern GEN famat_to_nf_modidele(GEN nf, GEN g, GEN e, GEN bid);
-extern GEN check_and_build_cycgen(GEN bnf);
 extern GEN gmul_mat_smallvec(GEN x, GEN y);
+extern GEN idealaddtoone_i(GEN nf, GEN x, GEN y);
+extern GEN idealprodprime(GEN nf, GEN L);
 extern GEN ideleaddone_aux(GEN nf,GEN x,GEN ideal);
+extern GEN isprincipalfact(GEN bnf,GEN P, GEN e, GEN C, long flag);
 extern GEN logunitmatrix(GEN nf,GEN funits,GEN racunit,GEN bid);
-extern GEN vconcat(GEN Q1, GEN Q2);
-extern void minim_alloc(long n, double ***q, GEN *x, double **y,  double **z, double **v);
+extern GEN make_integral(GEN nf, GEN L0, GEN f, GEN *listpr, GEN *ptd1);
+extern GEN sqred1_from_QR(GEN x, long prec);
+extern GEN subgroupcondlist(GEN cyc, GEN bound, GEN listKer);
+extern GEN to_Fp_simple(GEN nf, GEN x, GEN ffproj);
 extern GEN to_famat_all(GEN x, GEN y);
 extern GEN trivfact(void);
-extern GEN arch_mul(GEN x, GEN y);
-extern GEN isprincipalfact(GEN bnf,GEN P, GEN e, GEN C, long flag);
-extern GEN idealaddtoone_i(GEN nf, GEN x, GEN y);
-extern GEN ideallllred_elt(GEN nf, GEN I);
-extern GEN subgroupcondlist(GEN cyc, GEN bound, GEN listKer);
+extern GEN unif_mod_f(GEN nf, GEN pr, GEN sqf);
+extern GEN vconcat(GEN Q1, GEN Q2);
+extern long FqX_is_squarefree(GEN P, GEN T, GEN p);
+extern long int_elt_val(GEN nf, GEN x, GEN p, GEN b, GEN *newx);
+extern void minim_alloc(long n, double ***q, GEN *x, double **y,  double **z, double **v);
+extern void rowselect_p(GEN A, GEN B, GEN p, long init);
 
 /* FIXME: obsolete, see zarchstar (which is much slower unfortunately). */
 static GEN
@@ -214,10 +220,10 @@ redideal(GEN nf,GEN x,GEN f)
 {
   GEN q, y, b;
 
-  if (gcmp1(gcoeff(f,1,1))) return ideallllred_elt(nf, x); /* f = 1 */
+  if (gcmp1(gcoeff(f,1,1))) return idealred_elt(nf, x); /* f = 1 */
 
   b = idealaddtoone_i(nf,x,f); /* a = b mod (x f) */
-  y = ideallllred_elt(nf, idealmullll(nf,x,f));
+  y = idealred_elt(nf, idealmullll(nf,x,f));
   q = ground(element_div(nf,b,y));
   return gsub(b, element_mul(nf,q,y)); /* != 0 since = 1 mod f */
 }
@@ -237,7 +243,7 @@ too_big(GEN nf, GEN bet)
 
 /* GTM 193: Algo 4.3.4. Reduce x mod idele */
 static GEN
-idealmodidele(GEN nf, GEN x, GEN idele, GEN sarch)
+_idealmodidele(GEN nf, GEN x, GEN idele, GEN sarch)
 {
   gpmem_t av = avma;
   GEN a,A,D,G, f = (GEN)idele[1];
@@ -251,6 +257,15 @@ idealmodidele(GEN nf, GEN x, GEN idele, GEN sarch)
   return idealmul(nf, a, x);
 }
 
+GEN
+idealmodidele(GEN bnr, GEN x)
+{
+  GEN bid = (GEN)bnr[2], fa2 = (GEN)bid[4];
+  GEN idele = (GEN)bid[1];
+  GEN sarch = (GEN)fa2[lg(fa2)-1];
+  return _idealmodidele(checknf(bnr), x, idele, sarch);
+}
+
 /* v_pr(L0 * cx). tau = pr[5] or (more efficient) mult. table for pr[5] */
 static long
 fast_val(GEN nf,GEN L0,GEN cx,GEN pr,GEN tau)
@@ -387,7 +402,7 @@ compute_raygen(GEN nf, GEN u1, GEN gen, GEN bid)
     I = idealmul(nf,I,G);
     if (d) I = Q_div_to_int(I,d);
     /* more or less useless, but cheap at this point */
-    I = idealmodidele(nf,I,module,sarch);
+    I = _idealmodidele(nf,I,module,sarch);
     basecl[i] = (long)I;
   }
   return basecl;
@@ -490,7 +505,6 @@ buchrayall(GEN bnf,GEN module,long flag)
   clg = cgetg(add_gen? 4: 3, t_VEC);
   clg[1] = (long)detcyc(met);
   clg[2] = (long)met;
-  /* met[1] = group exponent */
   if (add_gen) clg[3] = (long)compute_raygen(nf,u1,genplus,bid);
   if (!do_init) return gerepilecopy(av, clg);
 
@@ -585,7 +599,7 @@ quick_isprincipalgen(GEN bnf, GEN x)
   GEN z = cgetg(3, t_VEC), gen = gmael3(bnf,8,1,3);
   GEN idep, ep = isprincipal(bnf,x);
   /* x \prod g[i]^(-ep[i]) = factorisation of principal ideal */
-  idep = isprincipalfact(bnf, gen, gneg(ep), x, nf_GENMAT | nf_GEN | nf_FORCE);
+  idep = isprincipalfact(bnf, gen, gneg(ep), x, nf_GENMAT|nf_FORCE);
   z[1] = (long)ep;
   z[2] = idep[2]; return z;
 }
@@ -737,62 +751,6 @@ zimmertbound(long N,long R2,GEN DK)
   avma = av; return itos(w);
 }
 
-/* all primes up to Minkowski bound factor on factorbase ? */
-static void
-testprime(GEN bnf, long minkowski)
-{
-  gpmem_t av = avma;
-  long pp,i,nbideal,k,pmax;
-  GEN f,p1,vectpp,fb,dK, nf=checknf(bnf);
-  byteptr delta = diffptr;
-
-  if (DEBUGLEVEL>1)
-    fprintferr("PHASE 1: check primes to Zimmert bound = %ld\n\n",minkowski);
-  f=(GEN)nf[4]; dK=(GEN)nf[3];
-  if (!gcmp1(f))
-  {
-    GEN different = gmael(nf,5,5);
-    if (DEBUGLEVEL>1)
-      fprintferr("**** Testing Different = %Z\n",different);
-    p1 = isprincipalall(bnf,different,nf_FORCE);
-    if (DEBUGLEVEL>1) fprintferr("     is %Z\n",p1);
-  }
-  fb=(GEN)bnf[5];
-  p1 = gmael(fb, lg(fb)-1, 1); /* largest p in factorbase */
-  pp = 0; pmax = is_bigint(p1)? VERYBIGINT: itos(p1);
-  if ((ulong)minkowski > maxprime()) err(primer1);
-  while (pp < minkowski)
-  {
-    pp += *delta++;
-    if (DEBUGLEVEL>1) fprintferr("*** p = %ld\n",pp);
-    vectpp=primedec(bnf,stoi(pp)); nbideal=lg(vectpp)-1;
-    /* loop through all P | p if ramified, all but one otherwise */
-    if (!smodis(dK,pp)) nbideal++;
-    for (i=1; i<nbideal; i++)
-    {
-      GEN P = (GEN)vectpp[i];
-      if (DEBUGLEVEL>1)
-        fprintferr("  Testing P = %Z\n",P);
-      if (cmpis(idealnorm(bnf,P), minkowski) < 1)
-      {
-	if (pp <= pmax && (k = tablesearch(fb, P, cmp_prime_ideal)))
-	{
-	  if (DEBUGLEVEL>1) fprintferr("    #%ld in factor base\n",k);
-	}
-	else
-	{
-	  p1 = isprincipal(bnf,P);
-	  if (DEBUGLEVEL>1) fprintferr("    is %Z\n",p1);
-	}
-      }
-      else if (DEBUGLEVEL>1)
-        fprintferr("    Norm(P) > Zimmert bound\n");
-    }
-    avma = av;
-  }
-  if (DEBUGLEVEL>1) { fprintferr("End of PHASE 1.\n\n"); flusherr(); }
-}
-
 /* return \gamma_n^n if known, an upper bound otherwise */
 static GEN
 hermiteconstant(long n)
@@ -812,7 +770,7 @@ hermiteconstant(long n)
     case 8: return stoi(256);
   }
   av = avma;
-  h  = gpuigs(divsr(2,mppi(DEFAULTPREC)), n);
+  h  = gpowgs(divsr(2,mppi(DEFAULTPREC)), n);
   h1 = gsqr(ggamma(gdivgs(stoi(n+4),2),DEFAULTPREC));
   return gerepileupto(av, gmul(h,h1));
 }
@@ -848,34 +806,32 @@ isprimitive(GEN nf)
   return 1;
 }
 
+static GEN
+dft_bound()
+{
+  if (DEBUGLEVEL>1) fprintferr("Default bound for regulator: 0.2\n");
+  return dbltor(0.2);
+}
+
 static GEN
 regulatorbound(GEN bnf)
 {
-  long N,R1,R2,R;
-  GEN nf,dKa,bound,p1,c1;
+  long N, R1, R2, R;
+  GEN nf, dKa, p1, c1;
 
   nf = (GEN)bnf[7]; N = degpol(nf[1]);
-  bound = dbltor(0.2);
-  if (!isprimitive(nf))
-  {
-    if (DEBUGLEVEL>1) fprintferr("Default bound for regulator: 0.2\n");
-    return bound;
-  }
+  if (!isprimitive(nf)) return dft_bound();
+
   dKa = absi((GEN)nf[3]);
-  R1 = itos(gmael(nf,2,1));
-  R2 = itos(gmael(nf,2,2)); R = R1+R2-1;
-  if (!R2 && N<12) c1 = gpuigs(stoi(4),N>>1); else c1 = gpuigs(stoi(N),N);
-  if (cmpii(dKa,c1) <= 0)
-  {
-    if (DEBUGLEVEL>1) fprintferr("Default bound for regulator: 0.2\n");
-    return bound;
-  }
+  nf_get_sign(nf, &R1, &R2); R = R1+R2-1;
+  if (!R2 && N<12) c1 = gpowgs(stoi(4),N>>1); else c1 = gpowgs(stoi(N),N);
+  if (cmpii(dKa,c1) <= 0) return dft_bound();
+
   p1 = gsqr(glog(gdiv(dKa,c1),DEFAULTPREC));
-  p1 = gdivgs(gmul2n(gpuigs(gdivgs(gmulgs(p1,3),N*(N*N-1)-6*R2),R),R2),N);
-  p1 = gsqrt(gdiv(p1, hermiteconstant(R)), DEFAULTPREC);
-  if (gcmp(p1,bound) > 0) bound = p1;
+  p1 = divrs(gmul2n(gpowgs(divrs(mulrs(p1,3),N*(N*N-1)-6*R2),R),R2), N);
+  p1 = mpsqrt(gdiv(p1, hermiteconstant(R)));
   if (DEBUGLEVEL>1) fprintferr("Mahler bound for regulator: %Z\n",p1);
-  return bound;
+  return gmax(p1, dbltor(0.2));
 }
 
 /* x given by its embeddings */
@@ -909,7 +865,7 @@ static GEN
 minimforunits(GEN nf, long BORNE, GEN w)
 {
   const long prec = MEDDEFAULTPREC;
-  long n, i, j, k, s, *x, r1;
+  long n, i, j, k, s, *x, r1, cnt = 0;
   gpmem_t av = avma;
   GEN u,r,a,M;
   double p, norme, normin, normax;
@@ -961,6 +917,8 @@ minimforunits(GEN nf, long BORNE, GEN w)
     if (!x[1] && y[1]<=eps) break;
 
     if (DEBUGLEVEL>8){ fprintferr("."); flusherr(); }
+    if (++cnt == 5000) return NULL; /* too expensive */
+
     p = x[1]+z[1]; norme = y[1] + p*p*v[1] + eps;
     if (norme > normax) normax = norme;
     if (is_unit(M,r1, x)
@@ -1031,8 +989,8 @@ compute_M0(GEN M_star,long N)
       { /* n3 > N/3 >= n1 */
 	k = n2; kk = N-2*k;
 	p2=gsub(M_star,gmulgs(X,k));
-	p3=gmul(gpuigs(stoi(kk),kk),gpuigs(gsubgs(gmul(M_star,p2),kk*kk),k));
-	pol=gsub(p3,gmul(gmul(gpuigs(stoi(k),k),gpuigs(X,k)),gpuigs(p2,N-k)));
+	p3=gmul(gpowgs(stoi(kk),kk),gpowgs(gsubgs(gmul(M_star,p2),kk*kk),k));
+	pol=gsub(p3,gmul(gmul(gpowgs(stoi(k),k),gpowgs(X,k)),gpowgs(p2,N-k)));
 	prec=gprecision(pol); if (!prec) prec = MEDDEFAULTPREC;
 	r=roots(pol,prec); lr = lg(r);
 	for (i=1; i<lr; i++)
@@ -1050,7 +1008,7 @@ compute_M0(GEN M_star,long N)
           if (gsigne(v) <= 0) continue;
 
           u=gmul2n(gadd(S,p6),-1);
-          w=gpui(P,gdivgs(stoi(-k),kk),prec);
+          w=gpow(P,gdivgs(stoi(-k),kk),prec);
           p6=gmulsg(k,gadd(gsqr(glog(u,prec)),gsqr(glog(v,prec))));
           M0_pro=gmul2n(gadd(p6,gmulsg(kk,gsqr(glog(w,prec)))),-2);
           if (DEBUGLEVEL>2)
@@ -1068,7 +1026,7 @@ compute_M0(GEN M_star,long N)
 	f2 = gadd(f2,gmulsg(n2,gmul(X,Z)));
 	f2 = gadd(f2,gmulsg(n3,gmul(X,Y)));
 	f2 = gsub(f2,gmul(M,gmul(X,gmul(Y,Z))));
-	f3 = gsub(gmul(gpuigs(X,n1),gmul(gpuigs(Y,n2),gpuigs(Z,n3))), gun);
+	f3 = gsub(gmul(gpowgs(X,n1),gmul(gpowgs(Y,n2),gpowgs(Z,n3))), gun);
         /* f1 = n1 X + n2 Y + n3 Z - M */
         /* f2 = n1 YZ + n2 XZ + n3 XY */
         /* f3 = X^n1 Y^n2 Z^n3 - 1*/
@@ -1153,6 +1111,7 @@ lowerboundforregulator_i(GEN bnf)
   if (gexpo(minunit) > 30) return NULL;
 
   vecminim = minimforunits(nf, itos(gceil(minunit)), gmael3(bnf,8,4,1));
+  if (!vecminim) return NULL;
   bound = (GEN)vecminim[3];
   i = gexpo(gsub(bound,minunit));
   if (i > -5) err(bugparier,"lowerboundforregulator");
@@ -1162,7 +1121,7 @@ lowerboundforregulator_i(GEN bnf)
     fprintferr("M* = %Z\n", bound);
     if (DEBUGLEVEL>2)
     {
-      p1=polx[0]; pol=gaddgs(gsub(gpuigs(p1,N),gmul(bound,p1)),N-1);
+      p1=polx[0]; pol=gaddgs(gsub(gpowgs(p1,N),gmul(bound,p1)),N-1);
       p1 = roots(pol,DEFAULTPREC);
       if (N&1) y=greal((GEN)p1[3]); else y=greal((GEN)p1[2]);
       M0 = gmul2n(gmulsg(N*(N-1),gsqr(glog(y,DEFAULTPREC))),-2);
@@ -1172,7 +1131,7 @@ lowerboundforregulator_i(GEN bnf)
   }
   M0 = compute_M0(bound,N);
   if (DEBUGLEVEL>1) { fprintferr("M0 = %Z\n",gprec_w(M0,3)); flusherr(); }
-  M = gmul2n(gdivgs(gdiv(gpuigs(M0,RU),hermiteconstant(RU)),N),R2);
+  M = gmul2n(gdivgs(gdiv(gpowgs(M0,RU),hermiteconstant(RU)),N),R2);
   if (gcmp(M, dbltor(0.04)) < 0) return NULL;
   M = gsqrt(M,DEFAULTPREC);
   if (DEBUGLEVEL>1)
@@ -1189,9 +1148,6 @@ lowerboundforregulator(GEN bnf)
   return x;
 }
 
-extern GEN to_Fp_simple(GEN nf, GEN x, GEN prh);
-extern GEN Fp_PHlog(GEN a, GEN g, GEN p, GEN ord);
-
 /* Compute a square matrix of rank length(beta) associated to a family
  * (P_i), 1<=i<=length(beta), of primes s.t. N(P_i) = 1 mod pp, and
  * (P_i,beta[j]) = 1 for all i,j */
@@ -1199,7 +1155,7 @@ static void
 primecertify(GEN bnf,GEN beta,long pp,GEN big)
 {
   long i,j,qq,nbcol,lb,nbqq,ra,N;
-  GEN nf,mat,mat1,qgen,decqq,newcol,Qh,Q,g,ord;
+  GEN nf,mat,mat1,qgen,decqq,newcol,Q,g,ord,modpr;
 
   ord = NULL; /* gcc -Wall */
   nbcol = 0; nf = (GEN)bnf[7]; N = degpol(nf[1]);
@@ -1220,11 +1176,11 @@ primecertify(GEN bnf,GEN beta,long pp,GEN big)
         g = lift_intern(gener(qgen)); /* primitive root */
         ord = decomp(stoi(qq-1));
       }
-      Qh = prime_to_ideal(nf,Q);
+      modpr = zkmodprinit(nf, Q);
       newcol = cgetg(lb+1,t_COL);
       for (j=1; j<=lb; j++)
       {
-        GEN t = to_Fp_simple(nf, (GEN)beta[j], Qh);
+        GEN t = to_Fp_simple(nf, (GEN)beta[j], modpr);
         newcol[j] = (long)Fp_PHlog(t,g,qgen,ord);
       }
       if (DEBUGLEVEL>3)
@@ -1279,9 +1235,9 @@ certifybuchall(GEN bnf)
 
   bnf = checkbnf(bnf); nf = (GEN)bnf[7];
   N=degpol(nf[1]); if (N==1) return 1;
-  R1=itos(gmael(nf,2,1)); R2=itos(gmael(nf,2,2)); R=R1+R2-1;
+  nf_get_sign(nf, &R1, &R2); R = R1+R2-1;
   funits = check_units(bnf,"bnfcertify");
-  testprime(bnf, zimmertbound(N,R2,absi((GEN)nf[3])));
+  testprimes(bnf, zimmertbound(N,R2,absi((GEN)nf[3])));
   reg = gmael(bnf,8,2);
   cyc = gmael3(bnf,8,1,2); nbgen = lg(cyc)-1;
   gen = gmael3(bnf,8,1,3); rootsofone = gmael(bnf,8,4);
@@ -1318,8 +1274,10 @@ certifybuchall(GEN bnf)
   rootsofone = dummycopy(rootsofone);
   rootsofone[2] = (long)algtobasis(nf, (GEN)rootsofone[2]);
 
-  for (p = *delta++; p <= bound; p += *delta++)
+  for (p = *delta++; p <= bound; ) {  
     check_prime(p,bnf,cyc,cycgen,funits,rootsofone,big);
+    NEXT_PRIME_VIADIFF(p, delta);
+  }
 
   if (nbgen)
   {
@@ -1548,7 +1506,7 @@ rnfnormgroup(GEN bnr, GEN polrel)
      * and including last pr^f or p^f is the same, but the last isprincipal
      * is much easier! oldf is used to track this */
 
-    p += *d++; if (!*d) err(primer1);
+    NEXT_PRIME_VIADIFF_CHECK(p,d);
     if (!smodis(index, p)) continue; /* can't be treated efficiently */
 
     fa = primedec(nf, stoi(p)); lfa = lg(fa)-1;
@@ -2127,8 +2085,6 @@ rayclassnointern(GEN blist, GEN h)
   return L;
 }
 
-extern void rowselect_p(GEN A, GEN B, GEN p, long init);
-
 static GEN
 rayclassnointernarch(GEN blist, GEN h, GEN matU)
 {
@@ -2280,7 +2236,7 @@ discrayabslistarchintern(GEN bnf, GEN arch, long bound, long ramip)
   if (bound > (long)maxprime()) err(primer1);
   for (p[2]=0; p[2]<=bound; )
   {
-    p[2] += *ptdif++;
+    NEXT_PRIME_VIADIFF(p[2], ptdif);
     if (!flbou && p[2]>sqbou)
     {
       if (DEBUGLEVEL>1) fprintferr("\nStarting rayclassno computations\n");
@@ -2451,7 +2407,7 @@ discrayabslistarchintern(GEN bnf, GEN arch, long bound, long ramip)
             ideal = idealmulpowprime(nf,ideal,p1,(GEN)ex[k]);
           }
           S=0; clhss=0;
-          normi = ii; normps= itos(gpuigs(gprime,resp));
+          normi = ii; normps= itos(gpowgs(gprime,resp));
           for (j=1; j<=ep; j++)
           {
             GEN fad, fad1, fad2;
