@@ -1,4 +1,4 @@
-/* $Id: galconj.c,v 1.101 2002/06/10 14:20:58 karim Exp $
+/* $Id: galconj.c,v 1.106 2002/09/08 10:47:36 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -343,6 +343,7 @@ galoisborne(GEN T, GEN dn, struct galois_borne *gb, long ppp)
   prec = ZX_get_prec(T);
   den = initgaloisborne(T,dn,prec, &L,&prep,NULL);
   if (!dn) den = gclone(den);
+  if (DEBUGLEVEL>=4) TIMERstart(&ti);
   M = vandermondeinverse(L, gmul(T, realun(prec)), den, prep);
   if (DEBUGLEVEL>=4) msgTIMER(&ti,"vandermondeinverse");
   borne = matrixnorm(M, prec);
@@ -1640,7 +1641,7 @@ galoisanalysis(GEN T, struct galois_analysis *ga, long calcul_l, long karma_type
       break;
   }
   /*Now, we study the orders of the Frobenius elements*/
-  min_prime=n*max(BITS_IN_LONG-bfffo(n)-4,2);
+  min_prime=n*max((long)(BITS_IN_LONG-bfffo(n)-4),2);
   plift = 0;
   nbmax = 8+(n>>1);
   nbtest = 0; karma = k_amoeba;
@@ -1653,13 +1654,10 @@ galoisanalysis(GEN T, struct galois_analysis *ga, long calcul_l, long karma_type
          && (nbtest < 3 * nbmax || !(group&ga_non_wss)) ;)
   {
     gpmem_t av;
-    long    prime_incr;
     GEN     ip,FS,p1;
     long o,norm_o=1;
-    prime_incr = *primepointer++;
-    if (!prime_incr)
-      err(primer1);
-    p += prime_incr;
+
+    NEXT_PRIME_VIADIFF_CHECK(p,primepointer);
     /*discard small primes*/
     if (p <= min_prime)
       continue;
@@ -1745,17 +1743,14 @@ galoisanalysis(GEN T, struct galois_analysis *ga, long calcul_l, long karma_type
   if (calcul_l && O[1]<=linf)
   {
     gpmem_t av;
-    long    prime_incr;
     long    l=0;
     /*we need a totally splited prime l*/
     av = avma;
     while (l == 0)
     {
       long nb;
-      prime_incr = *primepointer++;
-      if (!prime_incr)
-	err(primer1);
-      p += prime_incr;
+
+      NEXT_PRIME_VIADIFF_CHECK(p,primepointer);
       if (p<=linf) continue;
       nb=FpX_nbroots(T,stoi(p));
       if (nb == n)
@@ -2628,7 +2623,7 @@ galoisfindfrobenius(GEN T, GEN L, GEN den, struct galois_frobenius *gf,
   {
     gpmem_t av = avma;
     long    isram;
-    long    i,c;
+    long    i;
     GEN     ip,Tmod;
     ip = stoi(gf->p);
     Tmod = lift((GEN) factmod(T, ip));
@@ -2673,10 +2668,7 @@ galoisfindfrobenius(GEN T, GEN L, GEN den, struct galois_frobenius *gf,
 	  err(warner, "galoisconj _may_ hang up for this polynomial");
       }
     }
-    c = *primepointer++;
-    if (!c)
-      err(primer1);
-    gf->p += c;
+    NEXT_PRIME_VIADIFF_CHECK(gf->p, primepointer);
     if (DEBUGLEVEL >= 4)
       fprintferr("GaloisConj:next p=%ld\n", gf->p);
     avma = av;
@@ -3057,13 +3049,11 @@ numberofconjugates(GEN T, long pdepart)
   ltop2 = avma;
   for (p = 0, primepointer = diffptr; nbtest < nbmax && card > 1;)
   {
-    long    s, c;
+    long    s;
     long    isram;
     GEN     S;
-    c = *primepointer++;
-    if (!c)
-      err(primer1);
-    p += c;
+
+    NEXT_PRIME_VIADIFF_CHECK(p,primepointer);
     if (p < pdepart)
       continue;
     S = (GEN) simplefactmod(T, stoi(p));
@@ -3384,13 +3374,14 @@ galoistestabelian(GEN gal)
 GEN galoisisabelian(GEN gal, long flag)
 {
   long i, j;
-  long test, n=lg(gal[7]);
+  long test, n;
   GEN M;
   gal = checkgal(gal);
   test=galoistestabelian(gal);
   if (!test) return gzero;
   if (flag==1)  return gun;
   if (flag) err(flagerr,"galoisisabelian");
+  n=lg(gal[7]);
   M=cgetg(n,t_MAT);
   for(i=1;i<n;i++)
   {
