@@ -1,4 +1,4 @@
-/* $Id: gen1.c,v 1.50 2002/05/10 15:14:28 karim Exp $
+/* $Id: gen1.c,v 1.54 2002/09/12 18:37:01 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -116,8 +116,8 @@ gred_rfrac2_i(GEN x1, GEN x2)
   long tx,ty;
 
   if (gcmp0(x1)) return gcopy(x1);
-
-  tx=typ(x1); ty=typ(x2);
+  x1 = simplify_i(x1); tx = typ(x1);
+  x2 = simplify_i(x2); ty = typ(x2);
   if (ty!=t_POL)
   {
     if (tx!=t_POL) return gred_rfrac_copy(x1,x2);
@@ -880,14 +880,16 @@ gadd(GEN x, GEN y)
 GEN
 fix_rfrac_if_pol(GEN x, GEN y)
 {
-  if (gcmp1(y)) return x;
+  gpmem_t av = avma;
+  y = simplify(y);
+  if (gcmp1(y)) { avma = av; return x; }
   if (typ(y) != t_POL)
   {
     if (typ(x) != t_POL || gvar2(y) > varn(x))
       return gdiv(x,y);
   }
   else if (varn(y) > varn(x)) return gdiv(x,y);
-  return NULL;
+  avma = av; return NULL;
 }
 
 static long
@@ -1787,7 +1789,7 @@ gdiv(GEN x, GEN y)
     switch (signe(x))
     {
       case 0:
-        if (gcmp0(y)) err(gdiver2);
+        if (gcmp0(y)) err(gdiver);
         if (ty != t_INTMOD) return gzero;
         z = cgetg(3,t_INTMOD); icopyifstack(y[1],z[1]); z[2]=zero;
         return z;
@@ -1846,7 +1848,7 @@ gdiv(GEN x, GEN y)
         return gerepile(av,tetpil,gdiv(p2,p1));
     }
   }
-  if (gcmp0(y) && ty != t_MAT) err(gdiver2);
+  if (gcmp0(y) && ty != t_MAT) err(gdiver);
 
   if (is_const_t(tx) && is_const_t(ty))
   {
@@ -1892,7 +1894,7 @@ gdiv(GEN x, GEN y)
 
 	  case t_INTMOD: z=cgetg(3,t_INTMOD); p2=(GEN)x[1]; p1=(GEN)y[1];
 	    if (p1==p2 || egalii(p1,p2))
-            { icopyifstack(p2,z[1]); }
+              icopyifstack(p2,z[1]);
             else
             { p2 = mppgcd(p1,p2); z[1] = (long)p2; }
             av=avma; (void)new_chunk(lgefint(x[1])+(lgefint(p1)<<1)); /* HACK */
@@ -2228,7 +2230,7 @@ gdiv(GEN x, GEN y)
               y_lead = (GEN)y[2]; ly--; l--;
               if (!gcmp0(y_lead)) break;
             }
-            if (i>=ly) err(gdiver2);
+            if (i>=ly) err(gdiver);
           }
 	  if (ly < lx) lx = ly;
 	  p2 = (GEN)gpmalloc(lx*sizeof(long));
