@@ -1,4 +1,4 @@
-/* $Id: gen2.c,v 1.51 2002/06/03 09:14:46 karim Exp $
+/* $Id: gen2.c,v 1.53 2002/09/11 22:29:31 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -501,6 +501,18 @@ vecegal(GEN x, GEN y)
   return 1;
 }
 
+static int
+gegal_try(GEN x, GEN y)
+{
+  int i;
+  CATCH(CATCH_ALL) {
+    CATCH_RELEASE(); return 0;
+  } TRY {
+    i = gcmp0(gadd(x, gneg_i(y)));
+  } ENDCATCH;
+  return i;
+}
+
 int
 gegal(GEN x, GEN y)
 {
@@ -552,20 +564,8 @@ gegal(GEN x, GEN y)
           if (x[i] != y[i]) return 0;
         return 1;
     }
-  {
-    VOLATILE gpmem_t av = avma;
-    jmp_buf env;
-    void *c;
-    if (setjmp(env)) i = 0;
-    else
-    {
-      c = err_catch(-1, env, NULL);
-      i = gcmp0(gadd(x, gneg_i(y)));
-    }
-    err_leave(&c);
-    avma = av;
-  }
-  return i;
+  av = avma; i = gegal_try(x, y);
+  avma = av; return i;
 }
 #undef MASK
 
