@@ -1,4 +1,4 @@
-/* $Id: gen3.c,v 1.72 2002/06/09 16:35:39 karim Exp $
+/* $Id: gen3.c,v 1.76 2002/09/12 18:37:01 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -500,7 +500,7 @@ gdivgs(GEN x, long s)
   gpmem_t av;
   GEN z,p1;
 
-  if (!s) err(gdiver2);
+  if (!s) err(gdiver);
   switch(tx)
   {
     case t_INT:
@@ -683,7 +683,7 @@ gmod(GEN x, GEN y)
       {
 	case t_INT: case t_REAL: case t_FRAC: case t_FRACN:
 	  av = avma;
-          return gerepileupto(av, gsub(x, gmul(quot(x,y),y)));
+          return gerepileupto(av, gsub(x, gmul(_quot(x,y),y)));
 
 	case t_POLMOD: case t_POL:
 	  return gzero;
@@ -1200,7 +1200,7 @@ ginv(GEN x)
   {
     case t_INT:
       if (is_pm1(x)) return icopy(x);
-      if (!signe(x)) err(gdiver2);
+      if (!signe(x)) err(gdiver);
       z=cgetg(3,t_FRAC);
       z[1] = (signe(x)<0)? lnegi(gun): un;
       z[2]=labsi(x); return z;
@@ -1214,7 +1214,7 @@ ginv(GEN x)
 
     case t_FRAC: case t_FRACN:
       s = signe(x[1]);
-      if (!s) err(gdiver2);
+      if (!s) err(gdiver);
       if (is_pm1(x[1]))
         return s>0? icopy((GEN)x[2]): negi((GEN)x[2]);
       z = cgetg(3,tx);
@@ -1232,7 +1232,7 @@ ginv(GEN x)
       return gerepile(av,tetpil,gdiv(p2,p1));
 
     case t_PADIC: z = cgetg(5,t_PADIC);
-      if (!signe(x[4])) err(gdiver2);
+      if (!signe(x[4])) err(gdiver);
       z[1] = evalprecp(precp(x)) | evalvalp(-valp(x));
       icopyifstack(x[2], z[2]);
       z[3] = licopy((GEN)x[3]);
@@ -1246,7 +1246,7 @@ ginv(GEN x)
       return gdiv(gun,x);
 
     case t_RFRAC: case t_RFRACN:
-      if (gcmp0((GEN) x[1])) err(gdiver2);
+      if (gcmp0((GEN) x[1])) err(gdiver);
       p1 = fix_rfrac_if_pol((GEN)x[2],(GEN)x[1]);
       if (p1) return p1;
       z=cgetg(3,tx);
@@ -1896,8 +1896,7 @@ ground(GEN x)
       return gerepileuptoint(av, mpent(p1));
     }
     case t_FRAC: case t_FRACN:
-      av=avma; p1 = addii(shifti((GEN)x[2], -1), (GEN)x[1]);
-      return gerepileuptoint(av, truedvmdii(p1, (GEN)x[2], NULL));
+      return diviiround((GEN)x[1], (GEN)x[2]);
 
     case t_POLMOD: y=cgetg(3,t_POLMOD);
       copyifstack(x[1],y[1]);
@@ -2366,18 +2365,12 @@ compo(GEN x, long n)
   return gcopy((GEN)x[l]);
 }
 
-/* assume x a t_POL */
+/* assume v > varn(x), extract coeff of polx[v]^n */
 static GEN
-_polcoeff(GEN x, long n, long v)
+multi_coeff(GEN x, long n, long v, long dx)
 {
-  long i, w, dx;
+  long i;
   GEN z;
-  dx = degpol(x);
-  if (dx < 0) return gzero;
-  if (v < 0 || v == (w=varn(x)))
-    return (n < 0 || n > dx)? gzero: (GEN)x[n+2];
-  if (w > v) return n? gzero: x;
-  /* w < v */
   if (dx == 0) return polcoeff_i((GEN)x[2], n, v);
   z = cgetg(dx+3, t_POL);
   z[1] = x[1];
@@ -2385,13 +2378,25 @@ _polcoeff(GEN x, long n, long v)
   return normalizepol(z);
 }
 
+/* assume x a t_POL */
+static GEN
+_polcoeff(GEN x, long n, long v)
+{
+  long w, dx;
+  dx = degpol(x);
+  if (dx < 0) return gzero;
+  if (v < 0 || v == (w=varn(x)))
+    return (n < 0 || n > dx)? gzero: (GEN)x[n+2];
+  if (w > v) return n? gzero: x;
+  /* w < v */
+  return multi_coeff(x, n, v, dx);
+}
+
 /* assume x a t_SER */
 static GEN
 _sercoeff(GEN x, long n, long v)
 {
-  long i, w, dx, ex;
-  GEN z;
- 
+  long w, dx, ex;
   if (!signe(x)) return gzero;
   dx = lg(x)-3; ex = valp(x); n -= ex;
   if (v < 0 || v == (w=varn(x)))
@@ -2401,26 +2406,23 @@ _sercoeff(GEN x, long n, long v)
   }
   if (w > v) return n? gzero: x;
   /* w < v */
-  if (dx == 0) return polcoeff_i((GEN)x[2], n, v);
-  z = cgetg(dx+3, t_POL);
-  z[1] = x[1];
-  for (i=2; i<dx+3; i++) z[i] = (long)polcoeff_i((GEN)x[i], n, v);
-  return normalizepol(z);
+  return multi_coeff(x, n, v, dx);
 }
 
 /* assume x a t_RFRAC(n) */
 static GEN
 _rfraccoeff(GEN x, long n, long v)
 {
-  long ex, w = precdl;
-  if (v<0) v = gvar(x);
-  ex = ggval((GEN)x[2], polx[v]);
-  precdl = n + ex + 1; x = gtoser(x, v);
-  precdl = w;
-  return _sercoeff(x,n,v);
+  GEN P,Q, p = (GEN)x[1], q = (GEN)x[2];
+  long vp = gvar(p), vq = gvar(q);
+  if (v < 0) v = min(vp, vq);
+  P = (vp == v)? p: swap_vars(p, v);
+  Q = (vq == v)? q: swap_vars(q, v);
+  if (!ismonome(Q)) err(typeer, "polcoeff");
+  n += degpol(Q);
+  return gdiv(_polcoeff(P, n, v), leading_term(Q));
 }
 
-
 GEN
 polcoeff_i(GEN x, long n, long v)
 {
@@ -2428,8 +2430,8 @@ polcoeff_i(GEN x, long n, long v)
   {
     case t_POL: return _polcoeff(x,n,v);
     case t_SER: return _sercoeff(x,n,v);
-    case t_RFRAC:
-    case t_RFRACN: return _rfraccoeff(x,n,v);
+    case t_RFRACN: err(typeer, "polcoeff"); /* too expensive to reduce */
+    case t_RFRAC: return _rfraccoeff(x,n,v);
     default: return n? gzero: x;
   }
 }
