@@ -1,4 +1,4 @@
-/* $Id: polarit1.c,v 1.110 2002/06/08 15:33:12 karim Exp $
+/* $Id: polarit1.c,v 1.114 2002/09/11 02:29:15 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -655,7 +655,7 @@ FpV_roots_to_pol(GEN V, GEN p, long v)
   long i;
   GEN g=cgetg(lg(V),t_VEC);
   for(i=1;i<lg(V);i++)
-    g[i]=(long)deg1pol(gun,negi((GEN)V[i]),v);
+    g[i]=(long)deg1pol(gun,modii(negi((GEN)V[i]),p),v);
   modulo=p;
   g=divide_conquer_prod(g,&gsmul);
   return gerepileupto(ltop,g);
@@ -950,6 +950,79 @@ vec_to_pol(GEN x, long v)
   return p;
 }
 
+GEN
+u_vec_to_pol(GEN x)
+{
+  long i, k = lg(x);
+  GEN p;
+
+  while (--k && !x[k]);
+  if (!k) return u_zeropol();
+  i = k+2; p = cgetg(i,t_POL);
+  p[1] = evalsigne(1) | evallgef(i) | evalvarn(0);
+  x--; for (k=2; k<i; k++) p[k] = x[k];
+  return p;
+}
+
+#if 0
+static GEN
+u_FpM_FpV_mul(GEN x, GEN y, ulong p)
+{
+  long i,k,l,lx=lg(x), ly=lg(y);
+  GEN z;
+  if (lx != ly) err(operi,"* [mod p]",x,y);
+  if (lx==1) return cgetg(1,t_VECSMALL);
+  l = lg(x[1]);
+  z = cgetg(l,t_VECSMALL);
+  for (i=1; i<l; i++)
+  {
+    ulong p1 = 0;
+    for (k=1; k<lx; k++)
+    {
+      p1 += coeff(x,i,k) * y[k];
+      if (p1 & HIGHBIT) p1 %= p; 
+    }
+    z[i] = p1 % p;
+  }
+  return z;
+}
+#endif
+
+/* u_vec_to_pol(u_FpM_FpV_mul(x, u_pol_to_vec(y), p)) */
+GEN
+u_FpM_FpX_mul(GEN x, GEN y, ulong p)
+{
+  long i,k,l, ly=lgef(y)-1;
+  GEN z;
+  if (ly==1) return u_zeropol();
+  l = lg(x[1]);
+  y++;
+  z = vecsmall_const(l,0) + 1;
+  for (k=1; k<ly; k++)
+  {
+    GEN c;
+    if (!y[k]) continue;
+    c = (GEN)x[k];
+    if (y[k] == 1)
+      for (i=1; i<l; i++)
+      {
+        z[i] += c[i];
+        if (z[i] & HIGHBIT) z[i] %= p; 
+      }
+    else
+      for (i=1; i<l; i++)
+      {
+        z[i] += c[i] * y[k];
+        if (z[i] & HIGHBIT) z[i] %= p; 
+      }
+  }
+  for (i=1; i<l; i++) z[i] %= p;
+  while (--l && !z[l]);
+  if (!l) return u_zeropol();
+  *z-- = evalsigne(1) | evallgef(l+2) | evalvarn(0);
+  return z;
+}
+
 /* return the (N-dimensional) vector of coeffs of p */
 GEN
 pol_to_vec(GEN x, long N)
@@ -968,6 +1041,19 @@ pol_to_vec(GEN x, long N)
   return z;
 }
 
+/* return the (N-dimensional) vector of coeffs of p */
+GEN
+u_pol_to_vec(GEN x, long N)
+{
+  long i, l;
+  GEN z = cgetg(N+1,t_VECSMALL);
+  if (typ(x) != t_VECSMALL) err(typeer,"u_pol_to_vec");
+  l = lgef(x)-1; x++;
+  for (i=1; i<l ; i++) z[i]=x[i];
+  for (   ; i<=N; i++) z[i]=0;
+  return z;
+}
+
 /* vector of polynomials (in v) whose coeffs are given by the columns of x */
 GEN
 mat_to_vecpol(GEN x, long v)
@@ -1110,7 +1196,7 @@ FpX_split_berlekamp(GEN *t, GEN p)
   po2 = shifti(p, -1); /* (p-1) / 2 */
   pol = cgetg(N+3,t_POL);
   ir = 0;
-  /* t[i] irreducible for i < ir, still to be treated for i < L */
+  /* t[i] irreducible for i <= ir, still to be treated for i < L */
   for (L=1; L<d; )
   {
     GEN polt;
@@ -1798,7 +1884,7 @@ apprgen9(GEN f, GEN a)
     ps_1=itos(p)-1; x2=ggrandocp(p,1);
   }
   f = poleval(f,gadd(a,gmul(p,polx[varn(f)])));
-  f = gdiv(f,gpuigs(p,ggval(f,p)));
+  f = gdiv(f,gpowgs(p,ggval(f,p)));
 
   d=degpol(T); vecg=cgetg(d+1,t_COL);
   for (i=1; i<=d; i++)
@@ -1848,34 +1934,32 @@ cmp_padic(GEN x, GEN y)
 /*   Using Buchmann--Lenstra   */
 /*******************************/
 
-/* factor nf[1] in Zp to precision pr */
+/* factor T = nf[1] in Zp to precision k */
 static GEN
-padicff2(GEN nf,GEN p,long pr)
+padicff2(GEN nf,GEN p,long k)
 {
-  long N=degpol(nf[1]),i,j,d,l;
-  GEN mat,V,D,fa,p1,pk,dec_p,pke,a,theta;
+  GEN z, mat, D, U,fa, pk, dec_p, Ui, mulx;
+  long i, l;
 
-  pk=gpuigs(p,pr); dec_p=primedec(nf,p);
-  l=lg(dec_p); fa=cgetg(l,t_COL);
+  mulx = eltmul_get_table(nf, gmael(nf,8,2)); /* mul by (x mod T) */
+  dec_p = primedec(nf,p);
+  l = lg(dec_p); fa = cgetg(l,t_COL);
+  D = NULL; /* -Wall */
   for (i=1; i<l; i++)
   {
-    p1 = (GEN)dec_p[i];
-    pke = idealpows(nf,p1, pr * itos((GEN)p1[3]));
-    V = smithall(pke, &D, NULL);
-    for (d=1; d<=N; d++)
-      if (! egalii(gcoeff(V,d,d),pk)) break;
-    a=ginv(D); theta=gmael(nf,8,2); mat=cgetg(d,t_MAT);
-    for (j=1; j<d; j++)
-    {
-      p1 = gmul(D, element_mul(nf,theta,(GEN)a[j]));
-      setlg(p1,d); mat[j]=(long)p1;
-    }
-    fa[i]=(long)caradj(mat,0,NULL);
+    GEN P = (GEN)dec_p[i];
+    long e = itos((GEN)P[3]), ef = e * itos((GEN)P[4]);
+    D = smithall(idealpows(nf,P, k*e), &U, NULL);
+    Ui= ginv(U); setlg(Ui, ef+1); /* cf smithrel */
+    U = rowextract_i(U, 1, ef);
+    mat = gmul(U, gmul(mulx, Ui));
+    fa[i] = (long)caradj(mat,0,NULL);
   }
-  a = cgetg(l,t_COL); pk = icopy(pk);
+  pk = gcoeff(D,1,1); /* = p^k */
+  z = cgetg(l,t_COL); pk = icopy(pk);
   for (i=1; i<l; i++)
-    a[i] = (long)pol_to_padic((GEN)fa[i],pk,p,pr);
-  return a;
+    z[i] = (long)pol_to_padic((GEN)fa[i],pk,p,k);
+  return z;
 }
 
 static GEN
@@ -1890,7 +1974,7 @@ padicff(GEN x,GEN p,long pr)
   g = cgetg(3,t_COL); fa[1] = (long)g;
   e = cgetg(3,t_COL); fa[2] = (long)e;
   dx = ZX_disc(x);
-  g[1] = (long)p; e[1] = lstoi(pvaluation(dx,p,&q));
+  g[1] = (long)p; e[1] = lstoi(pvaluation(absi(dx),p,&q));
   g[2] = (long)q; e[2] = un;
 
   bas = nfbasis(x, &dK, 0, fa);
@@ -2286,7 +2370,7 @@ factmod9(GEN f, GEN p, GEN T)
   frobinv = gpowgs(p, degpol(T)-1);
 
   X = polx[vf];
-  q = gpuigs(p, degpol(T));
+  q = gpowgs(p, degpol(T));
   e = nbfact = 1;
   pk = 1;
   f3 = NULL;
