@@ -1,4 +1,4 @@
-/* $Id: trans1.c,v 1.68 2002/06/08 14:48:35 karim Exp $
+/* $Id: trans1.c,v 1.70 2002/09/12 18:37:01 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1174,6 +1174,14 @@ gsqrtn(GEN x, GEN n, GEN *zetan, long prec)
     if (tx==t_INT && is_pm1(x) && signe(x)>0)
       y=gun;    /*speed-up since there is no way to call rootsof1complex
 		directly from gp*/
+    else if (gcmp0(x))
+    {
+      if (gsigne(n) < 0) err(gdiver);
+      if (isinexactreal(x))
+        y = realzero_bit( itos( gfloor(gdivsg(gexpo(x), n)) ) );
+      else
+        y = realzero(prec);
+    }
     else
     {
       av=avma;
