@@ -1,4 +1,4 @@
-/* $Id: gp.h,v 1.9 2002/04/27 18:42:13 karim Exp $
+/* $Id: gp.h,v 1.10 2002/06/12 00:24:11 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -25,7 +25,7 @@ int term_height(void);
 int term_width(void);
 void hit_return(void);
 
-extern ulong secure;
+extern int secure;
 extern ulong init_opts;
 extern char *current_logfile;
 extern jmp_buf environnement;
