@@ -1,4 +1,4 @@
-/* $Id: gp_rl.c,v 1.15 2002/03/11 21:05:58 karim Exp $
+/* $Id: gp_rl.c,v 1.18 2002/09/05 11:38:39 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -103,7 +103,7 @@ change_state(char *msg, int *opt, int count)
   }
   *opt = c;
   SAVE_PROMPT();
-  rl_message("%s: %s.", (long)msg, (long)(c? "on": "off"));
+  rl_message("%s: %s.", msg, c? "on": "off");
   c = rl_read_key();
   RESTORE_PROMPT();
   rl_clear_message();
@@ -141,7 +141,7 @@ pari_rl_matched_insert(int count, int key)
   if (count <= 0)
     return change_state("electric parens", &do_matched_insert, count);
   while (paropen[i] && paropen[i] != key) i++;
-  if (!paropen[i] || !do_matched_insert || under_emacs)
+  if (!paropen[i] || !do_matched_insert || GP_DATA->flags & EMACS)
     return ((RLCI)rl_insert)(count,key);
   rl_begin_undo_group();
   ((RLCI)rl_insert)(count,key);
@@ -319,7 +319,7 @@ get_matches(int end, char *text, char* f(char*,int))
     else if (add_comma(end))
       match_concat(matches,",");
   }
-  if (under_emacs) matches = matches_for_emacs(text,matches);
+  if (GP_DATA->flags & EMACS) matches = matches_for_emacs(text,matches);
   return matches;
 }
 #undef add_comma
@@ -460,7 +460,7 @@ rl_print_aide(char *s, int flag)
 
   rl_point = 0; rl_end = 0; pari_outfile = rl_outstream;
   SAVE_PROMPT();
-  rl_message("",0,0);
+  rl_message("%s",""); /* rl_message("") ==> "zero length format" warning */
   aide(s, flag);
   RESTORE_PROMPT();
   rl_point = p; rl_end = e; pari_outfile = save;
@@ -620,7 +620,7 @@ init_readline(void)
   rl_attempted_completion_function = (rl_completion_func_t*) pari_completion;
 
   /* we always want the whole list of completions under emacs */
-  if (under_emacs) rl_completion_query_items = 0x8fff;
+  if (GP_DATA->flags & EMACS) rl_completion_query_items = 0x8fff;
 
 #define Bind(a,b,c) (((void(*)(int,Function*,Keymap)) rl_bind_key_in_map)\
   ((a), (Function*)(b), (c)))
