@@ -1,4 +1,4 @@
-/* $Id: plotgnuplot.c,v 1.9 2002/06/09 18:49:11 karim Exp $
+/* $Id: plotgnuplot.c,v 1.10 2002/06/29 09:31:16 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -470,4 +470,13 @@ get_term_ftable_get(void) /* Establish runtime link with gnuplot engine */
     croak(buf);
     return 0;
 }
-#endif
+#else	/* !( defined HAS_DLOPEN ) */
+
+get_term_ftable_t *
+get_term_ftable_get(void) /* Establish runtime link with gnuplot engine */
+{
+    croak("No dlopen() support present, required for dynamic gnuplot-DLL link");
+    return NULL;
+}
+
+#endif	/* defined HAS_DLOPEN */ 
