@@ -1,4 +1,4 @@
-/* $Id: plotport.c,v 1.17 2001/12/05 19:37:13 karim Exp $
+/* $Id: plotport.c,v 1.25 2002/06/09 18:49:11 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -31,6 +31,8 @@ static void PARI_get_psplot(void);
 static long current_color[NUMRECT];
 PariRect **rectgraph = NULL;
 PARI_plot pari_plot, pari_psplot;
+PARI_plot *pari_plot_engine = &pari_plot;
+PARI_plot pari_X11plot;			/* Used if BOTH_GNUPLOT_AND_X11 */
 long  rectpoint_itype = 0;
 long  rectline_itype  = 0;
 
@@ -108,7 +110,8 @@ static GEN quark_gen;
 void
 plot(entree *ep, GEN a, GEN b, char *ch,GEN ysmlu,GEN ybigu, long prec)
 {
-  long av = avma, av2,limite,jz,j,i,sig;
+  long jz, j, i, sig;
+  gpmem_t av = avma, av2, limite;
   int jnew, jpre = 0; /* for lint */
   GEN p1,p2,ysml,ybig,x,diff,dyj,dx,y[ISCR+1];
   screen scr;
@@ -137,7 +140,7 @@ plot(entree *ep, GEN a, GEN b, char *ch,GEN ysmlu,GEN ybigu, long prec)
     x = addrr(x,dx);
     if (low_stack(limite, stack_lim(av2,1)))
     {
-      long tetpil=avma;
+      gpmem_t tetpil=avma;
       if (DEBUGMEM>1) err(warnmem,"plot");
       x = gerepile(av2,tetpil,rcopy(x));
     }
@@ -207,6 +210,8 @@ free_graph(void)
 {
   int i;
 
+  if (!rectgraph)
+      return;
   for (i=0; i<NUMRECT; i++)
   {
     PariRect *e=rectgraph[i];
@@ -215,13 +220,16 @@ free_graph(void)
     free((void *)e);
   }
   free((void *)rectgraph);
+  rectgraph = 0;
 }
 
 static PariRect *
 check_rect(long ne)
 {
   if (!GOODRECT(ne))
-    err(talker,"not an rplot vector type in graphic function");
+    err(talker,
+        "incorrect rectwindow number in graphic function (%ld not in [0, %ld])",
+        ne, NUMRECT-1);
   return rectgraph[ne];
 }
 
@@ -229,8 +237,7 @@ static PariRect *
 check_rect_init(long ne)
 {
   PariRect *e = check_rect(ne);
-  if (!RHead(e))
-    err(talker,"you must initialize the rectwindow first");
+  if (!RHead(e)) err(talker,"you must initialize the rectwindow first");
   return e;
 }
 
@@ -1192,81 +1199,77 @@ gtodblList(GEN data, long flags)
 
   if (! is_vec_t(tx)) err(talker,"not a vector in gtodblList");
   if (!nl) return NULL;
-  lx1=lg(data[1]);
+  lx1 = lg(data[1]);
 
+  if (nl == 1) err(talker,"single vector in gtodblList");
   /* Allocate memory, then convert coord. to double */
-  l=(dblPointList*) gpmalloc(nl*sizeof(dblPointList));
+  l = (dblPointList*) gpmalloc(nl*sizeof(dblPointList));
   for (i=0; i<nl-1; i+=2)
   {
-    u=i+1;
-    x=(GEN)data[u]; tx = typ(x);
-    y=(GEN)data[u+1]; ty = typ(y);
+    u = i+1;
+    x = (GEN)data[u];   tx = typ(x);
+    y = (GEN)data[u+1]; ty = typ(y);
     if (!is_vec_t(tx) || !is_vec_t(ty)) err(ploter4);
-    lx=lg(x); if (lg(y)!=lx) err(ploter5);
+    lx = lg(x); if (lg(y) != lx) err(ploter5);
     if (!param && lx != lx1) err(ploter5);
+
     lx--;
-
-    if (lx)
+    l[i].d = (double*) gpmalloc(lx*sizeof(double));
+    l[u].d = (double*) gpmalloc(lx*sizeof(double));
+    for (j=0; j<lx; j=v)
     {
-      l[i].d = (double*) gpmalloc(lx*sizeof(double));
-      l[u].d = (double*) gpmalloc(lx*sizeof(double));
-
-      for (j=0; j<lx; j=v)
-      {
-	v=j+1;
-	l[i].d[j]=gtodouble((GEN)x[v]);
-	l[u].d[j]=gtodouble((GEN)y[v]);
-      }
+      v = j+1;
+      l[i].d[j] = gtodouble((GEN)x[v]);
+      l[u].d[j] = gtodouble((GEN)y[v]);
     }
-    l[i].nb=l[u].nb=lx;
+    l[i].nb = l[u].nb = lx;
   }
 
-  xsml=xbig=l[0].d[0]; ysml=ybig=l[1].d[0];
-
   /* Now compute extremas */
   if (param)
   {
-    l[0].nb=nl/2;
-    for (i=0; i<l[0].nb; i+=2)
-    {
-      u=i+1;
-      for (j=0; j<l[u].nb; j++)
-      {
-	if (l[i].d[j]<xsml)
-	  xsml=l[i].d[j];
-	else
-	  if (l[i].d[j]>xbig) xbig=l[i].d[j];
+    l[0].nb = nl/2;
+    for (i=0; i < l[0].nb; i+=2)
+      if (l[i+1].nb) break;
+    if (i >= l[0].nb) { free(l); return NULL; }
+    xsml = xbig = l[i  ].d[0];
+    ysml = ybig = l[i+1].d[0];
 
-	if (l[u].d[j]<ysml)
-	  ysml=l[u].d[j];
-	else
-	  if (l[u].d[j]>ybig) ybig=l[u].d[j];
+    for (i=0; i < l[0].nb; i+=2)
+    {
+      u = i+1;
+      for (j=0; j < l[u].nb; j++)
+      {
+	if      (l[i].d[j] < xsml) xsml = l[i].d[j];
+	else if (l[i].d[j] > xbig) xbig = l[i].d[j];
+
+	if      (l[u].d[j] < ysml) ysml = l[u].d[j];
+	else if (l[u].d[j] > ybig) ybig = l[u].d[j];
       }
     }
   }
   else
   {
-    l[0].nb=nl-1;
-    for (j=0; j<l[1].nb; j++)
+    if (!l[0].nb) { free(l); return NULL; }
+    l[0].nb = nl-1;
+
+    xsml = xbig = l[0].d[0];
+    ysml = ybig = l[1].d[0];
+
+    for (j=0; j < l[1].nb; j++)
     {
-      if (l[0].d[j]<xsml)
-	xsml=l[0].d[j];
-      else
-	if (l[0].d[j]>xbig) xbig=l[0].d[j];
+      if      (l[0].d[j] < xsml) xsml = l[0].d[j];
+      else if (l[0].d[j] > xbig) xbig = l[0].d[j];
     }
-    for (i=1; i<=l[0].nb; i++)
-    {
-      for (j=0; j<l[i].nb; j++)
+    for (i=1; i <= l[0].nb; i++)
+      for (j=0; j < l[i].nb; j++)
       {
-	if (l[i].d[j]<ysml)
-	  ysml=l[i].d[j];
-	else
-	  if (l[i].d[j]>ybig) ybig=l[i].d[j];
+	if      (l[i].d[j] < ysml) ysml = l[i].d[j];
+	else if (l[i].d[j] > ybig) ybig = l[i].d[j];
       }
-    }
   }
-  l[0].xsml=xsml; l[0].xbig=xbig;
-  l[0].ysml=ysml; l[0].ybig=ybig;
+  l[0].xsml = xsml; l[0].xbig = xbig;
+  l[0].ysml = ysml; l[0].ybig = ybig;
   return l;
 }
 
@@ -1275,7 +1278,7 @@ single_recursion(dblPointList *pl,char *ch,entree *ep,GEN xleft,GEN yleft,
   GEN xright,GEN yright,long depth)
 {
   GEN xx,yy;
-  long av=avma;
+  gpmem_t av=avma;
   double dy=pl[0].ybig - pl[0].ysml;
 
   if (depth==RECUR_MAXDEPTH) return;
@@ -1303,7 +1306,7 @@ param_recursion(dblPointList *pl,char *ch,entree *ep, GEN tleft,GEN xleft,
   GEN yleft, GEN tright,GEN xright,GEN yright, long depth)
 {
   GEN tt,xx,yy, p1;
-  long av=avma;
+  gpmem_t av=avma;
   double dy=pl[0].ybig - pl[0].ysml;
   double dx=pl[0].xbig - pl[0].xsml;
 
@@ -1342,7 +1345,8 @@ rectplothin(entree *ep, GEN a, GEN b, char *ch, long prec, ulong flags,
   long recur=flags & PLOT_RECURSIVE;
   GEN p1,dx,x,xleft,xright,yleft,yright,tleft,tright;
   dblPointList *pl;
-  long tx,av = avma,av2,i,j,sig,nc,nl,nbpoints;
+  long tx, i, j, sig, nc, nl, nbpoints;
+  gpmem_t av = avma, av2;
   double xsml,xbig,ysml,ybig,fx,fy;
 
   if (!testpoints)
@@ -1510,7 +1514,8 @@ extern GEN polint_i(GEN xa, GEN ya, GEN x, long n, GEN *ptdy);
 static void
 rectsplines(long ne, double *x, double *y, long lx, long flag)
 {
-  long i, j, oldavma = avma;
+  long i, j;
+  gpmem_t oldavma = avma;
   GEN tas, xa = cgetg(lx+1, t_VEC), ya = cgetg(lx+1, t_VEC);
   entree *var0 = varentries[ordvar[0]];
 
@@ -1527,7 +1532,7 @@ rectsplines(long ne, double *x, double *y, long lx, long flag)
   }
   else tas = NULL; /* for lint */
   for (i = 0; i <= lx - 4; i++) {
-      long oavma = avma;
+      gpmem_t oavma = avma;
 
       xa++; ya++;
       if (flag & PLOT_PARAMETRIC) {
@@ -1584,7 +1589,8 @@ rectplothrawin(long stringrect, long drawrect, dblPointList *data,
   GEN res;
   dblPointList y,x;
   double xsml,xbig,ysml,ybig,tmp;
-  long ltype, ltop=avma;
+  long ltype;
+  gpmem_t ltop=avma;
   long i,nc,nbpoints, w[2], wx[2], wy[2];
 
   w[0]=stringrect; w[1]=drawrect;
@@ -1769,7 +1775,7 @@ static GEN
 plothraw0(long stringrect, long drawrect, GEN listx, GEN listy, long flags)
 {
   PARI_plot *output = init_output(flags);
-  long data[] = {evaltyp(t_VEC) | m_evallg(3), 0, 0};
+  long data[] = {evaltyp(t_VEC) | _evallg(3), 0, 0};
   dblPointList *pl;
 
   data[1] = (long) listx;
