@@ -1,4 +1,4 @@
-/* $Id: paricom.h,v 1.22 2002/04/10 23:29:53 karim Exp $
+/* $Id: paricom.h,v 1.31 2002/09/12 18:45:06 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -18,6 +18,44 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /*              PARI header file (common to all versions)         */
 /*                                                                */
 /******************************************************************/
+#ifdef STMT_START /* perl headers */
+#  undef STMT_START
+#endif
+#ifdef STMT_END
+#  undef STMT_END
+#endif
+/* STMT_START { statements; } STMT_END;
+ * can be used as a single statement, as in
+ * if (x) STMT_START { ... } STMT_END; else ...
+ * [ avoid "dangling else" problem in macros ] */
+#define STMT_START	do
+#define STMT_END	while (0)
+/*=====================================================================*/
+/* CATCH(numer) {
+ *   recovery 
+ * } TRY {
+ *   code
+ * } ENDCATCH
+ * will execute 'code', then 'recovery' if exception 'numer' is thrown
+ * [ any exception if numer < 0 ].
+ * RETRY = as TRY, but execute 'recovery', then 'code' again [still catching] */
+#define CATCH(err) {         \
+  VOLATILE long __err = err; \
+  int pari_errno;            \
+  jmp_buf __env;             \
+  void *__catcherr;          \
+  if ((pari_errno = setjmp(__env))) 
+
+#define RETRY { __catcherr = err_catch(__err, &__env); {
+#define TRY else RETRY
+
+/* Take address of __catcher to prevent compiler from putting it into a register
+ * (could be clobbered by longjmp otherwise) */
+#define CATCH_RELEASE() err_leave(&__catcherr)
+#define ENDCATCH }} CATCH_RELEASE(); }
+
+#define CATCH_ALL -1
+/*=====================================================================*/
 
 #define bit_accuracy(x) (((x)-2) << TWOPOTBITS_IN_LONG)
 
@@ -39,8 +77,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #define absi  mpabs
 #define negi  mpneg
 #define negr  mpneg
-#define mpnegz(x,y) {gpmem_t av=avma;mpaff(mpneg(x),y);avma=av;}
-#define mpabsz(x,y) {gpmem_t av=avma;mpaff(mpabs(x),y);avma=av;}
+#define mpnegz(x,y) \
+  STMT_START {gpmem_t av=avma;mpaff(mpneg(x),y);avma=av;} STMT_END
+#define mpabsz(x,y) \
+  STMT_START {gpmem_t av=avma;mpaff(mpabs(x),y);avma=av;} STMT_END
 #define absrz(x,z)  mpabsz((x),(z))
 #define negrz(x,z)  mpnegz((x),(z))
 
@@ -58,6 +98,8 @@ extern GEN   gun,gdeux,ghalf,gi,gzero;
 extern const long lontyp[];
 extern void* global_err_data;
 
+extern int new_galois_format;
+
 #define MAXITERPOL  10 /* max #of prec increase in polredabs-type operations */
 
                                                 /* let SL = sizeof(long) */
@@ -277,27 +319,9 @@ extern void* global_err_data;
 #define element_mulmodideal(nf,x,y,ideal) (\
   nfreducemodideal((nf),element_mul((nf),(x),(y)),(ideal))\
 )
-#define element_mulmodpr(nf,x,y,prhall) (\
-  nfreducemodpr((nf),element_mul((nf),(x),(y)),(prhall))\
-)
 #define element_sqrmodideal(nf,x,ideal) (\
   nfreducemodideal((nf),element_sqr((nf),(x)),(ideal))\
 )
-#define element_sqrmodpr(nf,x,prhall) (\
-  nfreducemodpr((nf),element_sqr((nf),(x)),(prhall))\
-)
-#define buchgen(P,gcbach,gcbach2,prec) (\
-  buchall((P),(gcbach),(gcbach2),stoi(5),gzero,4,3,0,(prec))\
-)
-#define buchgenfu(P,gcbach,gcbach2,prec) (\
-  buchall((P),(gcbach),(gcbach2),stoi(5),gzero,4,3,2,(prec))\
-)
-#define buchinit(P,gcbach,gcbach2,prec) (\
-  buchall((P),(gcbach),(gcbach2),stoi(5),gzero,4,3,-1,(prec))\
-)
-#define buchinitfu(P,gcbach,gcbach2,prec) (\
-  buchall((P),(gcbach),(gcbach2),stoi(5),gzero,4,3,-2,(prec))\
-)
 
 /* output of get_nf and get_bnf */
 #define typ_NULL 0
@@ -315,3 +339,9 @@ extern void* global_err_data;
 #define cmp_LEX 2
 #define cmp_REV 4
 #define cmp_C   8
+
+#define DIFFPTR_SKIP	255		/* Skip these entries */
+#define NEXT_PRIME_VIADIFF(p,d)	\
+  { while (*(d) == DIFFPTR_SKIP) (p) += *(d)++; (p) += *(d)++; }
+#define NEXT_PRIME_VIADIFF_CHECK(p,d) \
+  { if (!*(d)) err(primer1); NEXT_PRIME_VIADIFF(p,d) }
