@@ -1,4 +1,4 @@
-/* $Id: paridecl.h,v 1.203 2002/06/09 00:22:41 karim Exp $
+/* $Id: paridecl.h,v 1.226 2002/09/12 18:45:06 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -20,25 +20,27 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 /*******************************************************************/
 BEGINEXTERN
 /* alglin1.c */
-GEN     _col(GEN x);
-GEN     _vec(GEN x);
-GEN     _veccopy(GEN x);
 GEN     FpM_FpV_mul(GEN x, GEN y, GEN p);
 GEN     FpM_deplin(GEN x, GEN p);
 GEN     FpM_image(GEN x, GEN p);
 GEN     FpM_intersect(GEN x, GEN y, GEN p);
-GEN     FpM_invimage(GEN m, GEN v, GEN p);
 GEN     FpM_inv(GEN x, GEN p);
+GEN     FpM_invimage(GEN m, GEN v, GEN p);
 GEN     FpM_ker(GEN x, GEN p);
 GEN     FpM_mul(GEN x, GEN y, GEN p);
 GEN     FpM_sindexrank(GEN x, GEN p);
 GEN     FpM_suppl(GEN x, GEN p);
-GEN     Fq_mul(GEN x, GEN y, GEN T, GEN p);
 GEN     FqM_gauss(GEN a, GEN b, GEN T, GEN p);
 GEN     FqM_ker(GEN x, GEN T, GEN p);
 GEN     FqM_suppl(GEN x, GEN T, GEN p);
+GEN     Fq_mul(GEN x, GEN y, GEN T, GEN p);
 GEN     QM_inv(GEN M, GEN dM);
 GEN     ZM_inv(GEN M, GEN dM);
+GEN     _col(GEN x);
+GEN     _vec(GEN x);
+GEN     _veccopy(GEN x);
+void    appendL(GEN x, GEN t);
+GEN     cget1(long l, long t);
 GEN     concat(GEN x, GEN y);
 GEN     concatsp(GEN x, GEN y);
 GEN     concatsp3(GEN x, GEN y, GEN z);
@@ -64,8 +66,8 @@ GEN     gscalcol_proto(GEN z, GEN myzero, long n);
 GEN     gscalmat(GEN x, long n);
 GEN     gscalsmat(long x, long n);
 GEN     gtomat(GEN x);
-GEN     gtrans_i(GEN x);
 GEN     gtrans(GEN x);
+GEN     gtrans_i(GEN x);
 GEN     hnfadd(GEN mit,GEN perm,GEN* ptdep,GEN* ptA,GEN* ptC,GEN extramat,GEN extraC);
 GEN     hnfspec(long** mat,GEN perm,GEN* ptdep,GEN* ptA,GEN* ptC,long k0);
 GEN     idmat(long n);
@@ -73,7 +75,6 @@ GEN     idmat_intern(long n,GEN myun,GEN myzero);
 GEN     image(GEN x);
 GEN     image2(GEN x);
 GEN     imagecompl(GEN x);
-GEN     imagereel(GEN x, long prec);
 GEN     indexrank(GEN x);
 GEN     inverseimage(GEN mat, GEN y);
 long    isdiagonal(GEN x);
@@ -89,10 +90,14 @@ GEN     matsolvemod0(GEN M, GEN D, GEN Y,long flag);
 GEN     mattodiagonal(GEN m);
 GEN     mattodiagonal_i(GEN m);
 long    rank(GEN x);
+GEN     row(GEN A, long x1);
+GEN     row_i(GEN A, long x0, long x1, long x2);
 GEN     rowextract_i(GEN A, long x1, long x2);
 GEN     rowextract_p(GEN A, GEN p);
 GEN     sindexrank(GEN x);
+GEN     sum(GEN v, long a, long b);
 GEN     suppl(GEN x);
+GEN     vconcat(GEN A, GEN B);
 GEN     vecextract_i(GEN A, long y1, long y2);
 GEN     vecextract_p(GEN A, GEN p);
 GEN     zerocol(long n);
@@ -120,7 +125,6 @@ GEN     gtrace(GEN x);
 GEN     hess(GEN x);
 GEN     hnf(GEN x);
 GEN     hnfall(GEN x);
-GEN     hnfhavas(GEN x);
 GEN     hnflll(GEN x);
 GEN     hnflll_i(GEN A, GEN *ptB, int remove);
 GEN     hnfmod(GEN x, GEN detmat);
@@ -224,6 +228,7 @@ GEN     qfbclassno0(GEN x,long flag);
 GEN     quaddisc(GEN x);
 GEN     racine(GEN a);
 GEN     regula(GEN x, long prec);
+ulong   u_gener(ulong p);
 GEN     znstar(GEN x);
 
 /* arith2.c */
@@ -269,7 +274,7 @@ GEN     gomega(GEN n);
 GEN     gphi(GEN n);
 GEN     gsumdiv(GEN n);
 GEN     gsumdivk(GEN n,long k);
-byteptr initprimes(long maxnum);
+byteptr initprimes(ulong maxnum);
 long    issquarefree(GEN x);
 ulong   maxprime(void);
 long    mu(GEN n);
@@ -318,6 +323,7 @@ void    checkrnf(GEN rnf);
 GEN     galois(GEN x, long prec);
 GEN     galoisapply(GEN nf, GEN aut, GEN x);
 GEN     get_bnf(GEN x,int *t);
+GEN     get_bnfpol(GEN x, GEN *bnf, GEN *nf);
 GEN     get_nf(GEN x,int *t);
 GEN     get_nfpol(GEN x, GEN *nf);
 GEN     get_primeid(GEN x);
@@ -391,6 +397,7 @@ GEN     rnfsteinitz(GEN nf, GEN order);
 GEN     smallbase(GEN x, GEN *y);
 GEN     smalldiscf(GEN x);
 GEN     zk_to_ff_init(GEN nf, GEN *pr, GEN *T, GEN *p);
+GEN     zk_to_ff(GEN x, GEN modpr);
 GEN     zkmodprinit(GEN nf, GEN pr);
 
 /* base3.c */
@@ -412,7 +419,6 @@ GEN     element_powmodidele(GEN nf,GEN x,GEN k,GEN idele,GEN structarch);
 GEN     element_sqr(GEN nf,GEN x);
 GEN     element_sqri(GEN nf, GEN x);
 long    element_val(GEN nf, GEN x, GEN vp);
-long    element_val2(GEN nf, GEN x, GEN d, GEN vp);
 GEN     eltmul_get_table(GEN nf, GEN x);
 GEN     ideallist(GEN nf,long bound);
 GEN     ideallist0(GEN nf,long bound, long flag);
@@ -447,12 +453,13 @@ GEN     zidealstarinitjoin(GEN nf, GEN bid1, GEN bid2, long flag);
 GEN     zidealstarinitjoinarch(GEN nf, GEN bid1, GEN arch, long nba, long flag);
 GEN     znlog(GEN x, GEN g);
 GEN     zsigne(GEN nf,GEN alpha,GEN arch);
+GEN     zsigns(GEN nf,GEN alpha);
 
 /* base4.c */
 
 GEN     element_divmodpr(GEN nf, GEN x, GEN y, GEN modpr);
 GEN     element_invmodpr(GEN nf, GEN y, GEN modpr);
-GEN     element_mulmodpr2(GEN nf, GEN x, GEN y, GEN modpr);
+GEN     element_mulmodpr(GEN nf, GEN x, GEN y, GEN modpr);
 GEN     element_powmodpr(GEN nf, GEN x, GEN k, GEN modpr);
 GEN     element_reduce(GEN nf, GEN x, GEN ideal);
 GEN     ideal_two_elt(GEN nf, GEN ix);
@@ -479,7 +486,8 @@ GEN     idealhnf0(GEN nf, GEN a, GEN b);
 GEN     idealintersect(GEN nf, GEN x, GEN y);
 GEN     idealinv(GEN nf, GEN ix);
 GEN     ideallllred(GEN nf,GEN ix,GEN vdir,long prec);
-GEN     ideallllred_elt(GEN nf, GEN I);
+GEN     idealred_elt(GEN nf, GEN I);
+GEN     ideallllred_elt(GEN nf, GEN I, GEN vdir);
 GEN     idealmul(GEN nf, GEN ix, GEN iy);
 GEN     idealmul0(GEN nf, GEN ix, GEN iy, long flag, long prec);
 GEN     idealmulh(GEN nf, GEN ix, GEN iy);
@@ -504,10 +512,13 @@ GEN     nfkermodpr(GEN nf, GEN x, GEN modpr);
 GEN     nfmodprinit(GEN nf, GEN pr);
 GEN     nfsmith(GEN nf, GEN x);
 GEN     nfsolvemodpr(GEN nf, GEN a, GEN b, GEN modpr);
-GEN     oldidealinv(GEN nf, GEN ix);
 GEN     prime_to_ideal(GEN nf, GEN vp);
 GEN     principalideal(GEN nf, GEN a);
 GEN     principalidele(GEN nf, GEN a, long prec);
+GEN     vecdiv(GEN x, GEN y);
+GEN     vecinv(GEN x);
+GEN     vecmul(GEN x, GEN y);
+GEN     vecpow(GEN x, GEN n);
 
 /* base5.c */
 
@@ -634,6 +645,7 @@ GEN     sort(GEN x);
 long    tablesearch(GEN T, GEN x, int (*cmp)(GEN,GEN));
 GEN     tayl(GEN x, long v, long precdl);
 GEN     tchebi(long n, long v);
+GEN     vecbinome(long n);
 GEN     vecsort(GEN x, GEN k);
 GEN     vecsort0(GEN x, GEN k, long flag);
 
@@ -650,16 +662,18 @@ GEN     quadray(GEN bnf, GEN f, GEN flag, long prec);
 
 GEN     bnfclassunit0(GEN P,long flag,GEN data,long prec);
 GEN     bnfinit0(GEN P,long flag,GEN data,long prec);
+GEN     bnfmake(GEN sbnf,long prec);
 GEN     buchall(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, GEN gborne, long nbrelpid, long minsfb, long flun, long prec);
 GEN     buchfu(GEN bignf);
 GEN     classgrouponly(GEN P,GEN data,long prec);
 GEN     isprincipal(GEN bignf, GEN x);
 GEN     isprincipalall(GEN bignf, GEN x,long flall);
+GEN     isprincipalfact(GEN bnf,GEN P, GEN e, GEN C, long flag);
 GEN     isprincipalforce(GEN bignf,GEN x);
 GEN     isprincipalgen(GEN bignf, GEN x);
 GEN     isprincipalgenforce(GEN bignf,GEN x);
 GEN     isunit(GEN bignf, GEN x);
-GEN     bnfmake(GEN sbnf,long prec);
+GEN     quick_isprincipalgen(GEN bnf, GEN x);
 GEN     regulator(GEN P,GEN data,long prec);
 GEN     signunits(GEN bignf);
 GEN     smallbuchinit(GEN pol,GEN gcbach,GEN gcbach2,GEN gRELSUP,GEN gborne,long nbrelpid,long minsfb,long prec);
@@ -687,6 +701,7 @@ GEN     discrayabslistarch(GEN bnf, GEN arch, long bound);
 GEN     discrayabslistlong(GEN bnf, long bound);
 GEN     discrayrel(GEN bnr,GEN subgroup);
 GEN     discrayrelcond(GEN bnr,GEN subgroup);
+GEN     idealmodidele(GEN bnr, GEN x);
 GEN     isprincipalray(GEN bignf, GEN x);
 GEN     isprincipalrayall(GEN bignf, GEN x,long flall);
 GEN     isprincipalraygen(GEN bignf, GEN x);
@@ -700,7 +715,8 @@ GEN     subgrouplist0(GEN bnr, GEN indexbound, long all);
 /* buch4.c */
 
 GEN     bnfisnorm(GEN bnf,GEN x,long flag,long PREC);
-GEN     rnfisnorm(GEN bnf,GEN ext,GEN x,long flag,long PREC);
+GEN     rnfisnorm(GEN S, GEN x, long flag);
+GEN     rnfisnorminit(GEN bnf, GEN relpol, int galois);
 GEN     bnfissunit(GEN bnf,GEN suni,GEN x);
 GEN     bnfsunit(GEN bnf,GEN s,long PREC);
 long    nfhilbert(GEN bnf,GEN a,GEN b);
@@ -994,13 +1010,14 @@ GEN     precprime(GEN n);
 /* init.c */
 
 long    TIMER(pari_timer *T);
+void    TIMERstart(pari_timer *T);
 long    allocatemoremem(size_t newsize);
 GEN     changevar(GEN x, GEN y);
 void    checkmemory(GEN x);
 void    disable_dbg(long val);
 GEN     dummycopy(GEN x);
 void    err(long numerr, ...);
-void   *err_catch(long errnum, jmp_buf env, void *data);
+void   *err_catch(long errnum, jmp_buf *penv);
 void    err_leave(void **v);
 GEN     forcecopy(GEN x);
 void    freeall(void);
@@ -1023,7 +1040,7 @@ void    killbloc(GEN x);
 void    msgTIMER(pari_timer *T, char *format, ...);
 void    msgtimer(char *format, ...);
 GEN     newbloc(long n);
-void    pari_init(size_t parisize, long maxprime);
+void    pari_init(size_t parisize, ulong maxprime);
 GEN     reorder(GEN x);
 void    stackdummy(GEN x, long l);
 stackzone* switch_stack(stackzone *z, long n);
@@ -1053,6 +1070,7 @@ int     cmprr(GEN x, GEN y);
 int     cmpsi(long x, GEN y);
 GEN     dbltor(double x);
 GEN     diviiexact(GEN x, GEN y);
+GEN     diviuexact(GEN x, ulong y);
 void    diviiz(GEN x, GEN y, GEN z);
 GEN     divir(GEN x, GEN y);
 GEN     divis(GEN y, long x);
@@ -1098,6 +1116,7 @@ long    vals(ulong x);
 
 GEN     nffactor(GEN nf,GEN x);
 GEN     nffactormod(GEN nf,GEN pol,GEN pr);
+int     nfisgalois(GEN nf, GEN x);
 GEN     nfroots(GEN nf,GEN pol);
 GEN     rnfcharpoly(GEN nf,GEN T,GEN alpha,int n);
 GEN     rnfdedekind(GEN nf,GEN T,GEN pr);
@@ -1114,6 +1133,7 @@ GEN     cyclicperm(long l, long d);
 GEN     cyc_powtoperm(GEN cyc, long exp);
 GEN     group_elts(GEN G, long n);
 long    group_isabelian(GEN G);
+long    group_order(GEN G);
 GEN     group_subgroups(GEN G);
 GEN     perm_cycles(GEN v);
 GEN     perm_identity(long l);
@@ -1194,13 +1214,13 @@ GEN     factor(GEN x);
 GEN     factor0(GEN x,long flag);
 GEN     factorback(GEN fa,GEN nf);
 GEN     factorback0(GEN fa,GEN e, GEN nf);
+GEN     factorbackelt(GEN fa, GEN e, GEN nf);
 GEN     factpol(GEN x, long hint);
 GEN     gbezout(GEN x, GEN y, GEN *u, GEN *v);
 GEN     gcd0(GEN x, GEN y,long flag);
 GEN     gdeflate(GEN x, long v, long d);
 GEN     gdivexact(GEN x, GEN y);
 GEN     ggcd(GEN x, GEN y);
-GEN     ggcd0(GEN x, GEN y);
 GEN     ginvmod(GEN x, GEN y);
 GEN     gisirreducible(GEN x);
 GEN     glcm(GEN x, GEN y);
@@ -1258,7 +1278,6 @@ GEN     FpX_mul(GEN x,GEN y,GEN p);
 GEN     FpX_neg(GEN x,GEN p);
 GEN     FpX_normalize(GEN z, GEN p);
 GEN     FpX_red(GEN z, GEN p);
-GEN     FpX_small(GEN z, GEN p, long l);
 GEN     FpX_sqr(GEN x,GEN p);
 GEN     FpX_sub(GEN x,GEN y,GEN p);
 GEN     FpXQ_charpoly(GEN x, GEN T, GEN p);
@@ -1305,16 +1324,17 @@ GEN     small_to_pol(GEN z, long v);
 GEN     small_to_vec(GEN z);
 GEN     pol_to_small(GEN x);
 ulong   powuumod(ulong x, ulong n0, ulong p);
+GEN     powgumod(GEN x, ulong n0, GEN p);
 GEN     rescale_pol(GEN P, GEN h);
 GEN     unscale_pol(GEN P, GEN h);
 GEN     stopoly(long m, long p, long v);
 GEN     stopoly_gen(GEN m, GEN p, long v);
 GEN     u_FpXQ_pow(GEN x, GEN n, GEN pol, ulong p);
-GEN     u_FpX_divrem(GEN x, GEN y, ulong p, int malloc, GEN *pr);
+GEN     u_FpX_divrem(GEN x, GEN y, ulong p, GEN *pr);
 GEN     u_FpX_rem(GEN x, GEN y, ulong p);
 GEN     u_Fp_FpM(GEN x, ulong p);
 GEN     u_Fp_FpV(GEN x, ulong p);
-GEN     u_Fp_FpX(GEN x,int malloc, ulong p);
+GEN     u_Fp_FpX(GEN x, ulong p);
 int     u_FpX_is_squarefree(GEN z, ulong p);
 GEN     u_FpX_normalize(GEN z, ulong p);
 GEN     u_FpX_sub(GEN x, GEN y, ulong p);
@@ -1322,6 +1342,7 @@ GEN     u_FpX_gcd(GEN a, GEN b, ulong p);
 ulong   u_invmod(ulong x, ulong p);
 int     u_pow(int p, int k);
 int     u_val(ulong n, ulong p);
+GEN     u_zeropol(void);
 
 /* rootpol.c */
 
@@ -1340,7 +1361,6 @@ GEN     znstar_small(GEN zn);
 
 GEN     subfields(GEN nf,GEN d);
 GEN     subfields0(GEN nf,GEN d);
-GEN     conjugates(GEN pol);
 
 /* subgroup.c */
 
@@ -1372,10 +1392,6 @@ GEN     prodinf(entree *ep, GEN a, char *ch, long prec);
 GEN     prodinf0(entree *ep, GEN a, char *ch, long flag, long prec);
 GEN     prodinf1(entree *ep, GEN a, char *ch, long prec);
 GEN     produit(entree *ep, GEN a, GEN b, char *ch, GEN x);
-GEN     qromb(entree *ep, GEN a, GEN b, char *ch, long prec);
-GEN     qromi(entree *ep, GEN a, GEN b, char *ch, long prec);
-GEN     qromo(entree *ep, GEN a, GEN b, char *ch, long prec);
-GEN     rombint(entree *ep, GEN a, GEN b, char *ch, long prec);
 GEN     somme(entree *ep, GEN a, GEN b, char *ch, GEN x);
 GEN     sumalt(entree *ep, GEN a, char *ch, long prec);
 GEN     sumalt0(entree *ep, GEN a, char *ch,long flag, long prec);
