@@ -1,4 +1,4 @@
-/* $Id: parinf.h,v 1.15 2002/05/13 07:45:22 karim Exp $
+/* $Id: parinf.h,v 1.16 2002/08/21 15:09:57 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -59,10 +59,15 @@ typedef struct {
 #define nf_FORCE 2
 #define nf_GIVEPREC 4
 #define nf_GENMAT 8
+#define nf_GEN_IF_PRINCIPAL 512
 
 /* for buchray */
 #define nf_INIT  4
 
+/* for buchall */
+#define nf_ROOT1 512
+#define nf_UNITS 1024
+
 /* for discray */
 #define nf_REL  1
 #define nf_COND 2
