@@ -1,4 +1,4 @@
-/* $Id: paristio.h,v 1.15 2002/06/09 00:22:42 karim Exp $
+/* $Id: paristio.h,v 1.16 2002/09/08 10:41:22 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -67,10 +67,10 @@ extern char    *errmessage[], *current_psfile;
 
 #define is_universal_constant(x) ((GEN)(x) >= gzero && (GEN)(x) <= gi)
 
-#define copyifstack(x,y) {gpmem_t _t=(gpmem_t)(x); \
-			  (y)=(_t>=bot &&_t<top)? lcopy((GEN)_t): (long)_t;}
-#define icopyifstack(x,y) {gpmem_t _t=(gpmem_t)(x); \
-			  (y)=(_t>=bot &&_t<top)? licopy((GEN)_t): (long)_t;}
+#define copyifstack(x,y)  STMT_START {gpmem_t _t=(gpmem_t)(x); \
+  (y)=(_t>=bot &&_t<top)? lcopy((GEN)_t): (long)_t;} STMT_END
+#define icopyifstack(x,y) STMT_START {gpmem_t _t=(gpmem_t)(x); \
+  (y)=(_t>=bot &&_t<top)? licopy((GEN)_t): (long)_t;} STMT_END
 #define isonstack(x) ((gpmem_t)(x)>=bot && (gpmem_t)(x)<top)
 
 /* Define this to (1) locally (in a given file, NOT here) to check
