@@ -1,4 +1,4 @@
-/* $Id: anal.c,v 1.96 2002/06/08 14:37:45 karim Exp $
+/* $Id: anal.c,v 1.114 2002/09/11 21:43:01 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -22,6 +22,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "anal.h"
 #include "parinf.h"
 
+/* slightly more efficient than is_keyword_char. Not worth a static array. */
+#define is_key(c) (isalnum((int)(c)) || (c)=='_')
+
 #define separe(c) ((c)==';' || (c)==':')
 typedef GEN (*PFGEN)(ANYARG);
 typedef GEN (*F2GEN)(GEN,GEN);
@@ -50,6 +53,8 @@ static entree *installep(void *f,char *name,int l,int v,int add,entree **table);
 static entree *skipentry(void);
 
 extern void killbloc0(GEN x, int inspect);
+extern int term_width(void);
+extern GEN addsmulsi(long a, long b, GEN Y);
 
 /* last time we began parsing an object of specified type */
 static struct
@@ -57,13 +62,6 @@ static struct
   char *identifier, *symbol, *raw, *member, *start;
 } mark;
 
-/* when skipidentifier() detects that user function f() is being redefined,
- * (f()= ... ) this is set pointing to the opening parenthesis. Checked in
- * identifier(). Otherwise definition like f(x=1)= would change the value of
- * global variable x
- */
-static char *redefine_fun = NULL;
-
 /* points to the part of the string that remains to be parsed */
 static char *analyseur = NULL;
 
@@ -74,14 +72,195 @@ static long skipping_fun_def;
  * being checked). Used by the compatibility engine in the following way:
  *   when user types in a function whose name has changed, it is understood
  *   as EpNEW; first syntax error (missing = after function definition
- *   usually) triggers err_new_fun() if check_new_fun is set.
- */
+ *   usually) triggers err_new_fun() if check_new_fun is set. */
 static entree *check_new_fun;
 
 /* for control statements (check_break) */
 static long br_status, br_count;
 static GEN br_res = NULL;
 
+/* Mnemonic codes parser:
+ *
+ * TEMPLATE is assumed to be ";"-separated list of items.  Each item
+ * may have one of the following forms: id=value id==value id|value id&~value.
+ * Each id consists of alphanum characters, dashes and underscores.
+ * IDs are case-sensitive.
+
+ * ARG consists of several IDs separated by punctuation (and optional
+ * whitespace).  Each modifies the return value in a "natural" way: an
+ * ID from id=value should be the first in the sequence and sets RETVAL to
+ * VALUE (and cannot be negated), ID from id|value bit-ORs RETVAL with
+ * VALUE (and bit-ANDs RETVAL with ~VALUE if negated), ID from
+ * id&~value behaves as if it were noid|value, ID from
+ * id==value behaves the same as id=value, but should come alone.
+
+ * For items of the form id|value and id&~value negated forms are
+ * allowed: either when arg looks like no[-_]id, or when id looks like
+ * this, and arg is not-negated. */
+
+enum { A_ACTION_ASSIGN, A_ACTION_SET, A_ACTION_UNSET };
+enum { PARSEMNU_TEMPL_TERM_NL, PARSEMNU_ARG_WHITESP };
+#define IS_ID(c)	(isalnum((int)c) || ((c) == '_') || ((c) == '-'))
+#define ERR(reason)	STMT_START {	\
+    if (failure && first) {		\
+	*failure = reason; *failure_arg = NULL; return 0;		\
+    } else err(talker,reason); } STMT_END
+#define ERR2(reason,s)	STMT_START {	\
+    if (failure && first) {		\
+	*failure = reason; *failure_arg = s; return 0;		\
+    } else err(talker,reason,s); } STMT_END
+
+unsigned long
+parse_option_string(char *arg, char *template, long flag, char **failure, char **failure_arg)
+{
+    unsigned long retval = 0;
+    char *etemplate = NULL;
+
+    if (flag & PARSEMNU_TEMPL_TERM_NL)
+	etemplate = strchr(template, '\n');
+    if (!etemplate)
+	etemplate = template + strlen(template);
+
+    if (failure)
+	*failure = NULL;
+    while (1) {
+	long numarg;
+	char *e, *id;
+	char *negated;			/* action found with 'no'-ID */
+	int negate;			/* Arg has 'no' prefix removed */
+	int l, action = 0, first = 1, singleton = 0;
+	char b[80], *buf, *inibuf;
+
+	if (flag & PARSEMNU_ARG_WHITESP)
+	    while (isspace((int)*arg)) arg++;
+	if (!*arg)
+	    break;
+	e = arg;
+	while (IS_ID(*e))
+	    e++;
+	/* Now the ID is whatever is between arg and e. */
+	l = e - arg;
+	if (l >= sizeof(b))
+	    ERR("id too long in a stringified flag");
+	if (!l)				/* Garbage after whitespace? */
+	    ERR("a stringified flag does not start with an id");
+	strncpy(b, arg, l);
+	b[l] = 0;
+	arg = e;
+	e = inibuf = buf = b;
+	while (('0' <= *e) && (*e <= '9'))
+	    e++;
+	if (*e == 0)
+	    ERR("numeric id in a stringified flag");	
+	negate = 0;
+	negated = NULL;
+      find:
+	id = template;
+	while ((id = strstr(id, buf)) && id < etemplate) {
+	    if (IS_ID(id[l])) {		/* We do not allow abbreviations yet */
+		id = id + l;		/* False positive */
+		continue;
+	    }
+	    if ((id >= template + 2) && (IS_ID(id[-1]))) {
+		char *s = id;
+
+		if ( !negate && s >= template+3
+		     && ((id[-1] == '_') || (id[-1] == '-')) )
+		    s--;
+		/* Check whether we are preceeded by "no" */
+		if ( negate		/* buf initially started with "no" */
+		     || (s < template+2) || (s[-1] != 'o') || (s[-2] != 'n')
+		     || (s >= template+3 && IS_ID(s[-3]))) {
+		    id = id + l;		/* False positive */
+		    continue;
+		}
+		/* Found noID in the template! */
+		negated = id + l;
+		id = id + l;
+		continue;		/* Try to find without 'no'. */
+	    }
+	    /* Found as is */
+	    id = id + l;
+	    break;
+	}
+	if ( !id && !negated && !negate 
+	     && (l > 2) && buf[0] == 'n' && buf[1] == 'o' ) {
+	    /* Try to find the flag without the prefix "no". */
+	    buf += 2; l -= 2;
+	    if ((buf[0] == '_') || (buf[0] == '-')) { buf++; l--; }
+	    negate = 1;
+	    if (buf[0])
+		goto find;
+	}
+	if (!id && negated) {	/* Negated and AS_IS forms, prefer AS_IS */
+	    id = negated;	/* Otherwise, use negated form */
+	    negate = 1;
+	}
+	if (!id)
+	    ERR2("Unrecognized id '%s' in a stringified flag", inibuf);
+	if (singleton && !first)
+	    ERR("Singleton id non-single in a stringified flag");
+	if (id[0] == '=') {
+	    if (negate)
+		ERR("Cannot negate id=value in a stringified flag");
+	    if (!first)
+		ERR("Assign action should be first in a stringified flag");
+	    action = A_ACTION_ASSIGN;
+	    id++;
+	    if (id[0] == '=') {
+		singleton = 1;
+		id++;
+	    }
+	} else if (id[0] == '^') {
+	    if (id[1] != '~')
+		err(talker, "Unrecognized action in a template");
+	    id += 2;
+	    if (negate)
+		action = A_ACTION_SET;
+	    else
+		action = A_ACTION_UNSET;
+	} else if (id[0] == '|') {
+	    id++;
+	    if (negate)
+		action = A_ACTION_UNSET;
+	    else
+		action = A_ACTION_SET;
+	}
+
+	e = id;
+
+	while ((*e >= '0' && *e <= '9')) e++;
+	while (isspace((int)*e))
+	    e++;
+	if (*e && (*e != ';') && (*e != ','))
+	    err(talker, "Non-numeric argument of an action in a template");
+	numarg = atol(id);		/* Now it is safe to get it... */
+	switch (action) {
+	case A_ACTION_SET:
+	    retval |= numarg;
+	    break;
+	case A_ACTION_UNSET:
+	    retval &= ~numarg;
+	    break;
+	case A_ACTION_ASSIGN:
+	    retval = numarg;
+	    break;
+	default:
+	    ERR("error in parse_option_string");
+	}
+	first = 0;
+	if (flag & PARSEMNU_ARG_WHITESP)
+	    while (isspace((int)*arg))
+		arg++;
+	if (*arg && !(ispunct((int)*arg) && *arg != '-'))
+	    ERR("Junk after an id in a stringified flag");
+	/* Skip punctuation */
+	if (*arg)
+	    arg++;
+    }
+    return retval;
+}
+
 /*  Special characters:
  *     ' ', '\t', '\n', '\\' are forbidden internally (suppressed by filtre).
  *     { } are forbidden everywhere and will be used to denote optional
@@ -168,13 +347,13 @@ static GEN br_res = NULL;
  *      [0-9]+
  */
 char*
-_analyseur(void)
+get_analyseur(void)
 {
   return analyseur;
 }
 
 void
-_set_analyseur(char *s)
+set_analyseur(char *s)
 {
   analyseur = s;
 }
@@ -190,7 +369,6 @@ lisseq0(char *t, GEN (*f)(void))
   if (foreignExprHandler && *t == foreignExprSwitch)
     return (*foreignExprHandler)(t);
 
-  redefine_fun = NULL;
   check_new_fun = NULL;
   skipping_fun_def = 0;
   mark.start = analyseur = t;
@@ -205,6 +383,8 @@ lisseq0(char *t, GEN (*f)(void))
     return gerepilecopy(av, br_res);
   }
   if (res == NULL) { avma = av; return polx[fetch_user_var("NULL")]; }
+  /* ep->value, beware: it may be killed anytime.  */
+  if (isclone(res)) { avma = av; return forcecopy(res); }
   return gerepileupto(av, res);
 }
 
@@ -226,8 +406,13 @@ GEN flisexpr(char *s){ return flisseq0(s, expr);}
 GEN
 readseq(char *c, int strict)
 {
+  GEN z;
   check_new_fun=NULL; skipping_fun_def=0;
-  doskipseq(c, strict); return lisseq(c);
+  added_newline = 1;
+  doskipseq(c, strict);
+  z = lisseq0(c, seq);
+  if (!added_newline) pariputc('\n'); /* last output was print1() */
+  return z;
 }
 
 entree *
@@ -378,6 +563,21 @@ changevalue_p(entree *ep, GEN x)
   }
 }
 
+/* make GP variables safe for avma = top */
+void
+var_make_safe()
+{
+  long n;
+  entree *ep;
+  for (n = 0; n < functions_tblsz; n++)
+    for (ep = functions_hash[n]; ep; ep = ep->next)
+      if (EpVALENCE(ep) == EpVAR)
+      { /* make sure ep->value is a COPY */
+        var_cell *v = (var_cell*)ep->args;
+        if (v->flag == PUSH_VAL) changevalue(ep, (GEN)ep->value);
+      }
+}
+
 void
 kill_from_hashlist(entree *ep)
 {
@@ -608,7 +808,7 @@ err_match(char *s, char c)
 
 #define match2(s,c) if (*s != c) err_match(s,c);
 #define match(c) \
-  do { match2(analyseur, c); analyseur++; } while (0)
+  STMT_START { match2(analyseur, c); analyseur++; } STMT_END
 
 static long
 readlong()
@@ -676,7 +876,8 @@ typedef struct matcomp
 {
   GEN *ptcell;
   GEN parent;
-  long full_col, full_row;
+  int full_col, full_row;
+  void *extra; /* so far used by check_pointers only */
 } matcomp;
 
 /* Return the content of the matrix cell and sets members of corresponding
@@ -685,7 +886,8 @@ static GEN
 matcell(GEN p, matcomp *C)
 {
   GEN *pt = &p;
-  long c,r, tx, full_col, full_row;
+  long c,r, tx;
+  int full_col, full_row;
   tx = full_col = full_row = 0; 
   do {
     analyseur++; p = *pt; tx = typ(p);
@@ -711,20 +913,43 @@ matcell(GEN p, matcomp *C)
         full_col = full_row = 0;
         if (*analyseur==',') /* whole column */
         {
-          analyseur++; full_col = 1;
+          analyseur++;
           c = check_array_index(lg(p));
-          pt = (GEN*)(p + c); match(']'); break;
+          match(']');
+          if (*analyseur == '[')
+          { /* collapse [,c][r] into [r,c] */
+            analyseur++;
+            r = check_array_index(lg(p));
+            pt = (GEN*)(((GEN)p[c]) + r); /* &coeff(p,r,c) */
+            match(']');
+          }
+          else
+          {
+            full_col = 1;
+            pt = (GEN*)(p + c);
+          }
+          break;
         }
 
         r = check_array_index(lg(p[1]));
         match(',');
         if (*analyseur == ']') /* whole row */
         {
-          GEN p2 = cgetg(lg(p),t_VEC);
           analyseur++;
-          if (*analyseur != '[') full_row = r;
-          for (c=1; c<lg(p); c++) p2[c] = coeff(p,r,c);
-          pt = &p2;
+          if (*analyseur == '[')
+          { /* collapse [r,][c] into [r,c] */
+            analyseur++;
+            c = check_array_index(lg(p));
+            pt = (GEN*)(((GEN)p[c]) + r); /* &coeff(p,r,c) */
+            match(']');
+          }
+          else
+          {
+            GEN p2 = cgetg(lg(p),t_VEC);
+            full_row = r; /* record row number */
+            for (c=1; c<lg(p); c++) p2[c] = coeff(p,r,c);
+            pt = &p2;
+          }
         }
         else
         {
@@ -888,11 +1113,16 @@ truc(void)
       free(table); return p1;
 
     case '%':
-      old = analyseur-1; p = 0;
-      if (!gp_history_fun) err(talker2,"history not available",old,mark.start);
+    old = analyseur-1;
+    if (!GP_DATA) err(talker2,"history not available", old, mark.start);
+    else
+    {
+      gp_hist *H = GP_DATA->hist;
+      p = 0;
       while (*analyseur == '`') { analyseur++; p++; }
-      return p ? gp_history_fun(p         ,1,old,mark.start)
-               : gp_history_fun(number(&n),0,old,mark.start);
+      return p ? gp_history(H, -p        , old, mark.start)
+               : gp_history(H, number(&n), old, mark.start);
+    }
   }
   err(caracer1,analyseur-1,mark.start);
   return NULL; /* not reached */
@@ -976,10 +1206,11 @@ affect_block(GEN *res)
 
 /* assign res at *pt in "simple array object" p */
 static GEN
-change_compo(matcomp c, GEN res)
+change_compo(matcomp *c, GEN res)
 {
-  GEN p = c.parent, *pt = c.ptcell;
-  long i, full_row = c.full_row, full_col = c.full_col;
+  GEN p = c->parent, *pt = c->ptcell;
+  long i;
+  int full_row = c->full_row, full_col = c->full_col;
   char *old = analyseur;
 
   if (typ(p) == t_VECSMALL)
@@ -1023,7 +1254,7 @@ matrix_block(GEN p)
   if (res)
   {
     if (fun) res = fun(cpt, res);
-    res = change_compo(c,res);
+    res = change_compo(&c,res);
     analyseur = end;
   }
   else res = isonstack(cpt)? gcopy(cpt): cpt; /* no assignment */
@@ -1052,26 +1283,31 @@ realloc_buf(char *bp, long len, char **ptbuf,char **ptlimit)
 static char *
 expand_string(char *bp, char **ptbuf, char **ptlimit)
 {
-  char *tmp, *s = analyseur;
-  long len, alloc;
+  char *tmp = NULL; /* -Wall */
+  long len = 0; /* -Wall */
+  int alloc = 1;
 
-  while (is_keyword_char(*s)) s++;
+  if (is_keyword_char(*analyseur))
+  {
+    char *s = analyseur;
+    do s++; while (is_keyword_char(*s));
 
-  if ((*s == '"' || *s == ',' || *s == ')') && !is_entry(analyseur))
-  { /* Do not create new user variable. Consider as a literal */
-    tmp = analyseur;
-    len = s - analyseur;
-    analyseur = s;
-    alloc = 0;
+    if ((*s == '"' || *s == ',' || *s == ')') && !is_entry(analyseur))
+    { /* Do not create new user variable. Consider as a literal */
+      tmp = analyseur;
+      len = s - analyseur;
+      analyseur = s;
+      alloc = 0;
+    }
   }
-  else
+
+  if (alloc)
   {
     gpmem_t av = avma;
     GEN p1 = expr();
     if (br_status) err(breaker,"here (expanding string)");
     tmp = GENtostr0(p1, &DFLT_OUTPUT, &gen_output);
     len = strlen(tmp); avma = av;
-    alloc = 1;
   }
   if (ptlimit && bp + len > *ptlimit)
     bp = realloc_buf(bp, len, ptbuf,ptlimit);
@@ -1120,28 +1356,22 @@ static GEN
 any_string()
 {
   long n = 0, len = 16;
-  GEN p1, res = new_chunk(len + 1);
+  GEN res = new_chunk(len + 1);
 
   while (*analyseur)
   {
-    if (*analyseur == '"')
-    {
-      res[n++] = (long) strtoGENstr_t();
-      continue;
-    }
     if (*analyseur == ')' || *analyseur == ';') break;
     if (*analyseur == ',')
       analyseur++;
     else
     {
-      p1 = expr();
+      res[n++] = (long)expr();
       if (br_status) err(breaker,"here (print)");
-      res[n++] = (long) p1;
     }
     if (n == len)
     {
       long newlen = len << 1;
-      p1 = new_chunk(newlen + 1);
+      GEN p1 = new_chunk(newlen + 1);
       for (n = 0; n < len; n++) p1[n] = res[n];
       res = p1; len = newlen;
     }
@@ -1196,35 +1426,53 @@ strtoGENstr(char *s, long flag)
   return x;
 }
 
+/* x = gzero: no default value, otherwise a t_STR, formal expression for
+ * default argument. Evaluate and return. */
 static GEN
-make_arg(GEN x)
-{
-  return (x==gzero)? x
-                   : (typ(x) == t_STR)? geval(x): x;
-}
+make_arg(GEN x) { return (x==gzero)? x: geval(x); }
 
-/* p = NULL + array of variable numbers (longs) + function text */
 static GEN
-call_fun(GEN p, GEN *arg, GEN *loc, int narg, int nloc)
+fun_seq(char *p)
 {
-  GEN res;
-  long i;
-
-  p++; /* skip NULL */
-  /* push new values for formal parameters */
-  for (i=0; i<narg; i++) copyvalue(*p++, *arg++);
-  for (i=0; i<nloc; i++) pushvalue(*p++, make_arg(*loc++));
-  /* dumps arglist from identifier() to the garbage zone */
-  res = lisseq((char *)p);
+  GEN res = lisseq(p);
   if (br_status != br_NONE)
     br_status = br_NONE;
   else
     if (! is_universal_constant(res)) /* important for gnil */
       res = forcecopy(res); /* make result safe */
+  return res;
+}
 
+/* p = NULL + array of variable numbers (longs) + function text */
+static GEN
+call_fun(GEN p, GEN *arg, gp_args *f)
+{
+  GEN res, *loc = f->arg + f->narg;
+  long i;
+
+  p++; /* skip NULL */
+  /* push new values for formal parameters */
+  for (i=0; i<f->narg; i++) copyvalue(*p++, *arg++);
+  for (i=0; i<f->nloc; i++) pushvalue(*p++, make_arg(*loc++));
+  /* dumps arglist from identifier() to the garbage zone */
+  res = fun_seq((char *)p);
   /* pop out ancient values of formal parameters */
-  for (i=0; i<nloc; i++) killvalue(*--p);
-  for (i=0; i<narg; i++) killvalue(*--p);
+  for (i=0; i<f->nloc; i++) killvalue(*--p);
+  for (i=0; i<f->narg; i++) killvalue(*--p);
+  return res;
+}
+/* p = NULL + array of variable numbers (longs) + function text */
+static GEN
+call_member(GEN p, GEN x)
+{
+  GEN res;
+
+  p++; /* skip NULL */
+  /* push new values for formal parameters */
+  pushvalue(*p++, x);
+  res = fun_seq((char *)p);
+  /* pop out ancient values of formal parameters */
+  killvalue(*--p);
   return res;
 }
 
@@ -1265,17 +1513,17 @@ check_pointers(unsigned int ptrs, matcomp *init[])
       GEN *pt = c->ptcell, x = gclone(*pt);
       if (c->parent == NULL)
       {
-        if (isclone(*pt)) killbloc(*pt);
+        if (isclone(c->extra)) killbloc((GEN)c->extra);
         *pt = x;
       }
       else
-        (void)change_compo(*c, x);
+        (void)change_compo(c, x);
       free((void*)c);
     }
 }
 
 #define match_comma() \
-  do { if (matchcomma) match(','); else matchcomma = 1; } while (0)
+  STMT_START { if (matchcomma) match(','); else matchcomma = 1; } STMT_END
 
 static void
 skipdecl(void)
@@ -1283,6 +1531,18 @@ skipdecl(void)
   if (*analyseur == ':') { analyseur++; skipexpr(); }
 }
 
+static void
+skip_arg_block(gp_args *f)
+{
+  int i, matchcomma = 0;
+  for (i = f->narg; i; i--)
+  {
+    if (do_switch(0,matchcomma))
+      matchcomma = 1;
+    else { match_comma(); skipexpr(); skipdecl(); }
+  }
+}
+
 static long
 check_args()
 {
@@ -1382,7 +1642,7 @@ num_deriv(void *call, GEN argvec[])
 
 /* as above, for user functions */
 static GEN
-num_derivU(GEN p, GEN *arg, GEN *loc, int narg, int nloc)
+num_derivU(GEN p, GEN *arg, gp_args *f)
 {
   GEN eps,a,b, x = *arg;
   long fpr, pr, l, e, ex;
@@ -1390,7 +1650,7 @@ num_derivU(GEN p, GEN *arg, GEN *loc, int narg, int nloc)
 
   if (!is_const_t(typ(x)))
   {
-    a = call_fun(p,arg,loc,narg,nloc);
+    a = call_fun(p,arg,f);
     return gerepileupto(av, deriv(a,gvar9(a)));
   }
   fpr = precision(x)-2; /* required final prec (in sig. words) */
@@ -1402,8 +1662,8 @@ num_derivU(GEN p, GEN *arg, GEN *loc, int narg, int nloc)
   e = fpr * BITS_IN_HALFULONG; /* 1/2 required prec (in sig. bits) */
 
   eps = real2n(-e, l);
-  *arg = fix(gsub(x, eps), l); a = call_fun(p,arg,loc,narg,nloc);
-  *arg = fix(gadd(x, eps), l); b = call_fun(p,arg,loc,narg,nloc);
+  *arg = fix(gsub(x, eps), l); a = call_fun(p,arg,f);
+  *arg = fix(gadd(x, eps), l); b = call_fun(p,arg,f);
   setexpo(eps, e-1);
   return gerepileupto(av, gmul(gsub(b,a), eps));
 }
@@ -1471,7 +1731,7 @@ identifier(void)
     }
     return matrix_block((GEN)ep->value);
   }
-  ep = do_alias(ep); matchcomma = 0;
+  ep = do_alias(ep);
 #ifdef STACK_CHECK
   if (PARI_stack_limit && (void*) &ptr <= PARI_stack_limit)
       err(talker2, "deep recursion", mark.identifier, mark.start);
@@ -1485,6 +1745,7 @@ identifier(void)
     void *call = ep->value;
     GEN argvec[9];
     matcomp *init[9];
+    char *flags = NULL;
 
     deriv = (*analyseur == '\'' && analyseur[1] == '(') && analyseur++;
     if (*analyseur == '(')
@@ -1506,6 +1767,7 @@ identifier(void)
     else                  ret = RET_GEN;
     /* Optimized for G and p. */
     i = 0;
+    matchcomma = 0;
     while (*s == 'G')
     {
       match_comma(); s++;
@@ -1514,7 +1776,7 @@ identifier(void)
     }
     if (*s == 'p') { argvec[i++] = (GEN) prec; s++; }
 
-    while (*s)
+    while (*s && *s != '\n')
       switch (*s++)
       {
 	case 'G': /* GEN */
@@ -1553,6 +1815,7 @@ identifier(void)
           {
             c->parent = NULL;
             c->ptcell = (GEN*)&ep->value;
+            c->extra = (GEN*)ep->value;
           }
           has_pointer |= (1 << i);
           init[i] = c;
@@ -1583,6 +1846,24 @@ identifier(void)
 	  *bp++ = 0; argvec[i++] = (GEN) buf;
 	  break;
 
+	case 'M': /* Mnemonic flag */
+	  match_comma(); argvec[i] = expr();
+          if (br_status) err(breaker,"here (argument reading)");
+	  if (typ(argvec[i]) == t_STR) {
+	      if (!flags)
+		  flags = ep->code;
+	      flags = strchr(flags, '\n'); /* Skip to the following '\n' */
+	      if (!flags)
+		  err(talker, "not enough flags in string function signature");
+	      flags++;
+	      argvec[i] = (GEN) parse_option_string((char*)(argvec[i] + 1),
+			  flags, PARSEMNU_ARG_WHITESP | PARSEMNU_TEMPL_TERM_NL,
+			  NULL, NULL);
+	  } else
+	      argvec[i] = (GEN)itos(argvec[i]);
+	  i++;
+          break;
+
 	case 's': /* expanded string; empty arg yields "" */
 	  match_comma();
 	  if (*s == '*') /* any number of string objects */
@@ -1595,10 +1876,7 @@ identifier(void)
           while (*analyseur)
           {
             if (*analyseur == ',' || *analyseur == ')') break;
-            if (*analyseur == '"')
-              bp = readstring_i(bp, &buf,&limit);
-            else
-              bp = expand_string(bp, &buf,&limit);
+            bp = expand_string(bp, &buf,&limit);
           }
           *bp++ = 0; argvec[i++] = (GEN)buf;
           break;
@@ -1748,6 +2026,7 @@ identifier(void)
 
       case 88: /* global */
         if (*analyseur == ')') return global0();
+        matchcomma = 0;
         while (*analyseur != ')')
         {
           match_comma(); ch1=analyseur;
@@ -1756,10 +2035,6 @@ identifier(void)
           switch(EpVALENCE(ep))
           {
             case EpGVAR:
-#if 0
-              err(warner,"%s already declared global", ep->name);
-#endif
-              /* fall through */
             case EpVAR: break;
             default: err(talker2,"symbol already in use",ch1,mark.start);
           }
@@ -1784,35 +2059,35 @@ identifier(void)
   switch (EpVALENCE(ep))
   {
     GEN *defarg; /* = default args, and values for local variables */
-    int narg, nloc;
+    GEN *arglist;
     gp_args *f;
 
     case EpUSER: /* user-defined functions */
       f = (gp_args*)ep->args;
-      defarg = f->arg;
-      narg = f->narg;
-      nloc = f->nloc;
       deriv = (*analyseur == '\'' && analyseur[1] == '(') && analyseur++;
+      arglist = (GEN*) new_chunk(f->narg);
       if (*analyseur != '(') /* no args */
       {
 	if (*analyseur != '='  ||  analyseur[1] == '=')
         {
-          GEN *arglist = (GEN*) new_chunk(narg);
-          for (i=0; i<narg; i++)
-            arglist[i] = make_arg(defarg[i]);
-	  return call_fun((GEN)ep->value, arglist, defarg+narg, narg, nloc);
+          for (i=0; i<f->narg; i++)
+            arglist[i] = make_arg(f->arg[i]);
+	  return call_fun((GEN)ep->value, arglist, f);
         }
 	match('('); /* ==> error */
       }
-      if (analyseur != redefine_fun)
+      analyseur++; /* skip '(' */
+      ch1 = analyseur;
+      skip_arg_block(f);
+      if (*analyseur == ')' && (analyseur[1] != '=' || analyseur[2] == '='))
       {
-        GEN *arglist = (GEN*) new_chunk(narg);
-        ch1 = analyseur; analyseur++;
-        for (i=0; i<narg; i++)
+        matchcomma = 0;
+        analyseur = ch1;
+        for (i=0; i<f->narg; i++)
         {
           if (do_switch(0,matchcomma))
           { /* default arg */
-            arglist[i] = make_arg(defarg[i]);
+            arglist[i] = make_arg(f->arg[i]);
             matchcomma = 1;
           }
           else
@@ -1823,27 +2098,24 @@ identifier(void)
             if (br_status) err(breaker,"here (reading function args)");
           }
         }
-        if (*analyseur++ == ')' && (*analyseur != '=' || analyseur[1] == '='))
+        analyseur++; /* skip ')' */
+        if (deriv)
         {
-          if (deriv)
-          {
-            if (!narg)
-              err(talker2, "can't derive this", mark.identifier, mark.start);
-            return num_derivU((GEN)ep->value, arglist, defarg+narg, narg, nloc);
-          }
-          return call_fun((GEN)ep->value, arglist, defarg+narg, narg, nloc);
+          if (!f->narg)
+            err(talker2, "can't derive this", mark.identifier, mark.start);
+          return num_derivU((GEN)ep->value, arglist, f);
         }
-
-        /* should happen only in cases like (f()= f()=); f (!!!) */
-        analyseur--;
-        if (*analyseur != ',' && *analyseur != ')') skipexpr();
-        while (*analyseur == ',') { analyseur++; skipexpr(); }
-        match(')');
-        if (*analyseur != '='  ||  analyseur[1] == '=')
-          err(nparamer1,mark.identifier,mark.start);
-        matchcomma=0; analyseur = ch1;
+        return call_fun((GEN)ep->value, arglist, f);
       }
-      redefine_fun = NULL;
+
+      /* REDEFINE function */
+      if (*analyseur != ',' && *analyseur != ')') skipexpr();
+      while (*analyseur == ',') { analyseur++; skipexpr(); }
+      match(')');
+      if (*analyseur != '='  ||  analyseur[1] == '=')
+        err(nparamer1,mark.identifier,mark.start);
+      analyseur = ch1-1; /* points to '(' */
+
       free_args((gp_args*)ep->args);
     /* Fall through */
 
@@ -1851,7 +2123,7 @@ identifier(void)
     {
       GEN tmpargs = (GEN)avma;
       char *start;
-      long len;
+      long len, narg, nloc;
 
       check_new_fun = ep;
 
@@ -1864,17 +2136,16 @@ identifier(void)
         err(talker2, "local() bloc must appear before any other expression",
                      mark.identifier,mark.start);
       match('=');
-      while (strncmp(analyseur,"local(",6) == 0)
-      {
-        analyseur += 6;
-        nloc += check_args();
-        match(')'); while(separe(*analyseur)) analyseur++;
-      }
       { /* checking function definition */
-        char *oldredef = redefine_fun;
         skipping_fun_def++;
+        while (strncmp(analyseur,"local(",6) == 0)
+        {
+          analyseur += 6;
+          nloc += check_args();
+          match(')'); while(separe(*analyseur)) analyseur++;
+        }
         start = analyseur; skipseq(); len = analyseur-start;
-        skipping_fun_def--; redefine_fun = oldredef;
+        skipping_fun_def--;
       }
       /* function is ok. record it */
       newfun = ptr = (GEN) newbloc(narg+nloc + (len>>TWOPOTBYTES_IN_LONG) + 4);
@@ -1908,9 +2179,9 @@ identifier(void)
       /* record text */
       strncpy((char *)newfun, start, len);
       ((char *) newfun)[len] = 0;
-      if (EpVALENCE(ep) == EpUSER) gunclone((GEN)ep->value);
-     /* have to wait till here because of strncopy above. In pathological
+     /* wait till here for gunclone because of strncopy above. In pathological
       * cases, e.g. (f()=f()=x), new text is given by value of old one! */
+      if (EpVALENCE(ep) == EpUSER) gunclone((GEN)ep->value);
       ep->value = (void *)ptr;
       ep->valence = EpUSER;
       check_new_fun=NULL;
@@ -1929,8 +2200,6 @@ number(long *nb)
   return m;
 }
 
-extern GEN addsmulsi(long a, long b, GEN Y);
-
 static GEN
 constante()
 {
@@ -1951,6 +2220,9 @@ constante()
   {
     default: return y; /* integer */
     case '.':
+      if (isalpha((int)analyseur[1])
+          && analyseur[1] != 'e' && analyseur[1] != 'E')
+        return y; /* member function */
       analyseur++; i = 0;
       while (isdigit((int)*analyseur))
       {
@@ -2000,9 +2272,6 @@ constante()
 /**                   HASH TABLE MANIPULATIONS                     **/
 /**                                                                **/
 /********************************************************************/
-/* slightly more efficient than is_keyword_char. Not worth a static array. */
-#define is_key(c) (isalnum((int)(c)) || (c)=='_')
-
 long
 is_keyword_char(char c) { return is_key(c); }
 
@@ -2240,8 +2509,6 @@ entry(void)
 /**                          SKIP FUNCTIONS                        **/
 /**                                                                **/
 /********************************************************************/
-extern int term_width(void);
-
 /* as skipseq without modifying analyseur && al */
 static void
 doskipseq(char *c, int strict)
@@ -2503,9 +2770,8 @@ check_matcell()
 static void
 skipidentifier(void)
 {
-  int matchcomma=0;
+  int matchcomma;
   entree *ep;
-  char *old;
 
   mark.identifier = analyseur; ep = do_alias(skipentry());
   if (ep->code)
@@ -2521,11 +2787,12 @@ skipidentifier(void)
     analyseur++;
 
     /* Optimized for G and p. */
+    matchcomma = 0;
     while (*s == 'G') { match_comma(); skipexpr(); s++; }
     if (*s == 'p') s++;
-    while (*s) switch (*s++)
+    while (*s && *s != '\n') switch (*s++)
     {
-      case 'G': case 'n': case 'L':
+      case 'G': case 'n': case 'L': case 'M':
         match_comma();
         if (*analyseur == ',' || *analyseur == ')') break;
         skipexpr(); break;
@@ -2583,6 +2850,8 @@ skipidentifier(void)
         match('='); matchcomma = 0; break;
       case ',':
         matchcomma=1; break;
+      case '\n':			/* Before the mnemonic */
+	break;
       default:
         err(bugparier,"skipidentifier (unknown code)");
     }
@@ -2611,6 +2880,7 @@ skipidentifier(void)
 	  break;
       case 81: case 82: skipexpr(); match(','); skipseq(); break;
       case 88:
+        matchcomma = 0;
         while (*analyseur != ')') { match_comma(); skipexpr(); };
         break;
       default: err(valencer1);
@@ -2626,8 +2896,6 @@ skipidentifier(void)
     case EpUSER: /* fonctions utilisateur */
     {
       char *ch1 = analyseur;
-      gp_args *f;
-      int i;
 
       if (*analyseur == '\'') analyseur++;
       if (*analyseur != '(')
@@ -2635,21 +2903,12 @@ skipidentifier(void)
 	if ( *analyseur != '='  ||  analyseur[1] == '=' ) return;
 	match('('); /* error */
       }
-      f = (gp_args*)ep->args;
       analyseur++;  /* skip '(' */
-      for (i = f->nloc + f->narg; i; i--)
-      {
-	if (do_switch(0,matchcomma))
-          matchcomma = 1;
-	else { match_comma(); skipexpr(); skipdecl(); }
-      }
-
-      if (*analyseur == ')')
-	if ( analyseur[1] != '=' || analyseur[2] == '=' )
+      skip_arg_block((gp_args*)ep->args);
+      if (*analyseur == ')' && (analyseur[1] != '=' || analyseur[2] == '='))
 	  { analyseur++; return; }
 
       /* here we are redefining a user function */
-      old = analyseur;
       if (*analyseur != ',' && *analyseur != ')') skipexpr();
       while (*analyseur == ',') { analyseur++; skipexpr(); }
       match(')');
@@ -2657,10 +2916,9 @@ skipidentifier(void)
       if (*analyseur != '=' || analyseur[1] == '=')
       {
         if (skipping_fun_def) return;
-        err(nparamer1,old,mark.start);
+        err(nparamer1,mark.identifier,mark.start);
       }
-      analyseur = ch1; matchcomma = 0;
-      if (!redefine_fun) redefine_fun = analyseur;
+      analyseur = ch1;
     } /* fall through */
 
     case EpNEW: /* new function */
@@ -2670,6 +2928,7 @@ skipidentifier(void)
 	err(paramer1, mark.identifier, mark.start);
       }
       check_new_fun = NOT_CREATED_YET; match('(');
+      matchcomma = 0;
       while (*analyseur != ')') { match_comma(); skipexpr(); skipdecl(); };
       match(')');
       if (*analyseur == '=' && analyseur[1] != '=')
@@ -2689,7 +2948,13 @@ skipconstante(void)
 {
   while (isdigit((int)*analyseur)) analyseur++;
   if ( *analyseur!='.' && *analyseur!='e' && *analyseur!='E' ) return;
-  if (*analyseur=='.') analyseur++;
+  if (*analyseur=='.')
+  {
+    if (isalpha((int)analyseur[1])
+        && analyseur[1] != 'e' && analyseur[1] != 'E')
+      return; /* member function */
+    analyseur++;
+  }
   while (isdigit((int)*analyseur)) analyseur++;
   if ( *analyseur=='e'  ||  *analyseur=='E' )
   {
@@ -3319,7 +3584,7 @@ read_member(GEN x)
       gunclone((GEN)ep->value); return NULL;
     }
     if (EpVALENCE(ep) == EpMEMBER)
-      return call_fun((GEN)ep->value, NULL, &x, 0, 1);
+      return call_member((GEN)ep->value, x);
     else
     {
       GEN y = ((F1GEN)ep->value)(x);
@@ -3361,7 +3626,7 @@ GEN
 return0(GEN x)
 {
   GEN y = br_res;
-  br_res = x? gclone(x): NULL;
+  br_res = (x && x != gnil)? gclone(x): NULL;
   if (y) gunclone(y);
   br_status = br_RETURN; return NULL;
 }
@@ -3417,63 +3682,3 @@ alias0(char *s, char *old)
   (void)installep(x, s, strlen(s), EpALIAS, 0, functions_hash + hash);
 }
 
-extern void err_leave_default(long n);
-
-/* Try f (trapping error e), recover using r (break_loop, if NULL) */
-GEN
-trap0(char *e, char *r, char *f)
-{
-  VOLATILE gpmem_t av = avma;
-  VOLATILE long numerr = -1;
-  VOLATILE GEN x = gnil;
-  char *F;
-       if (!strcmp(e,"errpile")) numerr = errpile;
-  else if (!strcmp(e,"typeer")) numerr = typeer;
-  else if (!strcmp(e,"gdiver2")) numerr = gdiver2;
-  else if (!strcmp(e,"invmoder")) numerr = invmoder;
-  else if (!strcmp(e,"accurer")) numerr = accurer;
-  else if (!strcmp(e,"archer")) numerr = archer;
-  else if (*e) err(impl,"this trap keyword");
-  /* TO BE CONTINUED */
-
-  if (f && r)
-  { /* explicit recovery text */
-    char *a = analyseur;
-    void *catcherr;
-    jmp_buf env;
-
-    if (setjmp(env))
-    {
-      avma = av;
-      err_leave(&catcherr);
-      x = lisseq(r);
-      skipseq();
-    }
-    else
-    {
-      catcherr = err_catch(numerr, env, NULL);
-      x = lisseq(f);
-      err_leave(&catcherr);
-    }
-    analyseur = a;
-    return x;
-  }
-
-  F = f? f: r; /* define a default handler */
- /* default will execute F (or start a break loop), then jump to
-  * environnement */
-  if (F)
-  {
-    if (!*F || (*F == '"' && F[1] == '"')) /* unset previous handler */
-    {/* TODO: find a better interface
-      * TODO: no leaked handler from the library should have survived
-      */
-      err_leave_default(numerr);
-      return x;
-    }
-    F = pari_strdup(F);
-  }
-  (void)err_catch(numerr, NULL, F);
-  return x;
-}
-
