@@ -1,4 +1,4 @@
-/* $Id: errmsg.c,v 1.11 2002/05/07 01:08:40 karim Exp $
+/* $Id: errmsg.c,v 1.14 2002/09/12 18:37:02 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -67,6 +67,8 @@ char *errmessage[]=
   "incorrect type",
 /* consister */
   "inconsistent data",
+/* user */
+  "",
 
     /*  MP.S  */
 
@@ -188,7 +190,7 @@ char *errmessage[]=
 /* facter */
   "negative argument in factorial function",
 /* hiler1 */
-  "insufficient precision for p=2 in hil",
+  "insufficient precision for p=2 in hilbert",
 /* funder2 */
   "discriminant not congruent to 0 or 1 mod 4",
 /* generer */
@@ -238,7 +240,7 @@ char *errmessage[]=
   "impossible",
 /* operf */
   "forbidden",
-/* gdiver2 */
+/* gdiver */
   "division by zero in gdiv, gdivgs or ginv",
 /* inter2 */
   "a log/atan appears in the integration, PARI cannot handle that",
