@@ -1,4 +1,4 @@
-/* $Id: es.c,v 1.83 2002/06/08 14:02:31 karim Exp $
+/* $Id: es.c,v 1.94 2002/09/12 13:28:14 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -28,7 +28,7 @@ void
 hit_return(void)
 {
   int c;
-  if (under_texmacs || under_emacs) return;
+  if (GP_DATA && (GP_DATA->flags & (EMACS|TEXMACS))) return;
   pariputs("---- (type RETURN to continue) ----");
   /* if called from a readline callback, may be in a funny TTY mode,  */
   do c = fgetc(stdin); while (c >= 0 && c != '\n' && c != '\r' && c != ' ');
@@ -42,10 +42,7 @@ hit_return(void)
 /********************************************************************/
 #define ONE_LINE_COMMENT   2
 #define MULTI_LINE_COMMENT 1
-/* Filter s into t. If flag is a query, return s (yes) / NULL (no)
- * Otherwise, if t == NULL, allocate enough room, filter then return t.
- *            if not return pointer to ending '\0' in t.
- */
+/* Filter F->s into F->t */
 char *
 filtre0(filtre_t *F)
 {
@@ -85,7 +82,11 @@ filtre0(filtre_t *F)
       {
         while (c != '*' || *s != '/')
         {
-          if (!*s) goto END;
+          if (!*s) 
+          {
+            if (!F->more_input) F->more_input = 1;
+            goto END;
+          }
           c = *s++;
         }
         s++;
@@ -271,25 +272,30 @@ vpariputs(char* format, va_list args)
   for(;;)
   {
     int l;
-    s = buf = gpmalloc(bufsize);
+    buf = gpmalloc(bufsize);
     l = vsnprintf(buf,bufsize,str,args);
     if (l < 0) l = bufsize<<1; else if (l < bufsize) break;
     free(buf); bufsize++;
   }
   buf[bufsize] = 0; /* just in case */
 #else
-  s = buf = gpmalloc(bufsize);
+  buf = gpmalloc(bufsize);
   (void)vsprintf(buf,str,args); /* pray it does fit */
 #endif
+  f = s = buf;
   if (nb)
-    for (f=s; *f; f++)
+    while ( *f )
+    {
       if (*f == '\003' && f[21] == '\003')
       {
         *f = 0; f[21] = 0; /* remove the bracing chars */
         pariOut->puts(s); bruteall((GEN)atol(f+1),'g',-1,1);
         f += 22; s = f;
-        if (!--nb) break;
+        if (!--nb) break; 
       }
+      else
+        f++;
+    }
   pariOut->puts(s); free(buf); free(str);
 }
 
@@ -364,8 +370,8 @@ term_width_intern(void)
 #ifdef HAS_TIOCGWINSZ
   {
     struct winsize s;
-    if (!under_emacs && !under_texmacs && !ioctl(0, TIOCGWINSZ, &s))
-      return s.ws_col;
+    if (!(GP_DATA && (GP_DATA->flags & (EMACS|TEXMACS)))
+     && !ioctl(0, TIOCGWINSZ, &s)) return s.ws_col;
   }
 #endif
 #ifdef UNIX
@@ -389,8 +395,8 @@ term_height_intern(void)
 #ifdef HAS_TIOCGWINSZ
   {
     struct winsize s;
-    if (!under_emacs && !under_texmacs && !ioctl(0, TIOCGWINSZ, &s))
-      return s.ws_row;
+    if (!(GP_DATA && (GP_DATA->flags & (EMACS|TEXMACS)))
+     && !ioctl(0, TIOCGWINSZ, &s)) return s.ws_row;
   }
 #endif
 #ifdef UNIX
@@ -443,7 +449,7 @@ puts80(char *s)
 }
 PariOUT pariOut80= {putc80, puts80, normalOutF, NULL};
 
-void
+static void
 init80(long n)
 {
   col_index = n; pariOut = &pariOut80;
@@ -481,16 +487,18 @@ puts_lim_lines(char *s)
 
 PariOUT pariOut_lim_lines= {putc_lim_lines, puts_lim_lines, normalOutF, NULL};
 
-/* s = prefix already printed (print up to max lines) */
+/* n = length of prefix already printed (print up to max lines) */
 void
-init_lim_lines(char *s, long max)
+lim_lines_output(GEN z, pariout_t *fmt, long n, long max)
 {
-  if (!max) return;
-  if (!s) { pariOut = &defaultOut; return; }
+  PariOUT *tmp = pariOut;
   max_width = term_width();
   max_lin = max;
-  lin_index = 1; col_index = strlen(s);
+  lin_index = 1;
+  col_index = n;
   pariOut = &pariOut_lim_lines;
+  gen_output(z, fmt);
+  pariOut = tmp;
 }
 
 #define is_blank_or_null(c) (!(c) || is_blank(c))
@@ -793,7 +801,7 @@ wr_float(pariout_t *T, GEN x)
   {
     GEN arrondi = cgetr(3);
     ex = expo(x) - (long)((((double)BITS_IN_LONG)/pariK)*dec+2);
-    arrondi[1] = evalsigne(1)|evalexpo(ex);
+    arrondi[1] = (x[1] & ~EXPOBITS) | evalexpo(ex);
     arrondi[2] = x[2]; x = addrr(x,arrondi);
   }
   ex = expo(x); e = bit_accuracy(lg(x)); /* significant bits */
@@ -1340,7 +1348,7 @@ wr_texnome(pariout_t *T, GEN a, char *v, long d)
     }
     if (d)
     {
-      if (under_texmacs) pariputs("\\*");
+      if (GP_DATA && (GP_DATA->flags & TEXMACS)) pariputs("\\*");
       texnome(v,d);
     }
   }
@@ -1384,7 +1392,7 @@ wr_lead_texnome(pariout_t *T, GEN a, char *v, long d, int nosign)
     }
     if (d)
     {
-      if (under_texmacs) pariputs("\\*");
+      if (GP_DATA && (GP_DATA->flags & TEXMACS)) pariputs("\\*");
       texnome(v,d);
     }
   }
@@ -1928,11 +1936,147 @@ texi(GEN g, pariout_t *T, int nosign)
   pariputc('}');
 }
 
+/*******************************************************************/
+/**                                                               **/
+/**                          GP OUTPUT                            **/
+/**                                                               **/
+/*******************************************************************/
+
+/* EXTERNAL PRETTYPRINTER */
+
+/* Wait for prettinprinter to finish, to prevent new prompt from overwriting
+ * the output.  Fill the output buffer, wait until it is read.
+ * Better than sleep(2): give possibility to print */
+static void
+prettyp_wait(void)
+{
+  char *s = "                                                     \n";
+  int i = 400;
+
+  pariputs("\n\n"); pariflush(); /* start translation */
+  while (--i) pariputs(s);
+  pariputs("\n"); pariflush();
+}
+
+/* initialise external prettyprinter (tex2mail) */
+static int
+prettyp_init(void)
+{
+  gp_pp *pp = GP_DATA->pp;
+  if (!pp->cmd) return 0;
+  if (!pp->file)
+    pp->file = try_pipe(pp->cmd, mf_OUT | mf_TEST);
+  if (pp->file) return 1;
+
+  err(warner,"broken prettyprinter: '%s'",pp->cmd);
+  free(pp->cmd); pp->cmd = NULL; return 0;
+}
+
+/* n = history number. if n = 0 no history */
+static int
+tex2mail_output(GEN z, long n)
+{
+  pariout_t T = *(GP_DATA->fmt); /* copy */
+  FILE *o_out;
+  
+  if (!prettyp_init()) return 0;
+  o_out = pari_outfile; /* save state */
+
+  /* Emit first: there may be lines before the prompt */
+  if (n) term_color(c_OUTPUT);
+  pariflush();
+  pari_outfile = GP_DATA->pp->file->file;
+  T.prettyp = f_TEX;
+
+  /* history number */
+  if (n)
+  {
+    char s[128];
+    if (*term_get_color(c_HIST) || *term_get_color(c_OUTPUT))
+    {
+      char col1[80];
+      strcpy(col1, term_get_color(c_HIST));
+      sprintf(s, "\\LITERALnoLENGTH{%s}\\%%%ld =\\LITERALnoLENGTH{%s} ",
+              col1, n, term_get_color(c_OUTPUT));
+    }
+    else
+      sprintf(s, "\\%%%ld = ", n);
+    pariputs_opt(s);
+  }
+  /* output */
+  gen_output(z, &T);
+
+  /* flush and restore */
+  prettyp_wait();
+  pari_outfile = o_out;
+  if (n) term_color(c_NONE);
+  return 1;
+}
+
+/* TEXMACS */
+
+static void
+texmacs_output(GEN z, long n)
+{
+  pariout_t T = *(GP_DATA->fmt); /* copy */
+  char *sz;
+
+  T.prettyp = f_TEX;
+  T.fieldw = 0;
+  sz = GENtostr0(z, &T, &gen_output);
+  printf("%clatex:", DATA_BEGIN);
+  if (n)
+    printf("\\magenta\\%%%ld = $\\blue ", n);
+  else
+    printf("$\\blue ");
+  printf("%s$%c", sz,DATA_END); free(sz);
+  fflush(stdout);
+}
+
+/* REGULAR */
+
+static void
+normal_output(GEN z, long n)
+{
+  long l = 0;
+  /* history number */
+  if (n)
+  {
+    char s[64];
+    term_color(c_HIST);
+    sprintf(s, "%%%ld = ", n);
+    pariputs_opt(s);
+    l = strlen(s);
+  }
+  /* output */
+  term_color(c_OUTPUT);
+  if (GP_DATA->lim_lines)
+    lim_lines_output(z, GP_DATA->fmt, l, GP_DATA->lim_lines);
+  else
+    gen_output(z, GP_DATA->fmt);
+  term_color(c_NONE); pariputc('\n');
+}
+
+void
+gp_output(GEN z, gp_data *G)
+{
+  if (G->flags & TEST) {
+    init80(0);
+    gen_output(z, G->fmt); pariputc('\n');
+  }
+  else if (G->flags & TEXMACS)
+    texmacs_output(z, G->hist->total);
+  else if (G->fmt->prettyp != f_PRETTY || !tex2mail_output(z, G->hist->total))
+    normal_output(z, G->hist->total);
+  pariflush();
+}
+
 /*******************************************************************/
 /**                                                               **/
 /**                        USER OUTPUT FUNCTIONS                  **/
 /**                                                               **/
 /*******************************************************************/
+
 void
 gen_output(GEN x, pariout_t *T)
 {
@@ -1963,9 +2107,9 @@ _initout(pariout_t *T, char f, long sigd, long sp, long fieldw, int prettyp)
 }
 
 void
-bruteall(GEN g, char f, long d, long flag)
+bruteall(GEN g, char f, long d, long sp)
 {
-  pariout_t T; _initout(&T,f,d,flag,0, f_RAW);
+  pariout_t T; _initout(&T,f,d,sp,0, f_RAW);
   gen_output(g, &T);
 }
 
@@ -2231,6 +2375,22 @@ killallfiles(int leaving)
   infile = stdin;
 }
 
+static int
+ok_pipe(FILE *f)
+{
+  if (DEBUGFILES) fprintferr("I/O: checking output pipe...\n");
+  CATCH(CATCH_ALL) {
+    CATCH_RELEASE(); return 0;
+  }
+  TRY {
+    int i;
+    fprintf(f,"\n\n"); fflush(f);
+    for (i=1; i<1000; i++) fprintf(f,"                  \n");
+    fprintf(f,"\n"); fflush(f);
+  } ENDCATCH;
+  return 1;
+}
+
 pariFILE *
 try_pipe(char *cmd, int fl)
 {
@@ -2239,7 +2399,7 @@ try_pipe(char *cmd, int fl)
 #else
   FILE *file;
   char *f;
-  VOLATILE int flag = fl;
+  int flag = fl;
 
 #  ifdef __EMX__
   if (_osmode == DOS_MODE) /* no pipes under DOS */
@@ -2257,20 +2417,7 @@ try_pipe(char *cmd, int fl)
   {
     file = (FILE *) popen(cmd, (flag & mf_OUT)? "w": "r");
     if (flag & mf_OUT) flag |= mf_PERM;
-    if (flag & (mf_TEST | mf_OUT))
-    {
-      jmp_buf env;
-      void *c;
-      int i;
-      if (DEBUGFILES) fprintferr("I/O: checking output pipe...\n");
-      if (setjmp(env)) return NULL;
-
-      c = err_catch(-1, env, NULL);
-      fprintf(file,"\n\n"); fflush(file);
-      for (i=1; i<1000; i++) fprintf(file,"                  \n");
-      fprintf(file,"\n"); fflush(file);
-      err_leave(&c);
-    }
+    if ((flag & (mf_TEST | mf_OUT)) && !ok_pipe(file)) return NULL;
     f = cmd;
   }
   if (!file) err(talker,"[pipe:] '%s' failed",cmd);
@@ -2342,7 +2489,6 @@ os_getenv(char *s)
 /**                                                               **/
 /*******************************************************************/
 static char *last_filename = NULL;
-static char **dir_list = NULL;
 
 #ifdef HAS_OPENDIR
 #  include <dirent.h>
@@ -2460,6 +2606,18 @@ expand_tilde(char *s)
   return _expand_env(_expand_tilde(s));
 }
 
+void
+delete_dirs(gp_path *p)
+{
+  char **v = p->dirs, **dirs;
+  if (v)
+  {
+    p->dirs = NULL; /* in case of error */
+    for (dirs = v; *dirs; dirs++) free(*dirs);
+    free(v);
+  }
+}
+
 #if defined __EMX__ || defined _WIN32
 #  define PATH_SEPARATOR ';'
 #else
@@ -2467,29 +2625,26 @@ expand_tilde(char *s)
 #endif
 
 void
-gp_expand_path(char *v)
+gp_expand_path(gp_path *p)
 {
-  char **path, **old, *s;
+  char **dirs, *s, *v = p->PATH;
   int i, n = 0;
 
+  delete_dirs(p);
   v = pari_strdup(v);
   for (s=v; *s; s++)
     if (*s == PATH_SEPARATOR) { *s = 0; n++; }
-  path = (char**) gpmalloc((n + 2)*sizeof(char *));
+  dirs = (char**) gpmalloc((n + 2)*sizeof(char *));
 
   for (s=v, i=0; i<=n; i++)
   {
     char *end = s + strlen(s), *f = end;
     while (f > s && *--f == '/') *f = 0;
-    path[i] = expand_tilde(s);
+    dirs[i] = expand_tilde(s);
     s = end + 1; /* next path component */
   }
-  path[i] = NULL; old = dir_list; dir_list = path;
-  if (old)
-  {
-    for (path=old; *path; path++) free(*path);
-    free(old);
-  }
+  free((void*)v);
+  dirs[i] = NULL; p->dirs = dirs;
 }
 
 /* name is a malloc'ed (existing) filename. Accept it as new infile
@@ -2566,9 +2721,9 @@ switchin(char *name0)
   /* if name contains '/',  don't use dir_list */
   s=name; while (*s && *s != '/' && *s != '\\') s++;
   if (*s) { if (try_name(name)) return; }
-  else
+  else if (GP_DATA)
   {
-    char **tmp = dir_list;
+    char **tmp = GP_DATA->path->dirs;
     for ( ; *tmp; tmp++)
     { /* make room for '/' and '\0', try_name frees it */
       s = gpmalloc(2 + strlen(*tmp) + strlen(name));
@@ -2837,6 +2992,86 @@ readbin(char *name, FILE *f)
   return x;
 }
 
+/*******************************************************************/
+/**                                                               **/
+/**                             GP I/O                            **/
+/**                                                               **/
+/*******************************************************************/
+/* print a sequence of (NULL terminated) GEN */
+void
+print0(GEN *g, long flag)
+{
+  pariout_t T = GP_DATA? *(GP_DATA->fmt): DFLT_OUTPUT; /* copy */
+  T.prettyp = flag;
+  for( ; *g; g++)
+    if (typ(*g)==t_STR)
+      pariputs(GSTR(*g)); /* text surrounded by "" otherwise */
+    else
+      gen_output(*g, &T);
+}
+
+#define PR_NL() {added_newline = 1; pariputc('\n'); pariflush(); }
+#define PR_NO() {added_newline = 0; pariflush(); }
+void print   (GEN *g) { print0(g, f_RAW);       PR_NL(); }
+void printp  (GEN *g) { print0(g, f_PRETTYOLD); PR_NL(); }
+void printtex(GEN *g) { print0(g, f_TEX);       PR_NL(); }
+void print1  (GEN *g) { print0(g, f_RAW);       PR_NO(); }
+void printp1 (GEN *g) { print0(g, f_PRETTYOLD); PR_NO(); }
+
+void error0(GEN *g) { err(user, g); }
+
+static char *
+wr_check(char *s) {
+  char *t = expand_tilde(s);
+  if (GP_DATA && GP_DATA->flags & SECURE)
+  {
+    fprintferr("[secure mode]: about to write to '%s'. OK ? (^C if not)\n",t);
+    hit_return();
+  }
+  return t;
+}
+
+static void wr_init(char *s)    { char *t=wr_check(s); switchout(t);   free(t);}
+void gpwritebin(char *s, GEN x) { char *t=wr_check(s); writebin(t, x); free(t);}
+
+#define WR_NL() {pariputc('\n'); pariflush(); switchout(NULL); }
+#define WR_NO() {pariflush(); switchout(NULL); }
+void write0  (char *s, GEN *g) { wr_init(s); print0(g, f_RAW); WR_NL(); }
+void writetex(char *s, GEN *g) { wr_init(s); print0(g, f_TEX); WR_NL(); }
+void write1  (char *s, GEN *g) { wr_init(s); print0(g, f_RAW); WR_NO(); }
+
+/*******************************************************************/
+/**                                                               **/
+/**                       HISTORY HANDLING                        **/
+/**                                                               **/
+/*******************************************************************/
+/* history management function:
+ *   p > 0, called from %p
+ *   p <= 0, called from %` (p backquotes, possibly 0) */
+GEN
+gp_history(gp_hist *H, long p, char *old, char *entry)
+{
+  GEN z;
+
+  if (p <= 0) p += H->total; /* count |p| entries starting from last */
+  if ((ulong)p > H->total)
+    err(talker2, "I can't see into the future", old, entry);
+
+  z = H->res[ (p-1) % H->size ];
+  if (!z || p <= 0 || p <= (long)(H->total - H->size))
+    err(talker2, "I can't remember before the big bang", old, entry);
+  return z;
+}
+
+GEN
+set_hist_entry(gp_hist *H, GEN x)
+{
+  int i = H->total % H->size;
+  H->total++;
+  if (H->res[i]) gunclone(H->res[i]);
+  return H->res[i] = gclone(x);
+}
+
 /*******************************************************************/
 /**                                                               **/
 /**                       TEMPORARY FILES                         **/
