@@ -1,4 +1,4 @@
-/* $Id: init.c,v 1.149 2002/06/09 22:45:47 karim Exp $
+/* $Id: init.c,v 1.169 2002/09/12 18:45:06 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -37,7 +37,6 @@ byteptr diffptr;
 char    *current_logfile, *current_psfile;
 int     gp_colors[c_LAST];
 int     disable_color = 1, added_newline = 1;
-int     under_emacs = 0, under_texmacs = 0;
 
 int     functions_tblsz = 135; /* size of functions_hash          */
 entree  **varentries;
@@ -45,12 +44,14 @@ entree  **varentries;
 void    *global_err_data;
 jmp_buf environnement;
 long    *ordvar;
-ulong   DEBUGFILES,DEBUGLEVEL,DEBUGMEM,compatible;
-ulong   prec,precdl;
+ulong   DEBUGFILES, DEBUGLEVEL, DEBUGMEM, compatible;
+ulong   prec, precdl;
 ulong   init_opts = INIT_JMPm | INIT_SIGm;
 gpmem_t bot = 0, top = 0, avma;
 size_t memused;
 
+gp_data *GP_DATA = NULL;
+
 void *foreignHandler; 	              /* Handler for foreign commands.   */
 char foreignExprSwitch = 3; 	      /* Just some unprobable char.      */
 GEN  (*foreignExprHandler)(char*);    /* Handler for foreign expressions.*/
@@ -58,10 +59,10 @@ entree * (*foreignAutoload)(char*, long); /* Autoloader                      */
 void (*foreignFuncFree)(entree *);    /* How to free external entree.    */
 
 int  (*default_exception_handler)(long);
-GEN  (*gp_history_fun)(long, long, char *, char *);
 int  (*whatnow_fun)(char *, int);
 pariout_t DFLT_OUTPUT = { 'g', 0, -1, 1, 0, f_RAW };
 
+extern void  delete_dirs(gp_path *p);
 extern void  initout(int initerr);
 extern int   term_width(void);
 
@@ -72,16 +73,15 @@ extern int   term_width(void);
   *PL_sv_yes, *PL_sv_no, *PL_curpad, *PL_op
 extern int EXTERM_DLL_DPES;
 int EXTERM_DLL_DPES;
-#endif	/* defined BOTH_GNUPLOT_AND_X11 */ 
+#endif	/* defined BOTH_GNUPLOT_AND_X11 */
 
-typedef struct cell {
-  void *env;
-  void *data;
+typedef struct {
+  jmp_buf *penv;
   long flag;
 } cell;
 
 static stack *err_catch_stack = NULL;
-static long *err_catch_array;
+static char **dft_handler;
 
 void
 push_stack(stack **pts, void *a)
@@ -426,11 +426,11 @@ pari_sig_init(void (*f)(int))
 }
 
 static void
-reset_traps(int warn)
+reset_traps()
 {
   long i;
-  if (warn) err(warner,"missing cell in err_catch_stack. Resetting all traps");
-  for (i=0; i <= noer; i++) err_catch_array[i] = 0;
+  if (DEBUGLEVEL) err(warner,"Resetting all traps");
+  for (i=0; i <= noer; i++) dft_handler[i] = NULL;
 }
 
 static void
@@ -493,7 +493,7 @@ init_stack(size_t size)
  * appel  pari_addfunctions si on ajoute d'autres fonctions au pool de base.
  */
 void
-pari_init(size_t parisize, long maxprime)
+pari_init(size_t parisize, ulong maxprime)
 {
   long i;
 
@@ -542,10 +542,9 @@ pari_init(size_t parisize, long maxprime)
   for (i = 0; i < functions_tblsz; i++) members_hash[i] = NULL;
   gp_init_entrees(pari_membermodules, members_hash, 1);
 
-  gp_history_fun = NULL;
   whatnow_fun = NULL;
-  err_catch_array = (long *) gpmalloc((noer + 1) *sizeof(long));
-  reset_traps(0);
+  dft_handler = (char **) gpmalloc((noer + 1) *sizeof(char *));
+  reset_traps();
   default_exception_handler = NULL;
 
   (void)manage_var(2,NULL); /* init nvar */
@@ -553,6 +552,33 @@ pari_init(size_t parisize, long maxprime)
   try_to_recover=1;
 }
 
+static void
+delete_hist(gp_hist *h)
+{
+  if (h->res) free((void*)h->res);
+}
+static void
+delete_pp(gp_pp *p)
+{
+  if (p->cmd) free((void*)p->cmd);
+}
+static void
+delete_path(gp_path *p)
+{
+  delete_dirs(p);
+  free((void*)p->PATH);
+}
+
+static void
+free_gp_data(gp_data *D)
+{
+  if (!D) return;
+  delete_hist(D->hist);
+  delete_path(D->path);
+  delete_pp(D->pp);
+  if (D->help) free((void*)D->help);
+}
+
 void
 freeall(void)
 {
@@ -562,17 +588,15 @@ freeall(void)
   while (delete_var()) /* empty */;
   for (i = 0; i < functions_tblsz; i++)
   {
-    for (ep = functions_hash[i]; ep; ep = ep1)
-    {
-      ep1 = ep->next; freeep(ep);
-    }
-    for (ep = members_hash[i]; ep; ep = ep1)
-    {
-      ep1 = ep->next; freeep(ep);
-    }
+    for (ep = functions_hash[i]; ep; ep = ep1) { ep1 = ep->next; freeep(ep); }
+    for (ep =   members_hash[i]; ep; ep = ep1) { ep1 = ep->next; freeep(ep); }
   }
-  free((void*)varentries); free((void*)ordvar); free((void*)polvar);
-  free((void*)polx[MAXVARN]); free((void*)polx); free((void*)polun);
+  free((void*)varentries);
+  free((void*)ordvar);
+  free((void*)polvar);
+  free((void*)polx[MAXVARN]);
+  free((void*)polx);
+  free((void*)polun);
   free((void*)primetab);
   free((void*)universal_constants);
 
@@ -580,12 +604,12 @@ freeall(void)
   while (cur_bloc) { *cur_bloc=0; killbloc(cur_bloc); }
   killallfiles(1);
   free((void *)functions_hash);
-  free((void *)bot); free((void *)diffptr);
+  free((void *)bot);
+  free((void *)diffptr);
   free(current_logfile);
   free(current_psfile);
 
-  if (gp_history_fun)
-    (void)gp_history_fun(0,-1,NULL,NULL);
+  free_gp_data(GP_DATA);
 }
 
 GEN
@@ -633,12 +657,32 @@ newbloc(long n)
   return cur_bloc = x;
 }
 
-void
-killbloc0(GEN x, int inspect)
+/* recursively look for clones in the container and kill them */
+static void
+inspect(GEN x)
 {
-  long tx,lx,l,i,j;
-  GEN p1;
+  long i, lx;
+  switch(typ(x)) /* HACK: if x is not a GEN, we have typ(x)=0 */
+  {
+    case t_VEC: case t_COL: case t_MAT:
+      lx = lg(x);
+      for (i=1;i<lx;i++) inspect((GEN)x[i]);
+      break;
+    case t_LIST:
+      lx = lgef(x);
+      for (i=2;i<lx;i++) inspect((GEN)x[i]);
+      break;
+  }
+  if (isclone(x)) gunclone(x); /* Don't inspect here! components are dead */
+}
 
+/* If insp is set, recursively inspect x, killing all clones found. The GP
+ * expression x[i] = y is implemented as x[i] := gclone(y) and we need to
+ * reclaim the memory. Useless to inspect when x does not correspond to a GP
+ * variable [not dangerous, though] */
+void
+killbloc0(GEN x, int insp)
+{
   if (!x || isonstack(x)) return;
   if (bl_next(x)) bl_prev(bl_next(x)) = bl_prev(x);
   else
@@ -649,45 +693,10 @@ killbloc0(GEN x, int inspect)
   if (bl_prev(x)) bl_next(bl_prev(x)) = bl_next(x);
   if (DEBUGMEM > 2)
     fprintferr("killing bloc (no %ld): %08lx\n", bl_num(x), x);
-  if (inspect)
-  {
-    /* FIXME: SIGINT should be blocked at this point */
-    tx=typ(x); /* if x is not a GEN, we will have tx=0 */
-    if (is_vec_t(tx))
-    {
-      lx = lg(x);
-      for (i=1;i<lx;i++)
-      {
-        p1=(GEN)x[i];
-        if (isclone(p1)) killbloc(p1);
-      }
-    }
-    else if (tx==t_MAT)
-    {
-      lx = lg(x);
-      if (lx>1)
-      {
-        l=lg(x[1]);
-        if (l>1)
-          for (i=1;i<lx;i++)
-            for (j=1;j<l;j++)
-            {
-              p1=gmael(x,i,j);
-              if (isclone(p1)) killbloc(p1);
-            }
-      }
-    }
-    else if (tx==t_LIST)
-    {
-      lx = lgef(x);
-      for (i=2;i<lx;i++)
-      {
-        p1=(GEN)x[i];
-        if (isclone(p1)) killbloc(p1);
-      }
-    }
-    unsetisclone(x);
-    /* FIXME: SIGINT should be released here */
+  if (insp)
+  { /* FIXME: SIGINT should be blocked until inspect() returns */
+    unsetisclone(x); /* important: oo recursion otherwise */
+    inspect(x);
   }
   free((void *)bl_base(x));
 }
@@ -925,108 +934,65 @@ errcontext(char *msg, char *s, char *entry)
 }
 
 void *
-err_catch(long errnum, jmp_buf env, void *data)
+err_catch(long errnum, jmp_buf *penv)
 {
-  cell *v = (cell*)gpmalloc(sizeof(cell));
-  if (errnum < 0) errnum = noer;
-  v->data = data;
-  v->env  = env;
+  cell *v;
+  /* for fear of infinite recursion... */
+  if (errnum == memer) err(talker, "can't trap memory errors");
+  if (errnum == CATCH_ALL) errnum = noer;
+  if (errnum > noer) err(talker, "no such error number: %ld", errnum);
+  v = (cell*)gpmalloc(sizeof(cell));
+  v->penv  = penv;
   v->flag = errnum;
-  err_catch_array[errnum]++;
   push_stack(&err_catch_stack, (void*)v);
   return (void*)v;
 }
 
-/* reset traps younger than V (included) */
-void
-err_leave(void **V)
+static void
+pop_catch_cell(stack **s)
 {
-  cell *t, *v = (cell*)*V;
-
-  for (;;)
+  cell *c = (cell*)pop_stack(s);
+  if (c)
   {
-    t = (cell*)pop_stack(&err_catch_stack);
-    if (t == v || !t) break;
-    err_catch_array[t->flag]--;
-    free(t);
+    free(c);
   }
-  if (!t) reset_traps(1);
-  err_catch_array[v->flag]--;
-  free(v);
 }
 
-/* get last (most recent) handler for error n */
+/* reset traps younger than v (included).
+ * Note the address of v is passed instead because we do not want compiler
+ * to put v into a register (could be clobbered by longjmp) */
+void
+err_leave(void **pv)
+{
+  while (err_catch_stack)
+  {
+    cell *t = (cell*)err_catch_stack->value;
+    pop_catch_cell(&err_catch_stack);
+    if (t == (cell*)(*pv)) return;
+  }
+  reset_traps();
+}
+
+/* Get last (most recent) handler for error n (or generic noer) killing all
+ * more recent non-applicable handlers (now obsolete) */
 static cell *
 err_seek(long n)
 {
-  stack *s = err_catch_stack;
-  cell *t = NULL;
-
-  for (;s; s = s->prev)
+  while (err_catch_stack)
   {
-    t = (cell*)s->value;
-    if (!t || t->flag == n) break;
+    cell *t = err_catch_stack->value;
+    if (t->flag == n || t->flag == noer) return t;
+    pop_catch_cell(&err_catch_stack);
   }
-  if (!t) reset_traps(1);
-  return t;
+  return NULL;
 }
 
-/* kill last handler for error n */
-void
-err_leave_default(long n)
-{
-  stack *s = err_catch_stack, *lasts = NULL;
-  cell *c;
-
-  if (n < 0) n = noer;
-  if (!s || !err_catch_array[n]) return;
-  for (;s; s = s->prev)
-  {
-    c = (cell*)s->value;
-    if (c->flag == n)
-    { /* kill */
-      stack *v = s->prev;
-      free((void*)s); s = v;
-      if (lasts) lasts->prev = s;
-      break;
-    }
-    else lasts = s;
-  }
-  if (!lasts)
-  {
-    err_catch_stack = s;
-    if (!s) reset_traps(0);
-  }
-}
-
-/* untrapped error: remove all traps depending from a longjmp */
+/* untrapped error: kill all error handlers */
 void
 err_clean(void)
 {
-  stack *s = err_catch_stack, *lasts = NULL;
-  cell *c;
-
-  if (!s) return;
-  while (s)
-  {
-    c = (cell*)s->value;
-    if (c->env)
-    { /* kill */
-      stack *v = s->prev;
-      free((void*)s); s = v;
-      if (lasts) lasts->prev = s;
-    }
-    else
-    { /* preserve */
-      if (lasts) lasts->prev = s; else err_catch_stack = s;
-      lasts = s; s = s->prev;
-    }
-  }
-  if (!lasts)
-  {
-    err_catch_stack = NULL;
-    reset_traps(0);
-  }
+  while (err_catch_stack)
+    pop_catch_cell(&err_catch_stack);
 }
 
 static int
@@ -1041,10 +1007,10 @@ err_recover(long numerr)
   initout(0);
   disable_dbg(-1);
   killallfiles(0);
+  err_clean();
 
   if (pariErr->die) pariErr->die();    /* Caller wants to catch exceptions? */
   global_err_data = NULL;
-  err_clean();
   fprintferr("\n"); flusherr();
   if (!environnement) exit(1);
 
@@ -1060,28 +1026,24 @@ err(long numerr, ...)
   int ret = 0;
   PariOUT *out = pariOut;
   va_list ap;
-  cell *trapped = NULL;
 
   va_start(ap,numerr);
 
+  global_err_data = NULL;
   if (err_catch_stack && !is_warn(numerr))
   {
-    int trap = 0;
-    if (numerr != memer)
-    { /* for fear of infinite recursion don't trap memory errors */
-      if (err_catch_array[numerr]) trap = numerr;
-      else if (err_catch_array[noer]) trap = noer;
+    cell *trapped = NULL;
+    if ( (trapped = err_seek(numerr)) )
+    {
+      jmp_buf *e = trapped->penv;
+      if (numerr == invmoder)
+      {
+        (void)va_arg(ap, char*); /* junk 1st arg */
+        global_err_data = (void*)va_arg(ap, GEN);
+      }
+      longjmp(*e, numerr);
     }
-    if (trap) trapped = err_seek(trap); else err_clean();
   }
-  if (trapped)
-  { /* all errors (noer), or numerr individually trapped */
-    void *e = trapped->env;
-    global_err_data = trapped->data;
-    if (e) longjmp(e, numerr);
-  }
-  else
-    global_err_data = NULL;
 
   if (!added_newline) { pariputc('\n'); added_newline=1; }
   pariflush(); pariOut = pariErr;
@@ -1118,6 +1080,12 @@ err(long numerr, ...)
         errcontext(s,ch1,va_arg(ap,char *));
     }
   }
+  else if (numerr == user)
+  {
+    GEN *g = va_arg(ap, GEN*);
+    pariputsf("  ###   user error: ");
+    print0(g, f_RAW);
+  }
   else
   {
     pariputsf("  ***   %s", errmessage[numerr]);
@@ -1178,11 +1146,69 @@ err(long numerr, ...)
     fprintferr("  [hint] you can increase GP stack with allocatemem()\n");
   }
   pariOut = out;
-  if (ret || (trapped && default_exception_handler &&
-              default_exception_handler(numerr))) { flusherr(); return; }
+  if (ret) { flusherr(); return; }
+  if (default_exception_handler)
+  {
+    if (dft_handler[numerr])
+      global_err_data = dft_handler[numerr];
+    else
+      global_err_data = dft_handler[noer];
+    if (default_exception_handler(numerr)) { flusherr(); return; }
+  }
   err_recover(numerr);
 }
 
+static char *BREAK_LOOP = "";
+
+static void
+kill_dft_handler(int numerr)
+{
+  char *s = dft_handler[numerr];
+  if (s && s != BREAK_LOOP) free(s);
+  dft_handler[numerr] = NULL;
+}
+
+/* Try f (trapping error e), recover using r (break_loop, if NULL) */
+GEN
+trap0(char *e, char *r, char *f)
+{
+  long numerr = CATCH_ALL;
+  char *F;
+       if (!strcmp(e,"errpile")) numerr = errpile;
+  else if (!strcmp(e,"typeer")) numerr = typeer;
+  else if (!strcmp(e,"gdiver")) numerr = gdiver;
+  else if (!strcmp(e,"invmoder")) numerr = invmoder;
+  else if (!strcmp(e,"accurer")) numerr = accurer;
+  else if (!strcmp(e,"archer")) numerr = archer;
+  else if (*e) err(impl,"this trap keyword");
+  /* TO BE CONTINUED */
+
+  if (f && r)
+  { /* explicit recovery text */
+    char *a = get_analyseur();
+    gpmem_t av = avma;
+    VOLATILE GEN x;
+
+    CATCH(numerr) { x = NULL; }
+    TRY { x = lisseq(f); } ENDCATCH;
+    if (!x) { avma = av; x = lisseq(r); }
+    set_analyseur(a); return x;
+  }
+
+  F = f? f: r; /* define a default handler */
+ /* will execute F (break loop if F = NULL), then jump to environnement */
+  if (numerr == CATCH_ALL) numerr = noer;
+  kill_dft_handler(numerr);
+  if (!F)
+    dft_handler[numerr] = BREAK_LOOP;
+  else if (*F && (*F != '"' || F[1] != '"'))
+  {
+    F = pari_strdup(F);
+    dft_handler[numerr] = F;
+  }
+  return gnil;
+}
+
 /*******************************************************************/
 /*                                                                 */
 /*                       CLONING & COPY                            */
@@ -1219,8 +1245,7 @@ gcopy(GEN x)
     for (   ; i<lontyp2[tx]; i++) copyifstack(x[i],y[i]);
     for (   ; i<lx;          i++) y[i]=lcopy((GEN)x[i]);
   }
-  /* unsetisclone(y); useless because gunclone checks isonstack */
-  return y;
+  unsetisclone(y); return y;
 }
 
 GEN
@@ -1239,7 +1264,7 @@ gcopy_i(GEN x, long lx)
     for (   ; i<lontyp2[tx]; i++) copyifstack(x[i],y[i]);
     for (   ; i<lx;          i++) y[i]=lcopy((GEN)x[i]);
   }
-  return y;
+  unsetisclone(y); return y;
 }
 
 GEN
@@ -1390,7 +1415,7 @@ void
 shiftaddress(GEN x, long dec)
 {
   long i,lx,tx;
-  
+
   tx = typ(x);
   if (is_recursive_t(tx))
   {
@@ -1470,7 +1495,7 @@ gerepilemany(gpmem_t av, GEN* gptr[], int n)
   free(l);
 }
 
-void 
+void
 gerepileall(gpmem_t av, int n, ...)
 {
   GENbin **l = (GENbin**)gpmalloc(n*sizeof(GENbin*));
@@ -1789,9 +1814,9 @@ TIMER(pari_timer *T)
   return _get_time(T, clock(), CLOCKS_PER_SEC);
 }
 #endif
+void
+TIMERstart(pari_timer *T) { (void)TIMER(T); }
 
-long
-gptimer(void) { static pari_timer T; return TIMER(&T);}
 long
 timer(void)   { static pari_timer T; return TIMER(&T);}
 long
@@ -1888,6 +1913,8 @@ geni(void) { return gi; }
  *     The unquoted components can be of any pari type (converted according to
  *     the current output format)
  *  s* any number of strings (see s)
+ *  M  Mnemonic or a flag (converted to a long); description follows
+ *	 after \n at the end of the argument description
  *  D  Has a default value. Format is "Dvalue,type," (the ending comma is
  *     mandatory). Ex: D0,L, (arg is long, 0 by default).
  *     Special syntax:
@@ -2144,7 +2171,7 @@ entree functions_basic[]={
 {"listput",25,(void*)listput,8,"GGD0,L,"},
 {"listsort",99,(void*)listsort,8,"GD0,L,"},
 {"lngamma",1,(void*)glngamma,3,"Gp"},
-{"log",99,(void*)log0,3,"GD0,L,p"},
+{"log",99,(void*)log0,99,"GD0,M,p\nAGM==1"},
 {"matadjoint",18,(void*)adj,8,"G"},
 {"matalgtobasis",2,(void*)matalgtobasis,6,"GG"},
 {"matbasistoalg",2,(void*)matbasistoalg,6,"GG"},
@@ -2197,7 +2224,7 @@ entree functions_basic[]={
 {"nfeltdivrem",3,(void*)nfdivres,6,"GGG"},
 {"nfeltmod",3,(void*)nfmod,6,"GGG"},
 {"nfeltmul",3,(void*)element_mul,6,"GGG"},
-{"nfeltmulmodpr",4,(void*)element_mulmodpr2,6,"GGGG"},
+{"nfeltmulmodpr",4,(void*)element_mulmodpr,6,"GGGG"},
 {"nfeltpow",3,(void*)element_pow,6,"GGG"},
 {"nfeltpowmodpr",4,(void*)element_powmodpr,6,"GGGG"},
 {"nfeltreduce",3,(void*)element_reduce,6,"GGG"},
@@ -2320,7 +2347,8 @@ entree functions_basic[]={
 {"rnfidealup",2,(void*)rnfidealup,6,"GG"},
 {"rnfinit",29,(void*)rnfinitalg,6,"GGp"},
 {"rnfisfree",20,(void*)rnfisfree,6,"lGG"},
-{"rnfisnorm",99,(void*)rnfisnorm,6,"GGGD1,L,p"},
+{"rnfisnorm",99,(void*)rnfisnorm,6,"GGD0,L,"},
+{"rnfisnorminit",99,(void*)rnfisnorminit,6,"GGD2,L,"},
 {"rnfkummer",99,(void*)rnfkummer,6,"GDGD0,L,p"},
 {"rnflllgram",99,(void*)rnflllgram,6,"GGGp"},
 {"rnfnormgroup",2,(void*)rnfnormgroup,6,"GG"},
