@@ -1,4 +1,4 @@
-/* $Id: elliptic.c,v 1.62 2002/06/08 14:02:31 karim Exp $
+/* $Id: elliptic.c,v 1.68 2002/09/12 00:12:14 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1037,6 +1037,10 @@ static GEN
 reduce_z(GEN z, long prec, SL2_red *T)
 {
   GEN Z = gdiv(z, T->W2);
+  long t = typ(z);
+
+  if (!is_scalar_t(t) || t == t_INTMOD || t == t_PADIC || t == t_POLMOD)
+    err(typeer,"reduction mod SL2 (reduce_z)");
   T->x = ground(gdiv(gimag(Z), gimag(T->tau)));
   Z = gsub(Z, gmul(T->x,T->tau));
   T->y = ground(greal(Z));
@@ -1492,11 +1496,11 @@ addsell_part2(GEN e, GEN z1, GEN z2, GEN p, GEN p2inv)
 }
 
 static GEN
-negsell(GEN f)
+negsell(GEN f, GEN p)
 {
   GEN g = cgetg(3, t_VEC);
   g[1] = f[1];
-  g[2] = lnegi((GEN)f[2]);
+  g[2] = signe(f[2])? lsubii(p, (GEN)f[2]): f[2];
   return g;
 }
 
@@ -1510,7 +1514,7 @@ powsell(GEN e, GEN z, GEN n, GEN p)
   if (!s || !z) return NULL;
   if (s < 0)
   {
-    z = negsell(z);
+    z = negsell(z, p);
     n = negi(n);
   }
   if (is_pm1(n)) return z;
@@ -1604,7 +1608,7 @@ apell1(GEN e, GEN p)
     p1 = gcopy(fh);
     if (s < 3)
     { /* we're nearly done: naive search */
-      GEN q1 = p1, mf = negsell(f); /* -F */
+      GEN q1 = p1, mf = negsell(f, p); /* -F */
       for (i=1;; i++)
       {
         p1 = addsell(cp4,p1, f,p); /* h.f + i.F */
@@ -1625,7 +1629,7 @@ apell1(GEN e, GEN p)
       p1 = addsell(cp4,p1,f,p); /* h.f + i.F */
       if (!p1) { h = addii(h, mulsi(i,B)); goto FOUND; }
     }
-    mfh = negsell(fh);
+    mfh = negsell(fh, p);
     fg = addsell(cp4,p1,mfh,p); /* h.f + nb.F - h.f = nb.F */
     if (!fg) { h = mulsi(nb,B); goto FOUND; }
     u = cgetg(nb+1, t_VEC);
@@ -2549,13 +2553,13 @@ localred_carac_23(GEN e, long p, int minim)
     if (smodis((GEN)e[5], p2))
       return localred_result(nudelta, 2, 1, v);
         /* II   */
-    p3 = p * p * p;
+    p3 = p2 * p;
     if (smodis((GEN)e[9], p3))
       return localred_result(nudelta - 1, 3, 2, v);
         /* III  */
     if (smodis((GEN)e[8], p3))
     {
-      if (smodis((GEN)e[8], (p==2)? 8: 27) == p2)
+      if (smodis((GEN)e[8], (p==2)? 32: 27) == p2)
         c = 3;
       else
         c = 1;
@@ -2584,7 +2588,7 @@ localred_carac_23(GEN e, long p, int minim)
             /* p | a1; p^2  | a2, a3; p^3 | a4; p^4 | a6 */
         nu = 1;
         pk  = stoi(p2);
-        p2k = stoi(p * p * p * p);
+        p2k = stoi(p2 * p2);
         for(;;)
         {
           be =  aux2((GEN)e[3], p, pk);
@@ -2625,7 +2629,7 @@ localred_carac_23(GEN e, long p, int minim)
             /* IV*  */
         if (theroot) cumule(&v, &e, gun, gzero, gzero, stoi(theroot*p*p));
             /* p | a1; p^2 | a2; p^3 | a3, a4; p^5 | a6 */
-        p4 = p * p * p * p;
+        p4 = p2 * p2;
         if (smodis((GEN)e[4], p4))
           return localred_result(nudelta - 7, -3, 2, v);
             /* III* */
@@ -3055,7 +3059,7 @@ torsellnagelllutz(GEN e)
 static long
 torsbound(GEN e)
 {
-  long m, b, c, d, prime = 2;
+  long m, b, c, prime = 2;
   gpmem_t av = avma;
   byteptr p = diffptr;
   GEN D = (GEN)e[12];
@@ -3064,8 +3068,7 @@ torsbound(GEN e)
   b = c = m = 0; p++;
   while (m<n)
   {
-    d = *p++; if (!d) err(primer1);
-    prime += d;
+    NEXT_PRIME_VIADIFF_CHECK(prime,p);
     if (smodis(D, prime))
     {
       b = cgcd(b, prime+1 - itos(apell0(e,prime)));
@@ -3085,12 +3088,12 @@ _round(GEN x, long *e)
   return y;
 }
 
-/* Input the curve, a point q of order n (approx.), returns q as a rational
- * point on the curve, or NULL if q is not rational. */
+/* E the curve, w in C/Lambda ~ E of order n, returns q = pointell(w) as a
+ * rational point on the curve, or NULL if q is not rational. */
 static GEN
-torspnt(GEN E, GEN q, long n)
+torspnt(GEN E, GEN w, long n, long prec)
 {
-  GEN p = cgetg(3,t_VEC);
+  GEN p = cgetg(3,t_VEC), q = pointell(E, w, prec);
   long e;
   p[1] = lmul2n(_round(gmul2n((GEN)q[1],2), &e),-2);
   if (e > -5) return NULL;
@@ -3176,8 +3179,9 @@ torselldoud(GEN e)
 {
   long b, i, ord, prec, k = 1;
   gpmem_t av=avma;
-  GEN v,w1,w22,w1j,w12,p,tor1,tor2;
+  GEN v,w,w1,w22,w1j,w12,p,tor1,tor2;
 
+  checkbell(e);
   v = ellintegralmodel(e);
   if (v) e = coordch(e,v);
 
@@ -3191,17 +3195,17 @@ torselldoud(GEN e)
   if (v) v[1] = linv((GEN)v[1]);
   w22 = gmul2n((GEN)e[16],-1);
   if (b % 4)
-  {
+  { /* cyclic of order 1, p, 2p, p <= 5 */
     p = NULL;
     for (i=10; i>1; i--)
     {
       if (b%i != 0) continue;
       w1j = gdivgs(w1,i);
-      p = torspnt(e,pointell(e,w1j,prec),i);
+      p = torspnt(e,w1j,i,prec);
       if (!p && i%2==0)
       {
-        p = torspnt(e,pointell(e,gadd(w22,w1j),prec),i);
-        if (!p) p = torspnt(e,pointell(e,gadd(w22,gmul2n(w1j,1)),prec),i);
+        p = torspnt(e,gadd(w22,w1j),i,prec);
+        if (!p) p = torspnt(e,gadd(w22,gmul2n(w1j,1)),i,prec);
       }
       if (p) { k = i; break; }
     }
@@ -3210,16 +3214,23 @@ torselldoud(GEN e)
 
   ord = 0; tor1 = tor2 = NULL;
   w12 = gmul2n(w1,-1);
-  if ((p = torspnt(e,pointell(e,w12,prec),2)))
+  if ((p = torspnt(e,w12,2,prec)))
   {
     tor1 = p; ord++;
   }
-  if ((p = torspnt(e,pointell(e,w22,prec),2))
-   || (!ord && (p = torspnt(e,pointell(e,gadd(w12,w22),prec),2))))
+  w = w22;
+  if ((p = torspnt(e,w,2,prec)))
   {
     tor2 = p; ord += 2;
   }
-
+  if (!ord)
+  {
+    w = gadd(w12,w22);
+    if ((p = torspnt(e,w,2,prec)))
+    {
+      tor2 = p; ord += 2;
+    }
+  }
   p = NULL;
   switch(ord)
   {
@@ -3228,7 +3239,7 @@ torselldoud(GEN e)
       {
         if (b%i!=0) continue;
         w1j = gdivgs(w1,i);
-        p = torspnt(e,pointell(e,w1j,prec),i);
+        p = torspnt(e,w1j,i,prec);
         if (p) { k = i; break; }
       }
       break;
@@ -3238,20 +3249,20 @@ torselldoud(GEN e)
       {
         if (b%i!=0) continue;
         w1j = gdivgs(w1,i);
-        p = torspnt(e,pointell(e,w1j,prec),i);
+        p = torspnt(e,w1j,i,prec);
         if (!p && i%4==0)
-          p = torspnt(e,pointell(e,gadd(w22,w1j),prec),i);
+          p = torspnt(e,gadd(w22,w1j),i,prec);
         if (p) { k = i; break; }
       }
       if (!p) { p = tor1; k = 2; }
       break;
 
-    case 2: /* 1 point of order 2: w2 / 2 or (w1+w2) / 2 */
+    case 2: /* 1 point of order 2: w = w2/2 or (w1+w2)/2 */
       for (i=5; i>1; i-=2)
       {
         if (b%i!=0) continue;
         w1j = gdivgs(w1,i);
-        p = torspnt(e,pointell(e,gadd(w22,w1j),prec),i+i);
+        p = torspnt(e,gadd(w,w1j),2*i,prec);
         if (p) { k = 2*i; break; }
       }
       if (!p) { p = tor2; k = 2; }
@@ -3262,7 +3273,7 @@ torselldoud(GEN e)
       {
         if (b%(2*i)!=0) continue;
         w1j = gdivgs(w1,i);
-        p = torspnt(e,pointell(e,w1j,prec),i);
+        p = torspnt(e,w1j,i,prec);
         if (p) { k = i; break; }
       }
       if (!p) { p = tor1; k = 2; }
