@@ -1,4 +1,4 @@
-/* $Id: nffactor.c,v 1.85 2002/06/10 17:59:54 karim Exp $
+/* $Id: nffactor.c,v 1.105 2002/09/12 19:30:14 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -21,6 +21,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "pari.h"
 #include "parinf.h"
 
+extern void init_dalloc();
+extern double *dalloc(size_t n);
+extern GEN gmul_mati_smallvec(GEN x, GEN y);
 extern GEN GS_norms(GEN B, long prec);
 extern GEN RXQX_divrem(GEN x, GEN y, GEN T, GEN *pr);
 extern GEN RXQX_red(GEN P, GEN T);
@@ -40,8 +43,8 @@ extern GEN vandermondeinverse(GEN L, GEN T, GEN den, GEN prep);
 extern GEN vconcat(GEN A, GEN B);
 extern int cmbf_precs(GEN q, GEN A, GEN B, long *a, long *b, GEN *qa, GEN *qb);
 extern int isrational(GEN x);
-extern long LLL_check_progress(GEN Bnorm, long n0, GEN m, GEN *CM_L, long *BPF, pari_timer *T, long *ti_LLL);
-extern void remake_GM(GEN nf, long prec, nffp_t *F);
+extern GEN LLL_check_progress(GEN Bnorm, long n0, GEN m, int final, pari_timer *T, long *ti_LLL);
+extern void remake_GM(GEN nf, nffp_t *F, long prec);
 #define RXQX_div(x,y,T) RXQX_divrem((x),(y),(T),NULL)
 #define RXQX_rem(x,y,T) RXQX_divrem((x),(y),(T),ONLY_REM)
 
@@ -50,10 +53,10 @@ static GEN nfsqff(GEN nf,GEN pol,long fl);
 /* for nf_bestlift: reconstruction of algebraic integers known mod \wp^k */
 /* FIXME: assume \wp has degree 1 for now */
 typedef struct {
-  long a;    /* input known mod \wp^a */
-  GEN pa;    /* p^a = Norm(\wp^a)*/
-  GEN den;   /* denom(prk^-1) = p^a */
-  GEN prk;   /* |.|^2 LLL-reduced basis (b_i) of \wp^a  (NOT T2-reduced) */
+  long k;    /* input known mod \wp^k */
+  GEN pk;    /* p^k = Norm(\wp^k)*/
+  GEN den;   /* denom(prk^-1) = p^k */
+  GEN prk;   /* |.|^2 LLL-reduced basis (b_i) of \wp^k  (NOT T2-reduced) */
   GEN iprk;  /* den * prk^-1 */
   GEN GSmin; /* min |b_i^*|^2 */
   GEN ZpProj;/* projector to Zp / \wp^k = Z/p^k  (\wp unramified, degree 1) */
@@ -64,7 +67,7 @@ typedef struct {
 typedef struct /* for use in nfsqff */
 {
   nflift_t *L;
-  GEN nf, pol, fact, res, bound, pr, pa, Br, ZC, dn, polbase, BS_2;
+  GEN nf, pol, fact, res, bound, pr, pk, Br, ZC, dn, polbase, BS_2;
   long hint;
 } nfcmbf_t;
 
@@ -179,7 +182,7 @@ choose_prime(GEN nf, GEN bad, GEN *p, byteptr *PT)
   gpmem_t av = avma;
   for (;;)
   {
-    pp += *pt++; if (!*pt) err(primer1);
+    NEXT_PRIME_VIADIFF_CHECK(pp, pt);
     if (! smodis(bad,pp)) continue;
     affui(pp, q);
     r = rootmod(x, q); if (lg(r) > 1) break;
@@ -238,6 +241,21 @@ nfroots(GEN nf,GEN pol)
   return gerepileupto(av, gen_sort(A, 0, cmp_pol));
 }
 
+int
+nfissplit(GEN nf, GEN x)
+{
+  gpmem_t av = avma;
+  long l = lg(nfsqff(checknf(nf), x, 2));
+  avma = av; return l != 1;
+}
+
+/* nf = K[y] / (P). Galois over K ? */
+int
+nfisgalois(GEN nf, GEN P)
+{
+  return degpol(P) <= 2 || nfissplit(nf, P);
+}
+
 /* return a minimal lift of elt modulo id */
 static GEN
 nf_bestlift(GEN elt, GEN bound, nflift_t *T)
@@ -248,12 +266,12 @@ nf_bestlift(GEN elt, GEN bound, nflift_t *T)
   {
     if (t == t_POL) elt = mulmat_pol(T->tozk, elt);
     u = gmul(T->iprk,elt);
-    for (i=1; i<l; i++) u[i] = (long)diviiround((GEN)u[i], T->pa);
+    for (i=1; i<l; i++) u[i] = (long)diviiround((GEN)u[i], T->pk);
   }
   else
   {
     u = gmul(elt, (GEN)T->iprk[1]);
-    for (i=1; i<l; i++) u[i] = (long)diviiround((GEN)u[i], T->pa);
+    for (i=1; i<l; i++) u[i] = (long)diviiround((GEN)u[i], T->pk);
     elt = gscalcol(elt, l-1);
   }
   u = gsub(elt, gmul(T->prk, u));
@@ -310,6 +328,9 @@ nffactor(GEN nf,GEN pol)
   }
 
   A = fix_relative_pol(nf,pol,0);
+  if (degpol(nf[1]) == 1)
+    return gerepileupto(av, factpol(simplify(pol), 0));
+
   A = primpart( lift_intern(A) );
   if (DEBUGLEVEL>3) fprintferr("test if polynomial is square-free\n");
   g = nfgcd(A, derivpol(A), T, NULL);
@@ -355,40 +376,128 @@ nffactor(GEN nf,GEN pol)
   rep[2] = (long)p1; return sort_factor(rep, cmp_pol);
 }
 
-/* return a bound for T_2(P), P | polbase */
+/* return a bound for T_2(P), P | polbase in C[X]
+ * NB: Mignotte bound: A | S ==>
+ *  |a_i| <= binom(d-1, i-1) || S ||_2 + binom(d-1, i) lc(S)
+ *
+ * Apply to sigma(S) for all embeddings sigma, then take the L_2 norm over
+ * sigma, then take the sup over i.
+ **/
 static GEN
-nf_factor_bound(GEN nf, GEN polbase)
+nf_Mignotte_bound(GEN nf, GEN polbase)
 {
-  GEN lt,C,run,p1,p2, G = gmael(nf,5,2);
-  long i,prec,precnf, d = lgef(polbase), n = degpol(nf[1]);
+  GEN G = gmael(nf,5,2), lS = leading_term(polbase); /* t_INT */
+  GEN p1, C, N2, matGS, binlS, bin;
+  long prec, i, j, d = degpol(polbase), n = degpol(nf[1]), r1 = nf_get_r1(nf);
+
+  if (typ(lS) == t_COL) lS = (GEN)lS[1];
+  binlS = bin = vecbinome(d-1);
+  if (!gcmp1(lS)) binlS = gmul(lS, bin);
+
+  N2 = cgetg(n+1, t_VEC);
+  prec = gprecision(G);
+  for (;;)
+  {
+    nffp_t F;
+
+    matGS = cgetg(d+2, t_MAT);
+    for (j=0; j<=d; j++) matGS[j+1] = lmul(G, (GEN)polbase[j+2]);
+    matGS = gtrans_i(matGS);
+    for (j=1; j <= r1; j++) /* N2[j] = || sigma_j(S) ||_2 */
+    {
+      N2[j] = lsqrt( QuickNormL2((GEN)matGS[j], DEFAULTPREC), DEFAULTPREC );
+      if (lg(N2[j]) < DEFAULTPREC) goto PRECPB;
+    }
+    for (   ; j <= n; j+=2)
+    {
+      GEN q1 = QuickNormL2((GEN)matGS[j  ], DEFAULTPREC);
+      GEN q2 = QuickNormL2((GEN)matGS[j+1], DEFAULTPREC);
+      p1 = gmul2n(mpadd(q1, q2), -1);
+      N2[j] = N2[j+1] = lsqrt( p1, DEFAULTPREC );
+      if (lg(N2[j]) < DEFAULTPREC) goto PRECPB;
+    }
+    if (j > n) break; /* done */
+PRECPB:
+    prec = (prec<<1)-2;
+    remake_GM(nf, &F, prec); G = F.G;
+    if (DEBUGLEVEL>1) err(warnprec, "nf_factor_bound", prec);
+  }
+
+  /* Take sup over 0 <= i <= d of
+   * sum_sigma | binom(d-1, i-1) ||sigma(S)||_2 + binom(d-1,i) lc(S) |^2 */
+
+  /* i = 0: n lc(S)^2 */
+  C = mulsi(n, sqri(lS));
+  /* i = d: sum_sigma ||sigma(S)||_2^2 */
+  p1 = gnorml2(N2); if (gcmp(C, p1) < 0) C = p1;
+  for (i = 1; i < d; i++)
+  {
+    GEN s = gzero;
+    for (j = 1; j <= n; j++)
+    {
+      p1 = mpadd( mpmul((GEN)bin[i], (GEN)N2[j]), (GEN)binlS[i+1] );
+      s = mpadd(s, gsqr(p1));
+    }
+    if (gcmp(C, s) < 0) C = s;
+  }
+  return C;
+}
+
+/* return a bound for T_2(P), P | polbase
+ * max |b_i|^2 <= 3^{3/2 + d} / (4 \pi d) [P]_2,
+ * where [P]_2 is Bombieri's 2-norm
+ * Sum over conjugates
+*/
+static GEN
+nf_Beauzamy_bound(GEN nf, GEN polbase)
+{
+  GEN lt,C,run,s, G = gmael(nf,5,2), POL, bin;
+  long i,prec,precnf, d = degpol(polbase), n = degpol(nf[1]);
 
   precnf = gprecision(G);
   prec   = MEDDEFAULTPREC;
+  bin = vecbinome(d);
+  POL = polbase + 2;
+  /* compute [POL]_2 */
   for (;;)
   {
     run= realun(prec);
-    p1 = realzero(prec);
-    for (i=2; i<d; i++)
+    s = realzero(prec);
+    for (i=0; i<=d; i++)
     {
-      p2 = gmul(run, (GEN)polbase[i]);
-      p2 = gnorml2(gmul(G, p2));
-      if (!signe(p2)) continue;
-      if (lg(p2) == 3) break;
-      p1 = addrr(p1, gdiv(p2, binome(stoi(d), i-2)));
+      GEN p1 = gnorml2(gmul(G, gmul(run, (GEN)POL[i]))); /* T2(POL[i]) */
+      if (!signe(p1)) continue;
+      if (lg(p1) == 3) break;
+      /* s += T2(POL[i]) / binomial(d,i) */
+      s = addrr(s, gdiv(p1, (GEN)bin[i+1]));
     }
-    if (i == d) break;
+    if (i > d) break;
 
     prec = (prec<<1)-2;
     if (prec > precnf)
     {
-      nffp_t F; remake_GM(nf, prec, &F); G = F.G;
+      nffp_t F; remake_GM(nf, &F, prec); G = F.G;
       if (DEBUGLEVEL>1) err(warnprec, "nf_factor_bound", prec);
     }
   }
   lt = (GEN)leading_term(polbase)[1];
-  p1 = gmul(p1, mulis(sqri(lt), n));
-  C = gpow(stoi(3), dbltor(0.75 + d), DEFAULTPREC);
-  return gdiv(gmul(C, p1), gmulsg(d, mppi(DEFAULTPREC)));
+  s = gmul(s, mulis(sqri(lt), n));
+  C = gpow(stoi(3), dbltor(1.5 + d), DEFAULTPREC); /* 3^{3/2 + d} */
+  return gdiv(gmul(C, s), gmulsg(d, mppi(DEFAULTPREC)));
+}
+
+static GEN
+nf_factor_bound(GEN nf, GEN polbase)
+{
+  gpmem_t av = avma;
+  GEN a = nf_Mignotte_bound(nf, polbase);
+  GEN b = nf_Beauzamy_bound(nf, polbase);
+  if (DEBUGLEVEL>2)
+  {
+    fprintferr("Mignotte bound: %Z\n",a);
+    fprintferr("Beauzamy bound: %Z\n",b);
+  }
+  return gerepileupto(av, gmin(a, b));
 }
 
 static long
@@ -519,16 +628,115 @@ normlp(GEN L, long p, long n)
  *    | S vS + P vP |^2 < Btra
  * This implies | S1 vS + P1 vP |^2 < Blow, assuming q > sqrt(Btra).
  * d = dimension of low part (= [nf:Q])
- * BvS = bound for |vS|^2
+ * n0 = bound for |vS|^2
  * */
 static double
-get_Blow(long BvS, long d)
+get_Blow(long n0, long d, GEN q)
 {
-  double sqrtBvS = sqrt((double)BvS);
-  double t = 1. + 0.5*sqrtBvS + 0.5 * sqrt((double)d) * (sqrtBvS + 1);
+#if 0
+  double sqrtn0 = sqrt((double)n0);
+  double t = sqrtn0 + sqrt((double)d) * (sqrtn0 + 1);
+#else
+  double sqrtd = sqrt((double)d);
+  double t, aux;
+  
+  if (gexpo(q)>30)
+    aux = 1.0001;
+  else
+  {
+    aux = gtodouble(q);
+    aux = sqrt(1 + 4/(aux*aux));
+  }
+  t = n0*sqrtd + sqrtd/2 * aux * (sqrtd * (n0+1)); /* assume pr degree 1 */
+#endif
+  t = 1. + 0.5 * t;
   return t * t;
 }
 
+typedef struct {
+  GEN d;
+  GEN dPinvS;   /* d P^(-1) S   [ integral ] */
+  double **PinvSdbl; /* P^(-1) S as double */
+  GEN S1, P1;   /* S = S0 + S1 q, idem P */
+} trace_data;
+
+/* S1 * u - P1 * round(P^-1 S u). K non-zero coords in u given by ind */
+static GEN
+get_trace(GEN ind, trace_data *T)
+{
+  long i, j, l, K = lg(ind)-1;
+  GEN z, s, v;
+
+  s = (GEN)T->S1[ ind[1] ];
+  if (K == 1) return s;
+
+  /* compute s = S1 u */
+  for (j=2; j<=K; j++) s = gadd(s, (GEN)T->S1[ ind[j] ]);
+
+  /* compute v := - round(P^1 S u) */
+  l = lg(s);
+  v = cgetg(l, t_VECSMALL);
+  for (i=1; i<l; i++)
+  {
+    double r, t = 0.;
+    /* quick approximate computation */
+    for (j=1; j<=K; j++) t += T->PinvSdbl[ ind[j] ][i];
+    r = floor(t + 0.5);
+    if (fabs(t + 0.5 - r) < 0.01)
+    { /* dubious, compute exactly */
+      z = gzero;
+      for (j=1; j<=K; j++) z = addii(z, ((GEN**)T->dPinvS)[ ind[j] ][i]);
+      v[i] = - itos( diviiround(z, T->d) );
+    }
+    else
+      v[i] = - (long)r;
+  }
+  return gadd(s, gmul_mati_smallvec(T->P1, v));
+} 
+
+static trace_data *
+init_trace(trace_data *T, GEN S, nflift_t *L, GEN q)
+{
+  long e = gexpo((GEN)S), i,j, l,h;
+  GEN qgood, S1, invd;
+
+  if (e < 0) return NULL; /* S = 0 */
+
+  qgood = shifti(gun, e - 32); /* single precision check */
+  if (cmpii(qgood, q) > 0) q = qgood;
+
+  S1 = gdivround(S, q);
+  if (gcmp0(S1)) return NULL;
+
+  invd = ginv(itor(L->den, DEFAULTPREC));
+
+  T->dPinvS = gmul(L->iprk, S);
+  l = lg(S);
+  h = lg(T->dPinvS[1]);
+  T->PinvSdbl = (double**)cgetg(l, t_MAT);
+  init_dalloc();
+  for (j = 1; j < l; j++)
+  {
+    double *t = dalloc(h * sizeof(double));
+    GEN c = (GEN)T->dPinvS[j];
+    T->PinvSdbl[j] = t;
+    for (i=1; i < h; i++) t[i] = rtodbl(gmul(invd, (GEN)c[i]));
+  }
+
+  T->d  = L->den;
+  T->P1 = gdivround(L->prk, q);
+  T->S1 = S1; return T;
+}
+
+static void
+update_trace(trace_data *T, long k, long i)
+{
+  if (!T) return;
+  T->S1[k]       = T->S1[i];
+  T->dPinvS[k]   = T->dPinvS[i];
+  T->PinvSdbl[k] = T->PinvSdbl[i];
+}
+
 /* Naive recombination of modular factors: combine up to maxK modular
  * factors, degree <= klim and divisible by hint
  *
@@ -541,15 +749,12 @@ nfcmbf(nfcmbf_t *T, GEN p, long a, long maxK, long klim)
 {
   long Sbound;
   GEN pol = T->pol, nf = T->nf, famod = T->fact;
-  GEN dn = T->dn, bound = T->bound;
-  GEN den = T->L->den, deno2 = shifti(den, -1);
+  GEN bound = T->bound;
   GEN nfpol = (GEN)nf[1];
   long K = 1, cnt = 1, i,j,k, curdeg, lfamod = lg(famod)-1, dnf = degpol(nfpol);
-  GEN lc, lcpol, h1 = NULL; /* gcc -Wall */
-  GEN pa = gpowgs(p,a), pas2 = shifti(pa,-1);
+  GEN lc, lcpol;
+  GEN pk = gpowgs(p,a), pas2 = shifti(pk,-1);
 
-  GEN hS1    = cgetg(lfamod+1, t_VEC);
-  GEN hS2    = cgetg(lfamod+1, t_VEC);
   GEN trace1   = cgetg(lfamod+1, t_MAT);
   GEN trace2   = cgetg(lfamod+1, t_MAT);
   GEN ind      = cgetg(lfamod+1, t_VECSMALL);
@@ -558,54 +763,40 @@ nfcmbf(nfcmbf_t *T, GEN p, long a, long maxK, long klim)
   GEN listmod  = cgetg(lfamod+1, t_COL);
   GEN fa       = cgetg(lfamod+1, t_COL);
   GEN res = cgetg(3, t_VEC);
-  const double Blow = get_Blow(lfamod, dnf);
+  GEN q = ceil_safe(mpsqrt(T->BS_2));
+  const double Blow = get_Blow(lfamod, dnf, q);
+  trace_data _T1, _T2, *T1, *T2;
 
   if (maxK < 0) maxK = lfamod-1;
-  (void)dn;
 
   lc = absi(leading_term(pol));
   if (gcmp1(lc)) lc = NULL;
   lcpol = lc? gmul(lc,pol): pol;
 
   {
-    GEN T1,T2, q, goodq, lc2 = lc? sqri(lc): NULL;
-    long e, e1, e2;
+    GEN t1,t2, lc2 = lc? sqri(lc): NULL;
 
-    q = ceil_safe(mpsqrt(T->BS_2));
     for (i=1; i <= lfamod; i++)
     {
       GEN P = (GEN)famod[i];
       long d = degpol(P);
 
       degpol[i] = d; P += 2;
-      T1 = (GEN)P[d-1];/* = - S_1 */
-      T2 = sqri(T1);
-      if (d > 1) T2 = subii(T2, shifti((GEN)P[d-2],1));
-      T2 = modii(T2, pa); /* = S_2 Newton sum */
+      t1 = (GEN)P[d-1];/* = - S_1 */
+      t2 = sqri(t1);
+      if (d > 1) t2 = subii(t2, shifti((GEN)P[d-2],1));
+      t2 = modii(t2, pk); /* = S_2 Newton sum */
       if (lc)
       {
-        T1 = modii(mulii(lc, T1), pa);
-        T2 = modii(mulii(lc2,T2), pa);
+        t1 = modii(mulii(lc, t1), pk);
+        t2 = modii(mulii(lc2,t2), pk);
       }
-      trace1[i] = (long)nf_bestlift(T1, NULL, T->L);
-      trace2[i] = (long)nf_bestlift(T2, NULL, T->L);
-    }
-    e1 = gexpo((GEN)trace1);
-    e2 = gexpo((GEN)trace2); e = max(e1, e2);
-    if (e < 0) /* trace1 = trace2 = 0 */
-      trace1 = trace2 = NULL;
-    else
-    {
-      goodq = shifti(gun, e2 - 32); /* single precision check */
-      if (cmpii(goodq, q) > 0) q = goodq;
-      T1 = trace1;
-      T2 = trace2;
-      trace1 = gdivround(T1, q); if (gcmp0(trace1)) trace1 = NULL;
-      trace2 = gdivround(T2, q); if (gcmp0(trace2)) trace2 = NULL;
-      if (trace1) hS1 = gsub(T1, gmul(T->L->prk, trace1));
-      if (trace2) hS2 = gsub(T2, gmul(T->L->prk, trace2)); /* <= q/2 */
-      h1 = gdivround(T->L->prk, q);
+      trace1[i] = (long)nf_bestlift(t1, NULL, T->L);
+      trace2[i] = (long)nf_bestlift(t2, NULL, T->L);
     }
+
+    T1 = init_trace(&_T1, trace1, T->L, q);
+    T2 = init_trace(&_T2, trace2, T->L, q);
   }
   degsofar[0] = 0; /* sentinel */
 
@@ -627,26 +818,14 @@ nextK:
     }
     if (curdeg <= klim && curdeg % T->hint == 0) /* trial divide */
     {
-      GEN s, t, y, q, list;
+      GEN t, y, q, list;
       gpmem_t av;
 
       av = avma;
       /* d - 1 test */
-      if (trace1)
+      if (T1)
       {
-        s = (GEN)hS1[ind[1]];
-        t = (GEN)trace1[ind[1]];
-        for (i=2; i<=K; i++)
-        {
-          t = gadd(t, (GEN)trace1[ind[i]]);
-          s = gadd(s, (GEN)hS1[ind[i]]);
-          for (j=1; j<=dnf; j++)
-            if (absi_cmp((GEN)s[i], deno2) > 0)
-            {
-              t = gsub(t, (GEN)h1[i]);
-              s[i] = lsubii((GEN)s[i], den);
-            }
-        }
+        t = get_trace(ind, T1);
         if (rtodbl(QuickNormL2(t,DEFAULTPREC)) > Blow)
         {
           if (DEBUGLEVEL>6) fprintferr(".");
@@ -654,21 +833,9 @@ nextK:
         }
       }
       /* d - 2 test */
-      if (trace2)
+      if (T2)
       {
-        s = (GEN)hS2[ind[1]];
-        t = (GEN)trace2[ind[1]];
-        for (i=2; i<=K; i++)
-        {
-          t = gadd(t, (GEN)trace2[ind[i]]);
-          s = gadd(s, (GEN)hS2[ind[i]]);
-          for (j=1; j<=dnf; j++)
-            if (absi_cmp((GEN)s[i], deno2) > 0)
-            {
-              t = gsub(t, (GEN)h1[i]);
-              s[i] = lsubii((GEN)s[i], den);
-            }
-        }
+        t = get_trace(ind, T2);
         if (rtodbl(QuickNormL2(t,DEFAULTPREC)) > Blow)
         {
           if (DEBUGLEVEL>3) fprintferr("|");
@@ -681,7 +848,7 @@ nextK:
       {
         GEN q = (GEN)famod[ind[i]];
         if (y) q = gmul(y, q);
-        y = centermod_i(q, pa, pas2);
+        y = centermod_i(q, pk, pas2);
       }
       y = nf_pol_lift(y, bound, T);
       if (!y)
@@ -713,8 +880,8 @@ nextK:
         else
         {
           famod[k] = famod[i];
-          if (trace1) { trace1[k] = trace1[i]; hS1[k] = hS2[i]; }
-          if (trace2) { trace2[k] = trace2[i]; hS2[k] = hS2[i]; }
+          update_trace(T1, k, i);
+          update_trace(T2, k, i);
           degpol[k] = degpol[i]; k++;
         }
       }
@@ -759,7 +926,7 @@ END:
 }
 
 static GEN
-nf_check_factors(nfcmbf_t *T, GEN P, GEN M_L, GEN famod, GEN pa)
+nf_check_factors(nfcmbf_t *T, GEN P, GEN M_L, GEN famod, GEN pk)
 {
   GEN nf = T->nf, bound = T->bound;
   GEN nfT = (GEN)nf[1];
@@ -770,7 +937,7 @@ nf_check_factors(nfcmbf_t *T, GEN P, GEN M_L, GEN famod, GEN pa)
   if (!piv) return NULL;
   if (DEBUGLEVEL>3) fprintferr("special_pivot output:\n%Z\n",piv);
 
-  pas2 = shifti(pa,-1);
+  pas2 = shifti(pk,-1);
   r  = lg(piv)-1;
   n0 = lg(piv[1])-1;
   list = cgetg(r+1, t_COL);
@@ -788,7 +955,7 @@ nf_check_factors(nfcmbf_t *T, GEN P, GEN M_L, GEN famod, GEN pa)
       {
         GEN q = (GEN)famod[j];
         if (y) q = gmul(y, q);
-        y = centermod_i(q, pa, pas2);
+        y = centermod_i(q, pk, pas2);
       }
     y = nf_pol_lift(y, bound, T);
     if (!y) return NULL;
@@ -811,84 +978,83 @@ nf_check_factors(nfcmbf_t *T, GEN P, GEN M_L, GEN famod, GEN pa)
 }
 
 static GEN
-nf_to_Zp(GEN x, GEN pa, GEN pas2, GEN ffproj)
+nf_to_Zp(GEN x, GEN pk, GEN pas2, GEN ffproj)
 {
-  return centermodii(gmul(ffproj, x), pa, pas2);
+  return centermodii(gmul(ffproj, x), pk, pas2);
 }
 
 /* assume lc(pol) != 0 mod prh. Reduce P to Zp[X] mod p^a */
 static GEN
-ZpX(GEN P, GEN pa, GEN ffproj)
+ZpX(GEN P, GEN pk, GEN ffproj)
 {
   long i, l;
-  GEN z, pas2 = shifti(pa,-1);
+  GEN z, pas2 = shifti(pk,-1);
 
-  if (typ(P)!=t_POL) return nf_to_Zp(P,pa,pas2,ffproj);
+  if (typ(P)!=t_POL) return nf_to_Zp(P,pk,pas2,ffproj);
   l = lgef(P);
   z = cgetg(l,t_POL); z[1] = P[1];
-  for (i=2; i<l; i++) z[i] = (long)nf_to_Zp((GEN)P[i],pa,pas2,ffproj);
+  for (i=2; i<l; i++) z[i] = (long)nf_to_Zp((GEN)P[i],pk,pas2,ffproj);
   return normalizepol(z);
 }
 
 /* We want to be able to reconstruct x, |x|^2 < C, from x mod pr^k
  * We want B := min B_i >= C. Let alpha the LLL parameter,
- * y = 1/(alpha-1/4) > 4/3, the theoretical guaranteed bound following from
- *   (Npr)^2k = det(L)^2 <= B^n * y^(n*(n-1))/2
- * is
- * (FIXME??? (n / 2log(Npr)) [ log(C/n) + n*(n-1) * log(4)/4 ] )
+ * y = 1/(alpha-1/4) > 4/3, the theoretical guaranteed bound follows from
+ *    (Npr)^(2k/n) = det(L)^(2/n) <= 1/n sum B_i
+ *                                <= 1/n B sum y^i
+ *                                <= 1/n B y^(n-1) / (y-1)
  *
- *    2k log Npr > n log(C) + n(n-1)/2 * log(y) */
+ *  Hence log(B/n) >= 2k/n log(Npr) - (n-1)log(y) + log(y-1)
+ *                 >= log(C/n), provided
+ *   k >= [ log(C/n) + (n-1)log(y) - log(y-1) ] * n/(2log(Npr))
+ */
 static double
 bestlift_bound(GEN C, long n, double alpha, GEN Npr)
 {
+  const double y = 1 / (alpha - 0.25); /* = 2 if alpha = 3/4 */
   double t;
   if (typ(C) != t_REAL) C = gmul(C, realun(DEFAULTPREC));
   setlg(C, DEFAULTPREC);
-#if 0 /* FIXME (KB). I don't understand the old bound ! (which is worse) */
-  /* cheat: log(4)/4=0.347 > 0.15 */
-  t = rtodbl(mplog(divrs(C,n))) + n*(n-1) * 0.15;
-#else
-  t = rtodbl(mplog(C)) + (n-1) * log( 1/(alpha - 0.25) );
-  return (t * n) / (2.* log(gtodouble(Npr)));
-#endif
+  t = rtodbl(mplog(divrs(C,n))) + (n-1) * log(y) - log(y-1);
+  return ceil((t * n) / (2.* log(gtodouble(Npr))));
 }
 
 static void
 bestlift_init(long a, GEN nf, GEN pr, GEN C, nflift_t *T)
 {
-  const int y = 4;
-  const double alpha = ((double)y-1) / y; /* LLL parameter */
+  const int D = 4;
+  const double alpha = ((double)D-1) / D; /* LLL parameter */
   gpmem_t av = avma;
-  GEN prk, PRK, B, GSmin, pa;
+  GEN prk, PRK, B, GSmin, pk;
 
-  if (!a)  a = (long) bestlift_bound(C, degpol(nf[1]), alpha, idealnorm(nf,pr));
+  if (!a) a = (long)bestlift_bound(C, degpol(nf[1]), alpha, idealnorm(nf,pr));
 
   for (;; avma = av, a<<=1)
   {
     if (DEBUGLEVEL>2) fprintferr("exponent: %ld\n",a);
     prk = idealpows(nf, pr, a);
-    pa = gcoeff(prk,1,1);
-    PRK = hnfmodid(prk, pa);
+    pk = gcoeff(prk,1,1);
+    PRK = hnfmodid(prk, pk);
 
-    PRK = lllint_i(PRK, y, 0, NULL, NULL, &B);
+    PRK = lllint_i(PRK, D, 0, NULL, NULL, &B);
     GSmin = vecmin(GS_norms(B, DEFAULTPREC));
     if (gcmp(GSmin, C) >= 0) break;
   }
   if (DEBUGLEVEL>2) fprintferr("for this exponent, GSmin = %Z\n",GSmin);
-  T->a = a;
-  T->pa = T->den = pa;
+  T->k = a;
+  T->pk = T->den = pk;
   T->prk = PRK;
-  T->iprk = ZM_inv(PRK, pa);
+  T->iprk = ZM_inv(PRK, pk);
   T->GSmin= GSmin;
   T->ZpProj = dim1proj(prk); /* nf --> Zp */
 }
 
 /* assume pr has degree 1 */
 static GEN
-nf_LLL_cmbf(nfcmbf_t *T, GEN p, long a, long rec)
+nf_LLL_cmbf(nfcmbf_t *T, GEN p, long k, long rec)
 {
   nflift_t *L = T->L;
-  GEN pa = L->pa, PRK = L->prk, PRKinv = L->iprk, GSmin = L->GSmin;
+  GEN pk = L->pk, PRK = L->prk, PRKinv = L->iprk, GSmin = L->GSmin;
 
   GEN famod = T->fact, nf = T->nf, ZC = T->ZC, Br = T->Br;
   GEN Pbase = T->polbase, P = T->pol, dn = T->dn;
@@ -896,13 +1062,13 @@ nf_LLL_cmbf(nfcmbf_t *T, GEN p, long a, long rec)
   GEN Btra;
   long dnf = degpol(nfT), dP = degpol(P);
 
-  long BitPerFactor = 3; /* nb bits / modular factor */
+  double BitPerFactor = 0.5; /* nb bits / modular factor */
   long i, C, tmax, n0;
   GEN lP, Bnorm, Tra, T2, TT, CM_L, m, list, ZERO;
   double Blow;
   gpmem_t av, av2, lim;
   long ti_LLL = 0, ti_CF = 0;
-  pari_timer ti;
+  pari_timer ti, ti2, TI;
 
   if (DEBUGLEVEL>2) (void)TIMER(&ti);
 
@@ -910,10 +1076,11 @@ nf_LLL_cmbf(nfcmbf_t *T, GEN p, long a, long rec)
   if (is_pm1(lP)) lP = NULL;
 
   n0 = lg(famod) - 1;
- /* Lattice: (S PRK), small vector (vS vP). To find a bound for the image,
-  * write S = S1 q + S0, P = P1 q + P0 
-  * q(S1 vS + P1 vP) <= Blow for all (vS,vP) assoc. to true factors */
-  Blow = get_Blow(n0, dnf);
+ /* Lattice: (S PRK), small vector (vS vP). To find k bound for the image,
+  * write S = S1 q + S0, P = P1 q + P0
+  * |S1 vS + P1 vP|^2 <= Blow for all (vS,vP) assoc. to true factors */
+  Btra = mulrr(ZC, mulsr(dP*dP, normlp(Br, 2, dnf)));
+  Blow = get_Blow(n0, dnf, gceil(Btra));
   C = (long)ceil(sqrt(Blow/n0)) + 1; /* C^2 n0 ~ Blow */
   Bnorm = dbltor( n0 * C * C + Blow );
   ZERO = zeromat(n0, dnf);
@@ -926,15 +1093,16 @@ nf_LLL_cmbf(nfcmbf_t *T, GEN p, long a, long rec)
   /* tmax = current number of traces used (and computed so far) */
   for(tmax = 0;; tmax++)
   {
-    long tnew = tmax + 1, r = lg(CM_L)-1;
-    GEN M_L, q, qmin, goodq;
-
-    if (DEBUGLEVEL>2)
-      fprintferr("nf_LLL_cmbf: %ld potential factors (tmax = %ld)\n", r, tmax);
+    long a, b, bmin, bgood, delta, tnew = tmax + 1, r = lg(CM_L)-1;
+    GEN oldCM_L, M_L, q, S1, P1, VV;
+    int first = 1;
 
     /* bound for f . S_k(genuine factor) = ZC * bound for T_2(S_tnew) */
     Btra = mulrr(ZC, mulsr(dP*dP, normlp(Br, 2*tnew, dnf)));
-    qmin = ceil_safe(mpsqrt(Btra));
+    bmin = logint(ceil_safe(mpsqrt(Btra)), gdeux, NULL);
+    if (DEBUGLEVEL>2)
+      fprintferr("\nLLL_cmbf: %ld potential factors (tmax = %ld, bmin = %ld)\n",
+                 r, tmax, bmin);
 
     /* compute Newton sums (possibly relifting first) */
     if (gcmp(GSmin, Btra) < 0)
@@ -942,21 +1110,21 @@ nf_LLL_cmbf(nfcmbf_t *T, GEN p, long a, long rec)
       nflift_t L1;
       GEN polred;
 
-      bestlift_init(a<<1, nf, T->pr, Btra, &L1);
-      a      = L1.a;
-      pa     = L1.pa;
+      bestlift_init(k<<1, nf, T->pr, Btra, &L1);
+      k      = L1.k;
+      pk     = L1.pk;
       PRK    = L1.prk;
       PRKinv = L1.iprk;
       GSmin  = L1.GSmin;
 
-      polred = ZpX(Pbase, pa, L1.ZpProj);
-      famod = hensel_lift_fact(polred,famod,NULL,p,pa,a);
+      polred = ZpX(Pbase, pk, L1.ZpProj);
+      famod = hensel_lift_fact(polred,famod,NULL,p,pk,k);
       for (i=1; i<=n0; i++) TT[i] = 0;
     }
     for (i=1; i<=n0; i++)
     {
       GEN p1, lPpow;
-      GEN p2 = polsym_gen((GEN)famod[i], (GEN)TT[i], tnew, NULL, pa);
+      GEN p2 = polsym_gen((GEN)famod[i], (GEN)TT[i], tnew, NULL, pk);
       TT[i] = (long)p2;
       p1 = (GEN)p2[tnew+1];
       /* make Newton sums integral */
@@ -966,52 +1134,82 @@ nf_LLL_cmbf(nfcmbf_t *T, GEN p, long a, long rec)
         p1 = mulii(p1, lPpow); /* assume pr has degree 1 */
       }
       if (dn) p1 = mulii(p1,dn);
-      if (dn || lP) p1 = modii(p1, pa);
-      Tra[i] = (long)gscalcol(p1, dnf); /* S_tnew(famod) */
+      if (dn || lP) p1 = modii(p1, pk);
+      Tra[i] = (long)nf_bestlift(p1, NULL, L); /* S_tnew(famod) */
     }
 
     /* compute truncation parameter */
+    if (DEBUGLEVEL>2) { TIMERstart(&ti2); TIMERstart(&TI); }
+    oldCM_L = CM_L;
     av2 = avma;
-    M_L = (C == 1)? CM_L : gdivexact(CM_L, stoi(C));
-    T2 = centermod( gmul(Tra, M_L), pa );
-    T2 = gsub(T2, gmul(PRK, gdivround(gmul(PRKinv, T2), pa)));
+    b = delta = 0; /* -Wall */
+AGAIN:
+    M_L = gdivexact(CM_L, stoi(C));
+    T2 = gmul(Tra, M_L);
+    VV = gdivround(gmul(PRKinv, T2), pk);
+    T2 = gsub(T2, gmul(PRK, VV));
 
-    goodq = shifti(gun, gexpo(T2) - BitPerFactor * r);
-    q = gmax(qmin, goodq);
-
-    /* build and reduce lattice */
-    m = concatsp( vconcat( CM_L, gdivround(T2, q) ),
-                  vconcat( ZERO, gdivround(PRK,q) ) );
-    /*     [ C M_L   0  ]
-     * m = [            ]   square matrix
-     *     [  T2'   PRK ]   T2' = Tra * M_L  truncated
-     */
-    i = LLL_check_progress(Bnorm, n0, m, &CM_L, &BitPerFactor,
-                           /*dbg:*/ &ti, &ti_LLL);
-    if (i == 1) { list = _col(P); break; }
-    if (i > r) { avma = av2; continue; } /* no progress */
-
-    if (rec && i*rec >= n0)
-    {
-      CM_L = gerepilecopy(av2, CM_L);
-      if (low_stack(lim, stack_lim(av,1)))
-      {
-        if(DEBUGMEM>1) err(warnmem,"nf_LLL_cmbf");
-        gerepileall(av, 8, &CM_L,&TT,&Tra,&famod,&pa,&GSmin,&PRK,&PRKinv);
-      }
-      continue;
+    if (first)
+    { /* initialize lattice, using few p-adic digits for traces */
+      a = gexpo(T2);
+      bgood = (long)(a - max(32, BitPerFactor * r));
+      b = max(bmin, bgood);
+      delta = a - b;
+    }
+    else
+    { /* add more p-adic digits and continue reduction */
+      long b0 = gexpo(T2);
+      if (b0 < b) b = b0;
+      b = max(b-delta, bmin);
+      if (b - delta/2 < bmin) b = bmin; /* near there. Go all the way */
     }
 
-    if (DEBUGLEVEL>2) (void)TIMER(&ti);
-    list = nf_check_factors(T, P, M_L, famod, pa);
-    if (DEBUGLEVEL>2) ti_CF += TIMER(&ti);
-    if (list) break;
+    /* restart with truncated entries */
+    q = shifti(gun, b);
+    P1 = gdivround(PRK, q);
+    S1 = gdivround(Tra, q);
+    T2 = gsub(gmul(S1, M_L), gmul(P1, VV));
+    m = vconcat( CM_L, T2 );
+    if (first)
+    {
+      first = 0;
+      m = concatsp( m, vconcat(ZERO, P1) );
+      /*     [ C M_L   0  ]
+       * m = [            ]   square matrix
+       *     [  T2'   PRK ]   T2' = Tra * M_L  truncated
+       */
+    }
+    CM_L = LLL_check_progress(Bnorm, n0, m, b == bmin, /*dbg:*/ &ti, &ti_LLL);
+    if (DEBUGLEVEL>2)
+      fprintferr("LLL_cmbf: b =%4ld; r =%3ld -->%3ld, time = %ld\n",
+                 b, lg(m)-1, CM_L? lg(CM_L)-1: 1, TIMER(&TI));
+    if (!CM_L) { list = _col(QXQ_normalize(P,nfT)); break; }
+    if (b > bmin)
+    {
+      CM_L = gerepilecopy(av2, CM_L);
+      goto AGAIN;
+    }
+    if (DEBUGLEVEL>2) msgTIMER(&ti2, "for this trace");
 
-    CM_L = gerepilecopy(av2, CM_L);
+    i = lg(CM_L) - 1;
+    if (i == r && gegal(CM_L, oldCM_L))
+    {
+      CM_L = oldCM_L;
+      avma = av2; continue;
+    }
+
+    if (i <= r && i*rec < n0)
+    {
+      if (DEBUGLEVEL>2) (void)TIMER(&ti);
+      list = nf_check_factors(T, P, M_L, famod, pk);
+      if (DEBUGLEVEL>2) ti_CF += TIMER(&ti);
+      if (list) break;
+      CM_L = gerepilecopy(av2, CM_L);
+    }
     if (low_stack(lim, stack_lim(av,1)))
     {
       if(DEBUGMEM>1) err(warnmem,"nf_LLL_cmbf");
-      gerepileall(av, 8, &CM_L,&TT,&Tra,&famod,&pa,&GSmin,&PRK,&PRKinv);
+      gerepileall(av, 8, &CM_L,&TT,&Tra,&famod,&pk,&GSmin,&PRK,&PRKinv);
     }
   }
   if (DEBUGLEVEL>2)
@@ -1025,7 +1223,7 @@ nf_combine_factors(nfcmbf_t *T, GEN polred, GEN p, long a, long klim)
   GEN z, res, L, listmod, famod = T->fact, nf = T->nf;
   long i, m, l, maxK = 3, nft = lg(famod)-1;
 
-  T->fact = hensel_lift_fact(polred,famod,NULL,p,T->pa,a);
+  T->fact = hensel_lift_fact(polred,famod,NULL,p,T->pk,a);
   if (nft < 11) maxK = -1; /* few modular factors: try all posibilities */
   if (DEBUGLEVEL>3) msgtimer("Hensel lift");
 
@@ -1039,6 +1237,7 @@ nf_combine_factors(nfcmbf_t *T, GEN polred, GEN p, long a, long klim)
   famod = (GEN)listmod[l];
   if (maxK >= 0 && lg(famod)-1 > 2*maxK)
   {
+    if (l > 1) T->polbase = unifpol(nf, (GEN)res[l], 0);
     L = nf_LLL_cmbf(T, p, a, maxK);
     /* remove last elt, possibly unfactored. Add all new ones. */
     setlg(res, l); res = concatsp(res, L);
@@ -1052,29 +1251,36 @@ nf_combine_factors(nfcmbf_t *T, GEN polred, GEN p, long a, long klim)
 
 /* return the factorization of the square-free polynomial x.
    The coeff of x are in Z_nf and its leading term is a rational integer.
-   deg(x) > 1
-   If fl = 1,return only the roots of x in nf */
+   deg(x) > 1, deg(nfpol) > 1
+   If fl = 1, return only the roots of x in nf
+   If fl = 2, as fl=1 if pol splits, [] otherwise */
 static GEN
 nfsqff(GEN nf, GEN pol, long fl)
 {
-  long i, a, m, n, nbf, ct, dpol = degpol(pol);
+  long i, m, n, nbf, ct, dpol = degpol(pol);
   gpmem_t av = avma;
-  GEN pr, C0, C, dk, bad, polbase, pa;
+  GEN pr, C0, C, dk, bad, polbase, pk;
   GEN N2, rep, p, ap, polmod, polred, lt, nfpol;
   byteptr pt=diffptr;
   nfcmbf_t T;
   nflift_t L;
 
   if (DEBUGLEVEL>3) msgtimer("square-free");
-  nfpol = (GEN)nf[1];
+  nfpol = (GEN)nf[1]; n = degpol(nfpol);
   polbase = unifpol(nf,pol,0);
   polmod  = unifpol(nf,pol,1);
+  /* heuristic */
+  if (dpol*3 < n) 
+  {
+    if (DEBUGLEVEL>2) fprintferr("Using Trager's method\n");
+    return gerepilecopy(av, (GEN)polfnf(polmod, nfpol)[1]);
+  }
+
   pol = simplify_i(lift(polmod));
   lt  = (GEN)leading_term(polbase)[1]; /* t_INT */
 
   dk = absi((GEN)nf[3]);
   bad = mulii(mulii(dk,(GEN)nf[4]), lt);
-  n = degpol(nfpol);
 
   p = polred = pr = NULL; /* gcc -Wall */
   nbf = 0; ap = NULL;
@@ -1094,9 +1300,11 @@ nfsqff(GEN nf, GEN pol, long fl)
       if (DEBUGLEVEL>3)
         fprintferr("%3ld %s at prime ideal above %Z\n",
                    nbf, fl?"roots": "factors", p);
+      if (fl == 2 && nbf < dpol) return cgetg(1,t_VEC);
       if (nbf <= 1)
       {
-        if (!fl) return gerepilecopy(av, _vec(polmod)); /* irreducible */
+        if (!fl) /* irreducible */
+          return gerepilecopy(av, _vec(QXQ_normalize(polmod, nfpol)));
         if (!nbf) return cgetg(1,t_VEC); /* no root */
       }
     }
@@ -1120,7 +1328,7 @@ nfsqff(GEN nf, GEN pol, long fl)
   N2 = mulsr(dpol*dpol, normlp(T.Br, 4, n)); /* bound for T_2(lt * S_2) */
   T.BS_2 = mulrr(T.ZC, N2); /* bound for |S_2|^2 on chosen Z-basis */
 
-  if (DEBUGLEVEL>3) {
+  if (DEBUGLEVEL>2) {
     msgtimer("bound computation");
     fprintferr("  1) T_2 bound for %s: %Z\n", fl?"root":"factor", C0);
     fprintferr("  2) Conversion from T_2 --> | |^2 bound : %Z\n", T.ZC);
@@ -1128,29 +1336,31 @@ nfsqff(GEN nf, GEN pol, long fl)
   }
 
   if (fl)
-    (void)logint(C, p, &pa);
+    (void)logint(C, p, &pk);
+#if 0 /* overkill */
   else
   { /* overlift needed for d-1/d-2 tests? */
     GEN pb; long b; /* junk */
-    if (cmbf_precs(p, C, T.BS_2, &a, &b, &pa, &pb))
+    if (cmbf_precs(p, C, T.BS_2, &a, &b, &pk, &pb))
     { /* Rare */
       if (DEBUGLEVEL) err(warner,"nffactor: overlift for d-1/d-2 test");
-      C = itor(pa, DEFAULTPREC);
+      C = itor(pk, DEFAULTPREC);
     }
   }
+#endif
 
   bestlift_init(0, nf, pr, C, &L);
   T.pr = pr;
   T.L  = &L;
 
   /* polred is monic */
-  pa = L.pa;
-  polred = ZpX(gmul(mpinvmod(lt,pa), polbase), pa, L.ZpProj);
+  pk = L.pk;
+  polred = ZpX(gmul(mpinvmod(lt,pk), polbase), pk, L.ZpProj);
 
   if (fl)
   { /* roots only */
     long x_r[] = { evaltyp(t_POL)|_evallg(4), 0,0,0 };
-    rep = rootpadicfast(polred, p, L.a);
+    rep = rootpadicfast(polred, p, L.k);
     x_r[1] = evalsigne(1) | evalvarn(varn(pol)) | evallgef(4);
     x_r[3] = un;
     for (m=1,i=1; i<lg(rep); i++)
@@ -1162,6 +1372,7 @@ nfsqff(GEN nf, GEN pol, long fl)
       x_r[2] = lneg(r); /* check P(r) == 0 */
       q = RXQX_divrem(pol, x_r, nfpol, ONLY_DIVIDES);
       if (q) { pol = q; rep[m++] = (long)r; }
+      else if (fl == 2) return cgetg(1, t_VEC);
     }
     rep[0] = evaltyp(t_VEC) | evallg(m);
     return gerepilecopy(av, rep);
@@ -1173,7 +1384,7 @@ nfsqff(GEN nf, GEN pol, long fl)
   T.fact  = (GEN)factmod0(polred,p)[1];
   T.hint  = 1; /* useless */
 
-  rep = nf_combine_factors(&T, polred, p, L.a, dpol-1);
+  rep = nf_combine_factors(&T, polred, p, L.k, dpol-1);
   return gerepileupto(av, rep);
 }
 
