@@ -1,4 +1,4 @@
-/* $Id: stark.c,v 1.97 2002/06/08 14:37:45 karim Exp $
+/* $Id: stark.c,v 1.104 2002/09/11 13:23:20 cvsadmin Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -31,7 +31,7 @@ extern GEN bnrGetSurj(GEN bnr1, GEN bnr2);
 typedef struct {
   GEN L0, L1, L11, L2; /* VECSMALL of p */
   GEN *L1ray, *L11ray; /* precomputed isprincipalray(pr), pr | p */
-  GEN *rayZ; /* recomputed isprincipalray(i), i < condZ */
+  GEN *rayZ; /* precomputed isprincipalray(i), i < condZ */
   long condZ; /* generates cond(bnr) \cap Z, assumed small */
 } LISTray;
 
@@ -771,6 +771,8 @@ ComputeArtinNumber(GEN bnr, GEN LCHI, long check, long prec)
 
   for (i = 1; i <= nz; i++)
   {
+    if (is_bigint(cyc[i]))
+      err(talker,"conductor too large in ComputeArtinNumber");
     gen[i] = set_sign_mod_idele(nf, NULL,gen[i], cond,sarch);
     classe = isprincipalray(bnr, gen[i]);
     for (ic = 1; ic <= nChar; ic++)
@@ -1481,32 +1483,25 @@ ComputeCoeff(GEN dtcr, LISTray *R, long n, long deg)
 /********************************************************************/
 /*              5th part: compute L-functions at s=1                */
 /********************************************************************/
-static void
-_append(GEN L, GEN z)
-{
-  long l = lg(L);
-  L[l] = (long)z; setlg(L,l+1);
-}
-
 static void
 deg11(LISTray *R, long p, GEN bnr, GEN pr) {
   GEN z = isprincipalray(bnr, pr); 
-  _append(R->L1, (GEN)p);
-  _append((GEN)R->L1ray, z);
+  appendL(R->L1, (GEN)p);
+  appendL((GEN)R->L1ray, z);
 }
 static void
 deg12(LISTray *R, long p, GEN bnr, GEN Lpr) {
   GEN z = isprincipalray(bnr, (GEN)Lpr[1]); 
-  _append(R->L11, (GEN)p);
-  _append((GEN)R->L11ray, z);
+  appendL(R->L11, (GEN)p);
+  appendL((GEN)R->L11ray, z);
 }
 static void
 deg0(LISTray *R, long p) {
-  _append(R->L0, (GEN)p);
+  appendL(R->L0, (GEN)p);
 }
 static void
 deg2(LISTray *R, long p) {
-  _append(R->L2, (GEN)p);
+  appendL(R->L2, (GEN)p);
 }
 
 /* pi(x) <= ?? */
@@ -1517,14 +1512,6 @@ PiBound(long x)
   return 1 + (long) (x/lx * (1 + 3/(2*lx)));
 }
 
-static GEN
-_alloc(long n, long t)
-{
-  GEN z = new_chunk(n);
-  z[0] = evaltyp(t) | evallg(1);
-  return z;
-}
-
 static void
 InitPrimesQuad(GEN bnr, long nmax, LISTray *R)
 {
@@ -1536,12 +1523,12 @@ InitPrimesQuad(GEN bnr, long nmax, LISTray *R)
   GEN *gptr[7];
 
   l = 1 + PiBound(nmax);
-  R->L0  = _alloc(l, t_VECSMALL);
-  R->L2  = _alloc(l, t_VECSMALL); R->condZ = condZ;
-  R->L1 = _alloc(l, t_VECSMALL); R->L1ray = (GEN*)_alloc(l, t_VEC);
-  R->L11 = _alloc(l, t_VECSMALL); R->L11ray = (GEN*)_alloc(l, t_VEC);
+  R->L0  = cget1(l, t_VECSMALL);
+  R->L2  = cget1(l, t_VECSMALL); R->condZ = condZ;
+  R->L1 = cget1(l, t_VECSMALL); R->L1ray = (GEN*)cget1(l, t_VEC);
+  R->L11 = cget1(l, t_VECSMALL); R->L11ray = (GEN*)cget1(l, t_VEC);
   prime = stoi(2);
-  for (p = 2; p <= nmax; p += *d++, prime[2] = p)
+  for (p = 2; p <= nmax; prime[2] = p) {
     switch (krogs(dk, p))
     {
     case -1: /* inert */
@@ -1566,6 +1553,8 @@ InitPrimesQuad(GEN bnr, long nmax, LISTray *R)
       }
       break;
     }
+    NEXT_PRIME_VIADIFF(p,d);
+  }  
   /* precompute isprincipalray(x), x in Z */
   R->rayZ = (GEN*)cgetg(condZ, t_VEC);
   for (i=1; i<condZ; i++)
@@ -1589,10 +1578,10 @@ InitPrimes(GEN bnr, long nmax, LISTray *R)
   GEN *gptr[7];
 
   R->condZ = condZ;
-  R->L1 = _alloc(nmax, t_VECSMALL);
-  R->L1ray = (GEN*)_alloc(nmax, t_VEC);
+  R->L1 = cget1(nmax, t_VECSMALL);
+  R->L1ray = (GEN*)cget1(nmax, t_VEC);
   prime = stoi(2);
-  for (p = 2; p <= nmax; p += *d++, prime[2] = p)
+  for (p = 2; p <= nmax; prime[2] = p)
   {
     tabpr = primedec(nf, prime);
     for (j = 1; j < lg(tabpr); j++)
@@ -1602,10 +1591,10 @@ InitPrimes(GEN bnr, long nmax, LISTray *R)
       if (is_bigint(Npr) || (np = Npr[2]) > nmax) continue;
       if (condZ % p == 0 && idealval(nf, cond, pr)) continue;
 
-      _append(R->L1, (GEN)np);
-      _append((GEN)R->L1ray, isprincipalray(bnr, pr));
+      appendL(R->L1, (GEN)np);
+      appendL((GEN)R->L1ray, isprincipalray(bnr, pr));
     }
-
+    NEXT_PRIME_VIADIFF(p,d);
   }
   gptr[0] = &(R->L1);  gptr[1] = (GEN*)&(R->L1ray);
   gerepilemany(av,gptr,2);
@@ -2204,7 +2193,7 @@ RecCoeff3(GEN nf, RC_data *d, long prec)
   eps = gpowgs(stoi(10), min(-8, (d->G >> 1)));
   tB  = gpow(gmul2n(eps, N), gdivgs(gun, 1-N), DEFAULTPREC);
 
-  Bd    = gmin(B, tB);
+  Bd    = gceil(gmin(B, tB));
   p1    = gceil(gdiv(glog(Bd, DEFAULTPREC), dbltor(2.3026)));
   prec2 = max((prec << 1) - 2, (long)(itos(p1) * pariK1 + BIGDEFAULTPREC));
   nf2   = nfnewprec(nf, prec2);
@@ -2421,8 +2410,12 @@ computean(GEN dtcr, LISTray *R, long n, long deg)
   for (i=1; i<l; i++, avma = av2)
   {
     p = L[i];
-    ray = R->rayZ[p % condZ];
-    chi  = EvalChar(&C, ray);
+    if (condZ == 1) chi = C.val[0]; /* 1 */
+    else
+    {
+      ray = R->rayZ[p % condZ];
+      chi  = EvalChar(&C, ray);
+    }
     chi1 = chi;
     for (q=p;;)
     {
@@ -2457,7 +2450,10 @@ computean(GEN dtcr, LISTray *R, long n, long deg)
     GEN ray1, ray2, chi11, chi12, chi2;
 
     p = L[i]; ray1 = R->L11ray[i]; /* use pr1 pr2 = (p) */
-    ray2 = gsub(R->rayZ[p % condZ],  ray1);
+    if (condZ == 1)
+      ray2 = gneg(ray1);
+    else
+      ray2 = gsub(R->rayZ[p % condZ],  ray1);
     chi11 = EvalChar(&C, ray1);
     chi12 = EvalChar(&C, ray2);
 
@@ -2868,11 +2864,10 @@ LABDOUB:
 GEN
 quadhilbertreal(GEN D, GEN flag, long prec)
 {
-  VOLATILE gpmem_t av = avma;
-  VOLATILE long cl;
+  gpmem_t av = avma;
   long newprec;
+  VOLATILE long cl;
   VOLATILE GEN pol, bnf, bnr, dataC, bnrh, nf, exp;
-  void *catcherr = NULL;
 
   (void)&prec; /* prevent longjmp clobbering it */
   if (DEBUGLEVEL) (void)timer2();
@@ -2891,45 +2886,37 @@ quadhilbertreal(GEN D, GEN flag, long prec)
   pol = quadpoly(D);
   setvarn(pol, fetch_var());
 
- START:
+START:
   /* compute the class group */
   bnf = bnfinit0(pol, 1, NULL, prec);
   nf  = (GEN)bnf[7];
   disable_dbg(-1);
-
   if (DEBUGLEVEL) msgtimer("Compute Cl(k)");
 
   /* if the exponent of the class group is 2, use rather Genus Field Theory */
   exp = gmael4(bnf, 8, 1, 2, 1);
   if (gegal(exp, gdeux)) { (void)delete_var(); return GenusField(bnf, prec); }
 
-  { /* catch precision problems (precision too small) */
-    jmp_buf env;
-    if (setjmp(env))
+  CATCH(precer) {
+    prec += EXTRA_PREC; pol = NULL;
+    err (warnprec, "quadhilbertreal", prec);
+  } TRY {
+    /* find the modulus defining N */
+    bnr   = buchrayinitgen(bnf, gun);
+    dataC = InitQuotient(bnr, gzero);
+    bnrh  = FindModulus(dataC, 1, &newprec, prec, gcmp0(flag)? 0: -10);
+
+    if (DEBUGLEVEL) msgtimer("FindModulus");
+
+    if (newprec > prec)
     {
-      prec += EXTRA_PREC;
-      err (warnprec, "quadhilbertreal", prec);
-      goto START;
+      if (DEBUGLEVEL >= 2) fprintferr("new precision: %ld\n", newprec);
+      nf = nfnewprec(nf, newprec);
     }
-    catcherr = err_catch(precer, env, NULL);
-  }
+    pol = AllStark(bnrh, nf, 1, newprec);
+  } ENDCATCH;
+  if (!pol) goto START;
 
-  /* find the modulus defining N */
-  bnr   = buchrayinitgen(bnf, gun);
-  dataC = InitQuotient(bnr, gzero);
-  bnrh  = FindModulus(dataC, 1, &newprec, prec, gcmp0(flag)? 0: -10);
-
-  if (DEBUGLEVEL) msgtimer("FindModulus");
-
-  if (newprec > prec)
-  {
-    if (DEBUGLEVEL >= 2) fprintferr("new precision: %ld\n", newprec);
-    nf = nfnewprec(nf, newprec);
-  }
-
-  /* use the generic function AllStark */
-  pol = AllStark(bnrh, nf, 1, newprec);
-  err_leave(&catcherr);
   pol = makescind(nf, pol, cl);
   (void)delete_var();
   return gerepileupto(av, pol);
