@@ -1,4 +1,4 @@
-/* $Id: subfield.c,v 1.51 2002/06/09 20:18:18 karim Exp $
+/* $Id: subfield.c,v 1.55 2002/09/07 19:15:13 karim Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -261,7 +261,7 @@ print_block_system(long N,GEN Y,long d,GEN vbs,long maxl)
   if (DEBUGLEVEL>5) fprintferr("Y = %Z\n",Y);
   empty = cgetg(1,t_VEC);
   n = new_chunk(N+1);
-  D = cgetg(N+1, t_VEC); setlg(D,1);
+  D = cget1(N+1, t_VEC);
   t=new_chunk(r+1); k=new_chunk(r+1); Z=cgetg(r+1,t_VEC);
   for (ns=0,i=1; i<r; i++)
   {
@@ -610,25 +610,27 @@ choose_prime(GEN pol,GEN dpol,long d,GEN *ptff,GEN *ptlistpotbl, long *ptlcm)
   minp = N*(m-1)/2;
   if (DEBUGLEVEL) (void)timer2();
   di++; p = stoi(2);
-  while (p[2]<=minp) p[2] += *di++;
+  while (p[2]<=minp)
+      NEXT_PRIME_VIADIFF(p[2], di);
   oldllist = 100000;
   oldlcm = 0;
   oldlistpotbl = oldff = oldn = NULL; pp = 0; /* gcc -Wall */
   av = avma;
-  for(k=1; k<11 || !oldn; k++,p[2]+= *di++,avma = av)
+  for(k=1; k<11 || !oldn; k++,avma = av)
   {
-    while (!smodis(dpol,p[2])) p[2] += *di++;
+    while (!smodis(dpol,p[2]))
+      NEXT_PRIME_VIADIFF(p[2], di);
     if (k > 50) err(talker,"sorry, too many block systems in nfsubfields");
     ff=(GEN)factmod0(pol,p)[1]; r=lg(ff)-1;
-    if (r == 1 || r == N) continue;
+    if (r == 1 || r == N) goto repeat;
 
     n = cgetg(r+1, t_VECSMALL);
     lcm = n[1] = degpol(ff[1]);
     for (j=2; j<=r; j++) { n[j] = degpol(ff[j]); lcm = clcm(lcm,n[j]); }
-    if (lcm < oldlcm) continue; /* false when oldlcm = 0 */
-    if (r >= BIL) { err(warner,"subfields: overflow in calc_block"); continue; }
+    if (lcm < oldlcm) goto repeat; /* false when oldlcm = 0 */
+    if (r >= BIL) { err(warner,"subfields: overflow in calc_block"); goto repeat; }
     if (DEBUGLEVEL) fprintferr("p = %ld,\tlcm = %ld,\torbits: %Z\n",p[2],lcm,n);
-    if (oldn && gegal(n,oldn)) continue;
+    if (oldn && gegal(n,oldn)) goto repeat;
 
     listpotbl = potential_block_systems(N,d,n, oldllist);
     if (!listpotbl) { oldlistpotbl = NULL; pp = p[2]; break; }
@@ -637,12 +639,14 @@ choose_prime(GEN pol,GEN dpol,long d,GEN *ptff,GEN *ptlistpotbl, long *ptlcm)
     {
       if (DEBUGLEVEL) msgtimer("#pbs >= %ld [aborted]",oldllist);
       for (j=1; j<llist; j++) free((void*)listpotbl[j]);
-      free((void*)(listpotbl-1)); continue;
+      free((void*)(listpotbl-1)); goto repeat;
     }
     oldllist = llist; oldlistpotbl = listpotbl;
     pp = p[2]; oldff = ff; oldlcm = lcm; oldn = n;
     if (DEBUGLEVEL) msgtimer("#pbs = %ld",llist);
     av = avma;
+   repeat:
+    NEXT_PRIME_VIADIFF(p[2], di);
   }
   if (DEBUGLEVEL) fprintferr("Chosen prime: p = %ld\n",pp);
   *ptlistpotbl=oldlistpotbl; *ptff=oldff; *ptlcm=oldlcm; return stoi(pp);
@@ -959,15 +963,6 @@ subfields_poldata(GEN T, struct poldata *PD)
   }
 }
 
-static long
-grouporder(GEN G)
-{
-  GEN Go = (GEN)G[2];
-  long i, l = lg(Go), o = 1;
-  for (i=1; i<l; i++) o *= Go[i];
-  return o;
-}
-
 GEN
 subfields(GEN nf,GEN d)
 {
@@ -993,8 +988,8 @@ subfields(GEN nf,GEN d)
     for (i=1; i<l; i++)
     {
       GEN H = (GEN)L[i];
-      if (grouporder(H) == o)
-        F[k++] = (long) galoisfixedfield(G, (GEN)H[1], 0, v0);
+      if (group_order(H) == o)
+        F[k++] = (long) lift_intern(galoisfixedfield(G, (GEN)H[1], 0, v0));
     }
     setlg(F, k);
     return gerepilecopy(av, F);
@@ -1020,7 +1015,7 @@ subfieldsall(GEN nf)
   if (typ(G) != t_INT)
   {
     pol = get_nfpol(nf, &nf);
-    return gerepileupto(av, galoissubfields(G, 0, varn(pol)));
+    return gerepilecopy(av, lift_intern( galoissubfields(G, 0, varn(pol)) ));
   }
 
   subfields_poldata(nf, &PD);
